[TOC]

# sdk 使用说明

## 1、sdk 下载

todo

## 2、目录结构

### 2.1 安装包目录结构

```
|-- lib                         # 动态库存放目录 
	|-- libagentsdk-x64-linux.so
	|-- libagentsdk-x86-linux.so
|-- include                     # 头文件目录  
    |-- c                       # c 头文件
|-- docs                        # 文档目录 
	|-- Api 说明文档-C_Agent.md   # Api接口文档
 	|-- readme-C_Agent.md	       # 使用说明文档
|-- examples		              # 使用示例目录
	|-- c                        # c 使用示例目录
       |-- grpc                  # grpc使用示例
       |--example.c              # c 使用示例
|--install.sh				       # sdk 安装脚本
|--uninstall.sh                  # sdk 卸载脚本
|--version.txt                   # sdk 版本信息
```

### 2.2 安装目标目录结构

```
|--[默认在/opt,如果安装时指定--prefix,则为指定目录]
	|-- bonree 
		|-- apm
			|-- agent
				|-- c
					|-- 版本目录
						|-- lib                          # 动态库存放目录 
							|-- libagentsdk-x64-linux.so
							|-- libagentsdk-x86-linux.so
						|-- include                      # 头文件目录  
							|-- c                        # c 头文件
						|-- docs                         # 文档目录 
							|-- Api 说明文档-C_Agent.md    # Api接口文档
							|-- readme-C_Agent.md	       # 使用说明文档
						|-- examples		               # 使用示例目录
							|-- c                          # c 使用示例目录
								|-- grpc                   # grpc使用示例目录
								|-- example.c              # c 使用示例
								|-- CMakeFile			
								|-- MakeFile
						|--install.sh				     # sdk 安装脚本
						|--uninstall.sh                  # sdk 卸载脚本
						|--version.txt                   # sdk 版本信息
			|-- logs
				|-- c         # c 探针日志存放目录
```

## 3、使用说明

sdk v2.0的部分功能依赖于smartagent, 使用之前先确保smartagent正常安装。 将下载好的sdk安装包解压，执行安装脚本   install.sh即可完成安装

```shell
tar -zxf sdk-v2.0.113797.tar.gz
cd v2.0.113797/
sh install.sh
```

### 3.1、Api 接口说明

- #### [Api文档](Api 说明文档-C_Agent.md)


### 3.2、 接口使用

#### 3.2.1、 sdk 初始化

```c
int main()
{
    //sdk 初始化
    int init = br_init_sdk(0);
    //do ...
    //释放sdk
    
    if (init == 0)//初始化成功
    	br_un_init_sdk();
    return 0;
}
```

#### 3.2.3、业务数据采集

1、采集简单的web 请求数据

```c
    //业务数据采集
    struct transaction_t transaction;
    br_open_transaction(&transaction);
    
    //创建bt对象，web 请求处理前
    struct bt_options_t options;
    struct http_req_data_t http_req;
    options.name.value = "test_bt_name";
    http_req.uri.value = "http://www.bonree.com/test/index";
    struct bt_t bt;
    br_get_or_create_bt(&transaction, &options, &http_req, &bt);
    
	/*
    	web请求处理处理逻辑
    */ 

    //结束bt，web 请求处理完成
    //bt_finish_options 填充信息
    struct bt_finish_options_t bt_finish_options;
    struct http_resp_data_t http_resp;
    bt_finish_options.flags = 0;
    http_resp.status_code.value = "200";
    br_finish_bt(&bt, &bt_finish_options, &http_resp);
    
	//关闭记录
    br_close_transaction(&transaction);
```

2、采集带后端（sql）的web 请求数据

```c
    //业务数据采集
    struct transaction_t transaction;
    br_open_transaction(&transaction);
    
    //创建bt对象，web 请求处理前
    struct bt_options_t options;
    struct http_req_data_t http_req;
    options.name.value = "test_bt_name";
    http_req.uri.value = "http://www.bonree.com/test/index";
    struct bt_t bt;
    br_get_or_create_bt(&transaction, &options,&http_req, &bt);

     //如果处理web 请求过程需要访问数据库（sql/nosql）,记录后端调用记录
     //添加后端调用
     struct be_call_options_t be_options;
     //后端信息
     be_options.be.conn_type.value = "StackExchangeRedis";
     be_options.be.type = BACKEND_TYPE_REDIS;
     be_options.be.name.value = "Redis-192.168.0.201:6379";
     be_options.be.host.value = "192.168.0.201";
     be_options.be.port = 6379;
     //后端调用细节
     be_options.call_detail.detail.value = "Connect 192.168.0.201:6379";
     //后端调用函数信息
     be_options.class_method.method_name.value = "";
     be_options.class_method.class_name.value = "";

     struct be_call_t be_call;
     br_start_be_call(&transaction, &be_options, &be_call);
     //结束becall
     struct be_call_finish_options_t be_finish_options;
     be_finish_options.flags = 0;
     br_finish_be_call(&be_call, &be_finish_options);

    //结束bt
    //bt_finish_options 填充信息
    struct bt_finish_options_t bt_finish_options;
    struct http_resp_data_t http_resp;
    bt_finish_options.flags = 0;
    http_resp.status_code.value = "200";
    http_resp.http_method.value= "Get";
    br_finish_bt(&bt, &bt_finish_options, &http_resp);

    //关闭记录
    br_close_transaction(&transaction);
```

2、采集带rpc 请求的web 请求数据

```c
    //业务数据采集
    struct transaction_t transaction;
    br_open_transaction(&transaction);
    
    //创建bt对象，web 请求处理前
    struct bt_options_t options;
    struct http_req_data_t http_req;
    options.name.value = "test_bt_name";
    http_req.uri.value = "http://www.bonree.com/test/index";
    struct bt_t bt;
    br_get_or_create_bt(&transaction, &options, &http_req, &bt);

    //如果处理web 请求过程需要访问其他rpc 服务

    //添加rpc后端数据
    struct be_call_options_t rpc_be_options;
    memset(&rpc_be_options, 0, sizeof(struct be_call_options_t));
    //...填充be ....
    rpc_be_options.be.conn_type.value = "http_client";
    rpc_be_options.be.type = BACKEND_TYPE_HTTP;
    rpc_be_options.be.name.value = "http://192.168.0.201:8080";
    rpc_be_options.be.host.value = "192.168.0.201";
    rpc_be_options.be.port = 8080;
    rpc_be_options.call_detail.detail.value = "http://192.168.0.201:8080";
    struct be_call_t be_call;
    struct cross_req_head_t cross_req_head;
	//当前请求头中获取cross req
    struct str_t cross_req_str;
     memset(&cross_req_head, 0, sizeof(struct cross_req_head_t));
     memcpy(cross_req_head.data, cross_req_str.value, 100);
     br_process_cross_req_head(&transaction, NULL);
     br_start_rpc(&transaction, &rpc_be_options, &cross_req_head, &be_call);

    struct be_call_finish_options_t be_finish_options;

    br_finish_rpc(&transaction, &be_call, &be_finish_options, NULL);

    //结束bt
    //bt_finish_options 填充信息
    struct bt_finish_options_t bt_finish_options;
    struct http_resp_data_t http_resp;
    bt_finish_options.flags = 0;
    http_resp.status_code.value = "200";
    br_finish_bt(&bt, &bt_finish_options, &http_resp);

    //关闭记录
    br_close_transaction(&transaction);
```

4、采集调用链数据，记录处理web请求中的函数调用

```c
    struct transaction_t transaction;
    br_open_transaction(&transaction);
     //创建bt
     struct bt_options_t options;
     memset(&options, 0, sizeof(struct bt_options_t));
     options.name.value = "test_collect_trace";
     struct bt_t bt;
     br_get_or_create_bt(&transaction, &options, &bt);

     struct tracer_t tracer;
     br_get_or_create_tracer(&transaction, &tracer);

     struct start_span_options_t span_options;
     memset(&span_options, 0, sizeof(struct start_span_options_t));
     span_options.thread_info.thread_name.value = "test_thread";
     span_options.thread_info.thread_id = getpid();
     span_options.method_info.class_name.value = "test";
     span_options.method_info.method_name.value = "collect_trace";
     span_options.method_info.ret_value.value = "void";
     span_options.method_info.params.value = "int,char*";

	 //开始一个函数调用的记录
     struct span_t span;
     start_span(&tracer, &span_options, &span);
	 //向函数记录中添加错误信息（可选）
     struct trace_error_t trace_error;
     trace_error.error_summary.value = "sum";
     trace_error.error_summary.length = 3;
     trace_error.error_details.value = "det";
     trace_error.error_details.length = 3;
     set_trace_error(&span, &trace_error);
     //结束一个函数调用的记录
     finish_span_options_t fin_span_opts;
     memset(&fin_span_opts,0x0,sizeof(fin_span_opts));
     br_finish_span(&span,&fin_span_opts);

     br_finish_be_call(&be_call, &be_finish_options);
     br_finish_tracer(&tracer);
     //结束bt
     //bt_finish_options 填充信息
     struct bt_finish_options_t bt_finish_options;
     memset(&bt_finish_options, 0, sizeof(struct bt_finish_options_t));
     bt_finish_options.flags = 0;
     br_finish_bt(&bt, &bt_finish_options, NULL);
     //关闭记录
     br_close_transaction(&transaction);
```



### 3.3、示例使用

1、示例编译

示例代码为cmake的项目， 可以使用目录下的帮助脚本build.sh 进行编译

```shell
cd  ｛sdk_install_path｝/examples
sh   build.sh
```
另有MakeFile,可直接在对应目录底下执行make生成示例

```shell
cd  ｛sdk_install_path｝/examples
sh   build.sh
```
GCC 编译64位程序示例
```
gcc -o <目标程序名> -I <sdk include目录所在路径> - L<sdk lib库所在目录> -lagentsdk-x64-linux -ldl
```
2、示例运行

在上一步编译成功后 会在examples/target 目录下生成相应的示例程序

```shell
./target/examples/c/c_example #运行示例

```

