
// Created by liuheng@bonree.com on 2019/11/21 0021.
// Copyright © 2007-2019 Bonree Data Technology Co.,Ltd
// All rights reserved

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "sdk.h"

#ifdef __linux
#include <pthread.h>
#include <unistd.h>
#define msleep(msecs) usleep(msecs*1000)
#endif

#ifdef WIN32
#include <windows.h>
#define msleep(msecs) Sleep(msecs)
#endif

static tracer_t g_tracer;
transaction_t g_transaction;


void sayHello(void *strName) {
    span_t span;
    start_span_options_t span_options;
    finish_span_options_t  fin_span_opts;
    memset(&span, 0x0, sizeof(span));
    memset(&span_options, 0x0, sizeof(span_options));
    memset(&fin_span_opts, 0x0, sizeof(fin_span_opts));
	
#ifdef _WIN32
	span_options.thread_info.thread_id = GetCurrentThreadId();
#else
	span_options.thread_info.thread_id = pthread_self();
#endif
    span_options.method_info.method_name.value = "sayHello";
    span_options.method_info.ret_value.value = "void";
    span_options.method_info.params.value = "void *";

    br_start_span(&g_tracer, &span_options, &span);

    char strTmpName[256]="";
    strncpy(strTmpName,(char *)strName, sizeof(strTmpName)-1);
    printf("Hi,I'm %s\n",strTmpName);
    //调用oracle后端，查找用户账户信息
    {
        be_call_options_t be_options;
        memset(&be_options, 0, sizeof(be_options));
        //后端信息
        be_options.be.conn_type.value = "PROC";
        be_options.be.type = BACKEND_TYPE_ORACLE;
        be_options.be.host.value = "192.168.0.202";
        be_options.be.port = 1521;
        be_options.be.db_name.value = "AcctDB";
        //后端调用细节
        str_malloc(&be_options.call_detail.detail, 1024);
        snprintf(be_options.call_detail.detail.value, 1023,"select * from ACCT_T where acct_name=%s",strTmpName);
        //设置标签
        str_t strKey,strVal;
        memset(&strKey,0x0, sizeof(strKey));
        memset(&strVal,0x0, sizeof(strVal));

        strKey.value="db.statement";
        strKey.length=strlen(strKey.value);
        strVal.value=be_options.call_detail.detail.value;
        strVal.length=strlen(strVal.value);

        br_set_tag(&span,&strKey,&strVal);


        be_call_t be_call;
        br_start_be_call(&g_transaction, &be_options, &be_call);
        br_link_to_be_call(&span,  &be_call);
        str_free(&be_options.call_detail.detail);

        be_call_finish_options_t opts_fin;
        memset(&opts_fin,0x0, sizeof(opts_fin));
        br_finish_be_call(&be_call, &opts_fin);
    }
	
    br_finish_span(&span,&fin_span_opts);
}
#ifdef _WIN32
int trace_with_multithread() {
	char *name1 = "Lily", *name2 = "Peter";
	start_span_options_t span_options;
	finish_span_options_t fin_span_opts;
	bt_options_t options;
	bt_t bt;
	span_t span;
	bt_finish_options_t bt_finish_options;

	memset(&span_options, 0x0, sizeof(span_options));
	memset(&fin_span_opts, 0x0, sizeof(fin_span_opts));
	memset(&options, 0x0, sizeof(options));
	memset(&bt_finish_options, 0x0, sizeof(bt_finish_options));


	memset(&g_transaction, 0x0, sizeof(g_transaction));

	if (0 != br_open_transaction(&g_transaction)) {
		return -1;
	}

	memset(&options, 0, sizeof(options));
	options.name.value = "trace_with_multithread";
	br_get_or_create_bt(&g_transaction, &options, NULL, &bt);

	br_get_or_create_tracer(&g_transaction, &g_tracer);

	span_options.thread_info.thread_name.value = "main_thread";
	span_options.thread_info.thread_id = getpid();
	span_options.method_info.method_name.value = "trace_with_multithread";
	span_options.method_info.ret_value.value = "int";
	span_options.method_info.params.value = "void";

	memset(&span, 0x0, sizeof(span));
	br_start_span(&g_tracer, &span_options, &span);

	HANDLE hThread[2];
	hThread[0] = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)sayHello, (void *)name1, 1, 0); //创建子线程

	//线程2
	hThread[1] = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)sayHello, (void *)name2, 1, 0); //创建子线程

	WaitForMultipleObjects(2, hThread, TRUE, INFINITE);
	
	for (int i = 0; i < 2; i++) {
		CloseHandle(hThread[i]);
	}

	br_finish_span(&span, &fin_span_opts);
	br_finish_tracer(&g_tracer);

	memset(&bt_finish_options, 0, sizeof(bt_finish_options));
	bt_finish_options.flags = 0;
	br_finish_bt(&bt, &bt_finish_options, NULL);

	br_close_transaction(&g_transaction);
	return 0;
}
#else
int trace_with_multithread(){

    pthread_t thrd_Lily, thrd_Peter;
    int nRet1=-1, nRet2=-1;
    char *name1 = "Lily", *name2 = "Peter";
    start_span_options_t span_options;
    finish_span_options_t fin_span_opts;
    bt_options_t options;
    bt_t bt;
    span_t span;
    bt_finish_options_t bt_finish_options;

    memset(&span_options, 0x0, sizeof(span_options));
    memset(&fin_span_opts, 0x0, sizeof(fin_span_opts));
    memset(&options, 0x0, sizeof(options));
    memset(&bt_finish_options, 0x0, sizeof(bt_finish_options));


    memset(&g_transaction, 0x0, sizeof(g_transaction));

    if(0 != br_open_transaction(&g_transaction)) {
        return -1;
    }

    memset(&options, 0, sizeof(options));
    options.name.value = "trace_with_multithread";
    br_get_or_create_bt(&g_transaction, &options, NULL,&bt);

    br_get_or_create_tracer(&g_transaction, &g_tracer);

    span_options.thread_info.thread_name.value = "main_thread";
    span_options.thread_info.thread_id = getpid();
    span_options.method_info.method_name.value = "trace_with_multithread";
    span_options.method_info.ret_value.value = "int";
    span_options.method_info.params.value = "void";

    memset(&span, 0x0, sizeof(span));
    br_start_span(&g_tracer, &span_options, &span);

    //线程1
    nRet1 = pthread_create(&thrd_Lily, NULL, sayHello, (void *) name1);
    if(0 != nRet1 ) {
        trace_error_t trace_error;
        trace_error.error_summary.value = "Fail to create Lily thread !";
        trace_error.error_summary.length = strlen(trace_error.error_summary.value);
        set_trace_error(&span, &trace_error);
    }

    //线程2
    nRet2 = pthread_create(&thrd_Peter, NULL, sayHello, (void *) name2);
    if(0 != nRet2 ) {
        trace_error_t trace_error;
        trace_error.error_summary.value = "Fail to create Peter thread !";
        trace_error.error_summary.length = strlen(trace_error.error_summary.value);
        set_trace_error(&span, &trace_error);
    }
    pthread_join(thrd_Lily, NULL);
    pthread_join(thrd_Peter, NULL);

    br_finish_span(&span, &fin_span_opts);
    br_finish_tracer(&g_tracer);

    memset(&bt_finish_options, 0, sizeof(bt_finish_options));
    bt_finish_options.flags = 0;
    br_finish_bt(&bt, &bt_finish_options,NULL);

    br_close_transaction(&g_transaction);
    return 0;
}
#endif

void collect_bt()
{
    //业务数据采集
    transaction_t transaction;
    static int cnt = 0;
    if (0 == br_open_transaction(&transaction)) {
        //创建bt
        bt_options_t options;
        http_req_data_t http_req;
        memset(&options, 0, sizeof(options));
        memset(&http_req, 0, sizeof(http_req));
        cnt++;
        cnt = cnt %5;
        str_malloc(&options.name, 1024);
        snprintf(options.name.value, 1023,"test_bt_name_%d", cnt);

        http_req.url.value = "http://www.bonree.com/test/index";
        bt_t bt;
        br_get_or_create_bt(&transaction, &options, &http_req, &bt);
        str_free(&options.name);
        //添加后端调用
        // 1、nosql数据库调用
        {
            be_call_options_t be_options;
            memset(&be_options, 0, sizeof(be_options));
            //后端信息
            be_options.be.conn_type.value = "StackExchangeRedis";
            be_options.be.type = BACKEND_TYPE_REDIS;
            be_options.be.host.value = "192.168.0.201";
            be_options.be.port = 6379;
            //后端调用细节
            be_options.call_detail.detail.value = "Connect 192.168.0.201:6379";
            be_call_t be_call;
            br_start_be_call(&transaction, &be_options, &be_call);
            //结束becall
            msleep(1);
            be_call_finish_options_t be_finish_options;
            memset(&be_finish_options, 0, sizeof(be_finish_options));
            //标记后端调用出错
            //be_finish_options.flags |= FLAG_ERROR;
            br_finish_be_call(&be_call, &be_finish_options);
        }
        //2、关系型数据库调用
        {
            be_call_options_t be_options;
            memset(&be_options, 0, sizeof(be_options));
            //后端信息
            be_options.be.conn_type.value = "PROC";
            be_options.be.type = BACKEND_TYPE_ORACLE;
            be_options.be.host.value = "192.168.0.202";
            be_options.be.port = 1521;
            be_options.be.db_name.value = "AcctDB";
            //后端调用细节
            be_options.call_detail.detail.value = "select * from table";
            be_call_t be_call;
            br_start_be_call(&transaction, &be_options, &be_call);
            //结束becall
            msleep(1);
            be_call_finish_options_t be_finish_options;
            memset(&be_finish_options, 0, sizeof(be_finish_options));
            //标记后端调用出错
            br_finish_be_call(&be_call, &be_finish_options);
        }
        //结束bt
        //bt_finish_options 填充信息
        bt_finish_options_t bt_finish_options;
        http_resp_data_t http_resp;
        memset(&bt_finish_options, 0, sizeof(bt_finish_options));
        memset(&http_resp, 0, sizeof(http_resp));
        bt_finish_options.flags = 0;
        http_resp.status_code.value = "200";
        br_finish_bt(&bt, &bt_finish_options, &http_resp);
        //关闭记录
        br_close_transaction(&transaction);
    }
}

void collect_trace()
{
//业务数据采集
    transaction_t transaction;
    if (0 == br_open_transaction(&transaction)) {
        //创建bt
        bt_options_t options;
        http_req_data_t http_req;
        memset(&options, 0, sizeof(options));
        memset(&http_req, 0, sizeof(http_req));

        options.name.value = "test_trace_name";
        http_req.url.value = "http://www.bonree.com/test/index";
        bt_t bt;
        br_get_or_create_bt(&transaction, &options, &http_req, &bt);

        //添加后端调用
        be_call_options_t be_options;
        memset(&be_options, 0, sizeof(be_options));
        //后端信息
        be_options.be.conn_type.value = "StackExchangeRedis";
        be_options.be.type = BACKEND_TYPE_REDIS;
        be_options.be.host.value = "192.168.0.201";
        be_options.be.port = 6379;
        //后端调用细节
        be_options.call_detail.detail.value = "Connect 192.168.0.201:6379";
        //后端调用函数信息
        be_options.class_method.method_name.value = "method_name";
        be_options.class_method.class_name.value = "class_name";

        be_call_t be_call;
        br_start_be_call(&transaction, &be_options, &be_call);
        be_call_finish_options_t be_finish_options;
        memset(&be_finish_options, 0, sizeof(be_finish_options));
        tracer_t tracer;
        br_get_or_create_tracer(&transaction, &tracer);
        start_span_options_t span_options;
        memset(&span_options, 0, sizeof(span_options));
        span_options.thread_info.thread_name.value = "test_thread";
        span_options.thread_info.thread_id = getpid();

        span_options.method_info.class_name.value = "test";
        span_options.method_info.method_name.value = "collect_trace";
        span_options.method_info.ret_value.value = "void";
        span_options.method_info.params.value = "int,char*";

        span_t span;
        start_span(&tracer, &span_options, &span);
        trace_error_t trace_error;
        trace_error.error_summary.value = "sum";
        trace_error.error_summary.length = 3;
        trace_error.error_details.value = "det";
        trace_error.error_details.length = 3;
        set_trace_error(&span, &trace_error);
        link_to_be_call(&span, &be_call);
        msleep(1);
        span_t sub_span;

        span_options.method_info.method_name.value = "collect_trace_sub";
        br_start_span(&tracer, &span_options, &sub_span);
        msleep(1);
        finish_span_options_t fin_span_opts_sub, fin_span_opts;
        memset(&fin_span_opts_sub, 0x0, sizeof(fin_span_opts_sub));
        memset(&fin_span_opts, 0x0, sizeof(fin_span_opts));

        br_finish_span(&sub_span, &fin_span_opts_sub);
        br_finish_span(&span, &fin_span_opts);
        br_finish_be_call(&be_call, &be_finish_options);
        br_finish_tracer(&tracer);
        //结束bt
        //bt_finish_options 填充信息
        bt_finish_options_t bt_finish_options;
        memset(&bt_finish_options, 0, sizeof(bt_finish_options));
        bt_finish_options.flags = 0;
        br_finish_bt(&bt, &bt_finish_options, NULL);
        //关闭记录
        br_close_transaction(&transaction);
    }
}

int main()
{
    srand((unsigned)time(NULL));
    //sdk 初始化
    if (0 == br_init_sdk(0)) {
        br_set_logging_level(LOG_LEVEL_DEBUG);
        while(!br_agent_enable()){
            msleep(1000);
        }
        printf("collect_trace\n");
        trace_with_multithread();
        while(1){
			collect_bt();
            msleep(5);
        }
    }
    //释放sdk
    br_un_init_sdk();
    return 0;
}
