/*
 * Copyright © 2007-2020 Bonree Data Technology Co.,Ltd
 * All rights reserved
 *
 */
 
#include <iostream>
#include <memory>
#include <string>
#include <unistd.h>

#include <grpcpp/grpcpp.h>
#include "bonree.grpc.pb.h"
#include "sdk.h"
#include "common/cross.h"

using grpc::Channel;
using grpc::ClientContext;
using grpc::Status;
using Bonree::BrApp;
using Bonree::CrossFuncARequest;
using Bonree::CrossFuncAResponse;

static transaction_t g_trans_t;
static tracer_t g_trace_t;
static span_t g_sub_span;
static cross_req_head_t g_cr_rsp_head;
static cross_req_head_t g_cr_req_head;



class BrAppClient {
public:
    BrAppClient(std::shared_ptr<Channel> channel)
    : stub_(BrApp::NewStub(channel)) {}
    int32_t CrossFuncA(int32_t param1, int32_t param2) {
        ClientContext context;
        Bonree::CrossFuncARequest request;
        Bonree::CrossFuncAResponse reply;
	
        be_call_t be_call;
	    be_call_options_t opts_bt_rpc_call;
        memset(&opts_bt_rpc_call,0x0,sizeof(opts_bt_rpc_call));
        be_call_finish_options_t opts_fin_be_call;
        memset(&opts_fin_be_call,0x0,sizeof(opts_fin_be_call));
        //填充后端调用详情选项参数
        opts_bt_rpc_call.be.type = BACKEND_TYPE_GRPC; //后端类型
        opts_bt_rpc_call.be.host.value = "127.0.0.1";    //后端ip
        opts_bt_rpc_call.be.port = 50098;          //后端端口
        opts_bt_rpc_call.call_detail.is_async = 0;    //是否是异步，1：为异步，0：不是异步
        opts_bt_rpc_call.call_detail.detail.value = "BrApp.CrossFuncA";
        opts_bt_rpc_call.class_method.class_name.value = "BrApp";
        opts_bt_rpc_call.class_method.method_name.value = "CrossFuncA";
       
        br_process_cross_req_head(&g_trans_t, NULL);
        br_start_rpc(&g_trans_t,&opts_bt_rpc_call,&g_cr_req_head,&be_call);
        br_link_to_be_call(&g_sub_span, &be_call);
        //设置远程调用请求头
        if(g_cr_req_head.data) {
              context.AddMetadata(grpc::string(cross_req_header_key), grpc::string(g_cr_req_head.data));
        }
	    request.set_param1(param1);
	    request.set_param2(param2);
        Status status = stub_->CrossFuncA(&context, request, &reply);
        if (status.ok()) {
            const auto& server_trailing_metadata = context.GetServerTrailingMetadata();
            //获取远程调用响应头
            auto iter = server_trailing_metadata.find(cross_rsp_header_key);
            if(iter != server_trailing_metadata.end()) {
                std::string res = std::string(iter->second.begin(), iter->second.end());
                if(reply.retval() < 0) {
                    opts_fin_be_call.flags = FLAG_ERROR;
                 }
                memset(&g_cr_rsp_head,0x0,sizeof(g_cr_rsp_head));
                strncpy(g_cr_req_head.data,res.c_str(),sizeof(g_cr_req_head.data)-1);
            }
             
	    
            br_finish_rpc(&g_trans_t,&be_call, &opts_fin_be_call,NULL);		
            return reply.retval();
        } else {

            br_finish_rpc(&g_trans_t,&be_call, &opts_fin_be_call,NULL);
            return -1;
        }
    }

private:
    std::unique_ptr<BrApp::Stub> stub_;

};

static BrAppClient * g_brApp = NULL;

void clientFunc(){
    start_span_options_t span_options;
    memset(&span_options,0x0,sizeof(span_options));
    span_options.thread_info.thread_name.value = "test_thread";
    span_options.thread_info.thread_id = getpid();

    span_options.method_info.class_name.value = "BrAppClient";
    span_options.method_info.method_name.value = "CrossFuncA";
    span_options.method_info.ret_value.value = "int";
    span_options.method_info.params.value = "int, int";
    br_start_span(&g_trace_t,&span_options,&g_sub_span);	
    //用户rpc调用
    auto ret = g_brApp->CrossFuncA(1, 2);
	finish_span_options_t fin_span_opts;
	memset(&fin_span_opts, 0x0, sizeof(fin_span_opts));
    br_finish_span(&g_sub_span, &fin_span_opts);
}


int main(int argc, char** argv) {
    g_brApp = new BrAppClient(grpc::CreateChannel(
            "localhost:50098", grpc::InsecureChannelCredentials()));
        if(br_init_sdk(0)==0) {
    	while(!br_agent_enable()){
            usleep(1000*1000);
        }
        br_set_logging_level(LOG_LEVEL_DEBUG);
         //开始一个事务记录，记录一次业务处理过程
         br_open_transaction(&g_trans_t);
         //开始业务
         bt_t bt;
         //填充创建业务选项，此处填入函数调用方法名
         bt_options_t opts_bt;
         memset(&opts_bt,0x0,sizeof(opts_bt));
         opts_bt.name.value = "clientFunc";
    
         //创建业务
         br_get_or_create_bt(&g_trans_t, &opts_bt,NULL, &bt);
       
        br_get_or_create_tracer(&g_trans_t, &g_trace_t);
        start_span_options_t span_options;
        memset(&span_options, 0x0, sizeof(span_options));
        span_options.thread_info.thread_name.value = "test_thread";
        span_options.thread_info.thread_id = getpid();

        span_options.method_info.class_name.value = "test";
        span_options.method_info.method_name.value = "clientFunc";
        span_options.method_info.ret_value.value = "void";
        span_options.method_info.params.value = "void";

        span_t span;
        br_start_span(&g_trace_t, &span_options, &span);
        //用户自定义函数调用
        clientFunc();
		finish_span_options_t fin_span_opts;
		memset(&fin_span_opts, 0x0, sizeof(fin_span_opts));
        br_finish_span(&span,&fin_span_opts);
        //填充结束业务参数选项(对于rpc调用，需填入调用结果 正常(0)/错误(1)/异常(2)，
         //错误异常的情况，需进一步填入异常详情
         bt_finish_options_t opts_bt_fin;
         memset(&opts_bt_fin, 0x0, sizeof(opts_bt_fin));
         opts_bt_fin.flags = 0;
         br_finish_tracer(&g_trace_t);
         //结束业务
         br_finish_bt(&bt, &opts_bt_fin,NULL);
	
         //结束事务
         br_close_transaction(&g_trans_t);
         while(1){
             usleep(5000);
         }
    }
    //释放sdk
    br_un_init_sdk();
    return 0;
}

