/*
 * Copyright © 2007-2019 Bonree Data Technology Co.,Ltd
 * All rights reserved
 *
 */
 
#include <iostream>
#include <memory>
#include <string>
#include <unistd.h>


#include <grpcpp/grpcpp.h>
#include "bonree.grpc.pb.h"
#include "sdk.h"
#include "common/cross.h"

using grpc::Server;
using grpc::ServerBuilder;
using grpc::ServerContext;
using grpc::Status;
using Bonree::BrApp;

static cross_req_head_t g_cr_rsp_head;
static cross_req_head_t g_cr_req_head;


//用户自定义函数
int32_t serverFunc(int32_t param1, int32_t param2) {
    //std::cout<<"This is serverFunc"<<std::endl;
    printf("This is serverFunc\n");
    return param1 + param2;
}

// Logic and data behind the server's behavior.
class BrAppServiceImpl final : public BrApp::Service {
    Status CrossFuncA(ServerContext* context, const Bonree::CrossFuncARequest* request,
            Bonree::CrossFuncAResponse *reply) override {
        //开启事务
        static transaction_t transaction;
        br_open_transaction(&transaction);

        //创建业务
        bt_t bt;
        //填充创建业务选项，此处填入函数调用方法名
        bt_options_t opts_bt;
        memset(&opts_bt,0x0,sizeof(opts_bt));
        opts_bt.name.value = "serverFunc";
        br_get_or_create_bt(&transaction, &opts_bt,NULL, &bt);
        //获取http请求头元数据
        auto clientMetadata = context->client_metadata();

        auto it = clientMetadata.find(cross_req_header_key);
        if(it != clientMetadata.end()) {
            cross_req_head_t cross_req_head;
            memset(cross_req_head.data,0x0,sizeof(cross_req_head.data));
            auto req = std::string(it->second.begin(), it->second.end());
            strncpy(cross_req_head.data,req.c_str(),sizeof(cross_req_head.data)-1);
            br_process_cross_req_head(&transaction,&cross_req_head);
        }

        tracer_t tracer; 
        br_get_or_create_tracer(&transaction, &tracer);
        start_span_options_t span_options;
        memset(&span_options, 0x0, sizeof(span_options));
        span_options.thread_info.thread_name.value = "test_server_thread";
        span_options.thread_info.thread_id = getpid();

        span_options.method_info.class_name.value = "";
        span_options.method_info.method_name.value = "serverFunc";
        span_options.method_info.ret_value.value = "int";
        span_options.method_info.params.value = "int, int";

        span_t span;
        br_start_span(&tracer, &span_options, &span);
         
        //用户自定义代码
        int32_t nRet = serverFunc(request->param1(),request->param2());
        
        reply->set_retval(nRet);

        bt_finish_options_t opts_bt_fin;
        memset(&opts_bt_fin, 0x0, sizeof(opts_bt_fin));
        if(nRet < 0) {
            opts_bt_fin.flags = FLAG_ERROR;
        }

        cross_req_head_t cross_resp_head;
        memset(&cross_resp_head,0x0, sizeof(cross_resp_head));
        br_gen_cross_resp_head(&transaction, &cross_resp_head);

        if(strlen(cross_resp_head.data)>0) {
            context->AddTrailingMetadata(grpc::string(cross_rsp_header_key),grpc::string(cross_resp_head.data));
        }

        //结束释放
		finish_span_options_t fin_span_opts;
		memset(&fin_span_opts, 0x0, sizeof(fin_span_opts));
        br_finish_span(&span,&fin_span_opts);
        br_finish_tracer(&tracer);
        br_finish_bt(&bt, &opts_bt_fin,NULL);
        br_close_transaction(&transaction);
        return Status::OK;
    }

};

void RunServer() {
    std::string server_address("0.0.0.0:50098");
    BrAppServiceImpl service;
    ServerBuilder builder;
    builder.AddListeningPort(server_address, grpc::InsecureServerCredentials());
    builder.RegisterService(&service);
    std::unique_ptr<Server> server(builder.BuildAndStart());
    //std::cout << "Server listening on " << server_address << std::endl;

    //sdk初始化
	
    if(br_init_sdk(0)==0) {
       br_set_logging_level(LOG_LEVEL_DEBUG);
       server->Wait(); 
    }
    br_un_init_sdk();
}
int main(int argc, char** argv) {
    RunServer();
    return 0;
}
