
// Created by liuheng@bonree.com on 2019/11/14 0014.
// Copyright © 2007-2019 Bonree Data Technology Co.,Ltd
// All rights reserved


#ifndef _BONREE_SDK_BTDETAIL_BT_DETAIL_H_
#define _BONREE_SDK_BTDETAIL_BT_DETAIL_H_


#include "bt.h"
#include "be_call.h"
#include "trace/tracer.h"

#ifdef __cplusplus
extern "C" {
#endif

//事务记录对象， 管理业务/后端相关数据
//struct transaction_t {
//    void *ins;
//};
typedef struct sdk_object_t transaction_t;

//获取业务(BT)接口对象
//参数：transaction[in]: transaction_t* 事务记录对象
//参数：options[in]: bt_options_t*, 创建bt的选项参数
//参数：http_req[in]:struct http_req_data_t*, http请求数据信息（可选， 非http业务该参数传null）
//参数：bt[out]: bt_t 获取的业务接口对象
//返回值：0：获取成功， 非0：获取失败
BONREE_SDK_API int br_get_or_create_bt(const transaction_t* transaction, const bt_options_t* options,
        const http_req_data_t* http_req, bt_t* bt);

//开始一个后端调用
//参数：transaction[in]: transaction_t* 事务记录对象
//参数：options[in]: be_call_options_t, 创建后端调用的选项参数
//参数：be_call[out]: be_call_t 获取的后端调用
//返回值：0：获取成功， 非0：获取失败
BONREE_SDK_API int br_start_be_call(const transaction_t* transaction, const be_call_options_t* options, be_call_t* be_call);

//开始一个rpc后端调用, br_start_be_call的升级版, 创建be后端并生成跨容器请求头字段
//参数：transaction[in]: transaction_t* 事务记录对象
//参数：options[in]: be_call_options_t, 创建后端调用的选项参数
//参数：cross_req_head[out]: cross_req_head_t*, 跨容器请求头字段值
//可用工具函数 get_cross_req_head_from_head(str_t *head, cross_req_head_t* cross_req_head) 从请求头中获取跨容器请求头字段值
//参数：be_call[out]: be_call_t* 获取的后端调用
//返回值：0：获取成功， 非0：获取失败
BONREE_SDK_API int br_start_rpc(const transaction_t* transaction, const be_call_options_t* options, cross_req_head_t* cross_req_head, be_call_t* be_call);

//结束rpc后端调用
//参数：transaction[in]: transaction_t* 事务记录对象
//参数：be_call[in]: be_call_t* 要结束的后端调用
//参数：options[in]: be_call_finish_options_t* 参数选项
//参数：cross_rsp_head[in]: cross_req_head_t*, 跨容器请求响应头字段值
//可用工具函数 get_cross_resp_head_from_head(str_t *head, cross_req_head_t* cross_req_head) 从响应头中获取跨容器响应头字段值
//返回值： 0：成功， 非0： 失败
BONREE_SDK_API int br_finish_rpc(const transaction_t* transaction, const be_call_t* be_call, const be_call_finish_options_t* options,  cross_req_head_t* cross_rsp_head);

//创建tracer 调用链对象
//参数：transaction[in]: transaction_t* 事务记录对象
//参数：tracer[out]: tracer_t* 获取的tracer对象
//返回值：0：获取成功， 非0：获取失败
BONREE_SDK_API int br_get_or_create_tracer(const transaction_t* transaction, tracer_t* trace);

//处理跨容器请求头
//参数：transaction[in]: transaction_t* 事务记录对象
//参数：cross_req_head[in]: cross_req_head_t*， 跨容器请求头字段值
//参数：is_sampling[out]: 是否采集trace，1：采集，其他：不采集
//返回值：0：成功， 非0：失败
BONREE_SDK_API int br_process_cross_req_head(const transaction_t* transaction, cross_req_head_t* cross_req_head, int* is_sampling);

//参数：transaction[in]: transaction_t* 事务记录对象
//参数：cross_req_head[out]: cross_req_head_t*， 跨容器请求响应头字段值
//返回值：0：成功， 非0：失败
BONREE_SDK_API int br_gen_cross_resp_head(const transaction_t* transaction,  cross_req_head_t* cross_rsp_head);

#ifdef __cplusplus
}
#endif

#endif //_BONREE_SDK_BTDETAIL_BT_DETAIL_H_
