
// Created by liuheng@bonree.com on 2019/11/14 0014.
// Copyright © 2007-2019 Bonree Data Technology Co.,Ltd
// All rights reserved


#ifndef _BONREE_SDK_COMMON_STRUCTS_H_
#define _BONREE_SDK_COMMON_STRUCTS_H_

#include "common/enums.h"

#ifdef __cplusplus
extern "C" {
#endif

//#ifdef WINDOWS
//#pragma pack(push, 1)
//#else
//#pragma pack(1)
//#endif

//字符串
typedef  struct str_t {
    char* value;            //字符数组
    unsigned int length;    //字符数组长度
} str_t;

//容器信息
typedef struct  container_info_t {
    str_t   agent_version;					        //agent版本
    unsigned long long    agent_install_timestamp;  //agent安装日期
    str_t   agent_install_dir;				        //agent安装路径
    str_t   listens;                                //监听ip及端口,ip1:port1|ip2:port2|ip3:port3|
    str_t   container_version;				        //容器版本
    str_t   container_install_dir;			        //容器安装目录
    str_t   vm_info;						        //虚拟机版本信息(java、donet等)
    str_t   vm_startup_options;				        //虚拟机启动参数
    str_t   vm_system_options;				        //虚拟机系统变量
} container_info_t;

typedef struct sdk_options_t {
    enum AgentType agent_type;                  //agent类型，默认为C_agent
    unsigned long pid;                          //进程id,默认为当前进程id
    int flags;                                  //标识
} sdk_options_t;

#pragma region btbe

//创建BT业务选项参数
typedef struct  bt_options_t {
    enum BTType type;                               //业务类别, 默认为BT_C_USER
    str_t internal_name;                            //业务名
    str_t uri;                                      //业务的URI属性，只有http业务有值（controller注册使用第一次上报的uri即可，后续不需要更新）
    unsigned long long start_time_stamp;		    //业务开始时间戳
} bt_options_t;

//http请求数据
typedef struct http_req_data_t {
     str_t url;             //url地址
     str_t http_method;     //http请求方法
     str_t parameters;      //请求参数数组,‘&’分割, e.g.  id=1000&name=lh
     str_t req_headers;     //http请求头字段
     str_t req_content;     //请求content内容
} http_req_data_t;

//http请求数据
typedef struct http_resp_data_t {
    str_t status_code;     //http响应码
	str_t res_headers;     //http响应码
    str_t resp_content;    //响应content内容
} http_resp_data_t;

//错误信息
typedef struct bt_error_t {
    enum ErrorType  error_type;                         //错误类型
    str_t error_name;                                   //错误name
    str_t error_sumary;                                 //错误概要
    str_t error_details;                                //错误描述
    unsigned long long start_time_stamp;		        //错误发生的时间戳
} bt_error_t;

//结束BT业务选项参数
typedef struct  bt_finish_options_t {
#define  FLAG_HTTPERROR 0x1
#define  FLAG_EXCEPTION 0x2
    unsigned int flags;                                 //标识业务是否出现错误或者异常
    unsigned long long finish_time_stamp;		        //业务结束时间戳
} bt_finish_options_t;

//后端信息
typedef struct be_t {
    enum BackendType type;      //后端类型
    str_t conn_type;            //连接类型，JDBC/ADO.NET等
    str_t host;                 //后端host
    unsigned int port;          //后端端口
    str_t db_name;              //sql 数据库名
} be_t;

//后端调用详情
typedef struct call_detail_t {
    int is_async;            //是否是异步，1：为异步，0：不是异步
    str_t  detail;           //后端调用的详细信息，如:详细的sql语句
} call_detail_t;

//方法信息
typedef struct class_method_t {
    str_t class_name;   //类名
    str_t method_name;  //方法名
} class_method_t;

//创建后端调用选项参数
typedef struct be_call_options_t {
    be_t be;                                            //后端信息
    call_detail_t call_detail;                          //调用详情
    class_method_t class_method;                        //调用的方法信息
    unsigned long long start_time_stamp;		        //后端调用开始时间戳
} be_call_options_t;

//结束后端调用选项参数
typedef struct be_call_finish_options_t {
#define FLAG_ERROR 0x1
    int flags;                                            //标识后端调用是否出错
    unsigned long long finish_time_stamp;		          //后端调用结束时间戳
	str_t error_name;									  //后端调用错误名
	str_t error_detail;									  //后端调用错误详情
    str_t sql_explain_plan;                               //sql语句执行计划
} be_call_finish_options_t;

//跨容器请求的请求头/响应头字段, 用于追加到http 业务的请求头或响应头
typedef struct cross_req_head_t {
#define MAX_CROSS_REQ_HEAD_SIZE (1024)
    char data[MAX_CROSS_REQ_HEAD_SIZE];
} cross_req_head_t;

//MQ信息
typedef struct  mq_info_t {
    unsigned long long mq_size;
    unsigned int direction;//0:未设置, 1:表示指向后端（比如MQ生产者）, 2:表示指向业务（比如MQ消费者）
} mq_info_t;

//创建span(调用链方法)选项参数
typedef struct span_thread_info_t {
    str_t        thread_name;        //方法执行所在的线程名
    unsigned long long  thread_id;   //方法执行所在的线程id
} span_thread_info_t;

typedef struct span_method_info_t {
    str_t class_name;        //方法的类名
    str_t method_name;       //方法名 (必填）
    str_t ret_value;         //方法的返回参数
    str_t params;            //方法的参数集, ',' 分割多个参数
    unsigned int line_number;       //方法调用在代码中的行数
} span_method_info_t;

typedef struct  start_span_options_t {
    span_thread_info_t thread_info;
    span_method_info_t method_info;
    unsigned long long start_time_stamp;		          //开始时间戳
} start_span_options_t;

typedef  struct  finish_span_options_t {
    unsigned long long finish_time_stamp;		          //结束时间戳
} finish_span_options_t;
//trace error
typedef struct trace_error_t{
    str_t error_summary;      //错误概要
    str_t error_details;      //错误描述
} trace_error_t;

//trace log
typedef struct trace_log_t {
    str_t level;
    str_t log_msg;
} trace_log_t;

typedef  struct event_t {
    enum EventType type;                    //事件类型
    str_t summary ;                         //事件概要
    str_t detail;                           //事件详情
    unsigned long long timestamp;           //事件发生时间
} event_t;
#pragma endregion

//#ifdef WINDOWS
//#pragma pack(pop)
//#else
//#pragma pack()
//#endif

#ifdef __cplusplus
}
#endif

#endif //_BONREE_SDK_COMMON_STRUCTS_H_
