'use strict';
var winston = require('winston');
var cluster = require('cluster');
var fs = require('fs');
var path = require('path');
function Logger(agent) {
    this.agent = agent;
    this.initialized = false;
}
var DEBUG_CATEGARIES ={
    DEBUG_CATEGARY_TRASACTION: 0,
    DEBUG_CATEGARY_SNAPSHOT:1,
    DEBUG_CATEGARY_METRIC:2,
};

exports.Logger = Logger;

Logger.prototype.init = function ()
{
    var self = this;
    var config;
    if(self.agent.config)
    {
        config = self.agent.config.config;
    }
    if(config)
    {
        self.dir = process.env.HOME || process.env.HOMEPATH || process.env.USERPROFILE;
        if(config.hasOwnProperty('dir'))//取自定义日志目录
        {
            self.dir = config.dir;
        }
        if(!self.dir) {//取smartagent目录
            var idx = __dirname.indexOf('Bonree');
            var agentPath = __dirname.substring(0,idx);
            self.dir = agentPath+'../../../logs/nodejs/';
        }
        if(!fs.existsSync(self.dir))
        {
            try{
                fs.mkdirSync(self.dir);
            }catch(err){
                console.log('Failed to created dir '+self.dir);
                return;
            }
        }
        var enabled = config.enablelogging;
        this.lastagentname = config.containerInfo.agentName
        this.lastprocessid = config.containerInfo.processId
        if(enabled){
            var logfile = ''
            if(this.lastagentname){
                logfile = path.join(self.dir, this.lastprocessid + '+' + this.lastagentname  + '.log');
            }else{
                logfile = path.join(self.dir, this.lastprocessid + '_nodejs_agent'+'.log');
            }
        }
        if(enabled)
        {
            var loglevel = config.loglevel;
            self._logger = new (winston.Logger)({
                transports:[
                    new (winston.transports.File)(
                        {
                            // stream: fs.createWriteStream(logfile, {flags: 'a'}),
                            filename: logfile,
                            tailable: true,
                            maxFiles: config.logFileCount?config.logFileCount:2,
                            maxsize: 1024*(config.logLimitInKbytes?config.logLimitInKbytes:20480),
                            level: loglevel,
                            json: false,
                            options:{flags: 'a'},
                            timestamp:function () {
                                return Date().toLocaleString();
                            }
                        }
                    )
                ]
            });

            self.initialized = true;
        }
    }
};
Logger.prototype.__log = function (level,msg,categary) {
    var self = this;
    var config = self.agent.config.config;
    if(self.initialized && !!self.agent)
    {
        /*
        var needRename = false;
        if(self.lastagentname != config.containerInfo.agentName) {
            self.lastagentname = config.containerInfo.agentName;
            needRename = true;
        }
        if(self.lastprocessid != config.containerInfo.processId) {
            self.lastprocessid = config.containerInfo.processId;
            needRename = true;
        }
        if(needRename) {
            self._logger.configure({
                filename: path.join(self.dir, self.lastagentname +'' + self.lastprocessid + '.log'),
            })
        }
         */
        switch(categary)
        {
            case DEBUG_CATEGARIES.DEBUG_CATEGARY_METRIC:
            {
                if(!self.agent.config.config.debug_metric)
                {
                    return;
                }
            }break;
            case DEBUG_CATEGARIES.DEBUG_CATEGARY_SNAPSHOT:
            {
                if(!self.agent.config.config.debug_snapsthot)
                {
                    return;
                }
            }break;
            case DEBUG_CATEGARIES.DEBUG_CATEGARY_TRASACTION:
            {
                if(!self.agent.config.config.debug_transaction)
                {
                    return;
                }
            }break;
        }
        self._logger.log(level,msg);
    }
};
Logger.prototype.log = function (msg,categary)
{
    var self = this;
    self.__log('info',msg,categary);
};

Logger.prototype.info = function (msg,categary)
{
    var self = this;
    self.__log('info',msg,categary);
};
Logger.prototype.warn = function (msg,categary)
{
    var self = this;
    self.__log('warn',msg,categary);
};

Logger.prototype.error = function (msg,categary)
{
    var self = this;
    self.__log('error',msg,categary);
};

Logger.prototype.debug = function (msg,categary)
{
    var self = this;
    self.__log('debug',msg,categary);
};

Logger.prototype.silly = function (msg,categary) {
    var self = this;
    self.__log('silly',msg,categary);
};
function logfunc(func,categary) {
    return function categaryLog(logger,msg){func.apply(logger,[msg,categary]);};
}
exports.Logger_funcs = {
    transaction_info : logfunc(Logger.prototype.info,DEBUG_CATEGARIES.DEBUG_CATEGARY_TRASACTION),
    transaction_error : logfunc(Logger.prototype.error,DEBUG_CATEGARIES.DEBUG_CATEGARY_TRASACTION),
    transaction_warn : logfunc(Logger.prototype.warn,DEBUG_CATEGARIES.DEBUG_CATEGARY_TRASACTION),
    transaction_debug : logfunc(Logger.prototype.debug,DEBUG_CATEGARIES.DEBUG_CATEGARY_TRASACTION),
    snapshot_info : logfunc(Logger.prototype.info,DEBUG_CATEGARIES.DEBUG_CATEGARY_SNAPSHOT),
    snapshot_error : logfunc(Logger.prototype.error,DEBUG_CATEGARIES.DEBUG_CATEGARY_SNAPSHOT),
    snapshot_warn : logfunc(Logger.prototype.warn,DEBUG_CATEGARIES.DEBUG_CATEGARY_SNAPSHOT),
    snapshot_debug : logfunc(Logger.prototype.debug,DEBUG_CATEGARIES.DEBUG_CATEGARY_SNAPSHOT),
    metric_info : logfunc(Logger.prototype.info,DEBUG_CATEGARIES.DEBUG_CATEGARY_METRIC),
    metric_error : logfunc(Logger.prototype.error,DEBUG_CATEGARIES.DEBUG_CATEGARY_METRIC),
    metric_warn : logfunc(Logger.prototype.warn,DEBUG_CATEGARIES.DEBUG_CATEGARY_METRIC),
    metric_debug : logfunc(Logger.prototype.debug,DEBUG_CATEGARIES.DEBUG_CATEGARY_METRIC),

};