var EventEmitter = require('events').EventEmitter;
var shells = require('./namedwrap');

function Wrap(agent) {
    this.agent = agent;
    this.shellMap = undefined;
    this.shellContextMap = undefined;
    this.shellIndex  = undefined;
    this.isShellWrapEnabled = false;
}


exports.Wrap = Wrap;

Wrap.prototype.init = function() {
    var self = this;

    // removeListener compairs objects, so the original callback
    // should be passed instead of the proxy
    self.before(EventEmitter.prototype, 'removeListener', function(obj, args) {
        if(args.length > 1 && args[1] && args[1].__bonreeWrap__) {
            args[1] = args[1].__bonreeWrap__;
        }
    });

};



var Locals = function() {
    this.stackTrace = undefined;
    this.params = undefined;
    this.opts = undefined;
    this.group = undefined;
    this.req = undefined;
    this.res = undefined;
    this.error = undefined;
    this.transaction = undefined;
    this.exitCall = undefined;
    this.callbackEmitter = new EventEmitter();

};



Wrap.prototype.before = function(obj, meths, actBefore, isCallbackHook,actErr) {
    var self = this;

    if(!obj) return false;

    if(!Array.isArray(meths)) meths = [meths];

    meths.forEach(function(meth) {
        var orig = obj[meth];
        if(!orig) return;


        obj[meth] = function bonr_wrap() {
            var locals = new Locals();
            if(isCallbackHook) {
                var selfWrap = this;

                // the hook code should contain try/catch
                actBefore(this, arguments, function() {
                    try {
                        return orig.apply(selfWrap, arguments);
                    }catch(e)
                    {
                        locals.error = e;
                        if(actErr)
                        {
                            try {
                                actErr(this, arguments, locals);
                            }catch (err)
                            {
                                self.logError(err);
                            }
                        }
                        throw e;
                    }
                });
            }
            else {
                try {
                    actBefore(this, arguments);
                }
                catch(e) {
                    self.logError(e);
                }

                // return orig.apply(this, arguments);
                try {
                    return orig.apply(this, arguments);
                }catch(e)
                {
                    locals.error = e;
                    if(actErr)
                    {
                        try {
                            actErr(this, arguments, locals);
                        }catch (err)
                        {
                            self.logError(err);
                        }
                    }
                    throw e;
                }
            }
        };
    });
};

Wrap.prototype.after = function(obj, meths, actAfter,actErr) {
    var self = this;

    if(!obj) return false;

    if(!Array.isArray(meths)) meths = [meths];

    meths.forEach(function(meth) {
        var orig = obj[meth];
        if(!orig) return;

        obj[meth] = function() {
            var locals = new Locals();
            var ret;
            try {
                ret = orig.apply(this, arguments);
            }catch(e)
            {
                locals.error = e;
                if(actErr)
                {
                    try {
                        actErr(this, arguments, locals);
                    }catch (err)
                    {
                        self.logError(err);
                    }
                }
                throw e;
            }
            // var ret = orig.apply(this, arguments);

            var hookRet;
            try {
                hookRet = actAfter(this, arguments, ret);
            }
            catch(e) {
                self.logError(e);
            }

            return hookRet || ret;
        };
    });
};

Wrap.prototype.around = function(obj, meths, actBefore, actAfter,actErr) {
    var self = this;

    if(!obj) return false;

    if(!Array.isArray(meths)) meths = [meths];

    meths.forEach(function(meth) {
        var orig = obj[meth];
        if(!orig) return;

        obj[meth] = function() {
            var locals = new Locals();
            locals.methodName = meth;


            try {
                actBefore(this, arguments, locals);
            }
            catch(e) {
                self.logError(e);
            }

            var ret;
            try {
                ret = orig.apply(this, arguments);
            }catch(e)
            {
                locals.error = e;
                if(actErr)
                {
                    try {
                        actErr(this, arguments, locals);
                    }catch (err)
                    {
                        self.logError(err);
                    }
                }
                throw e;
            }
            var hookRet;
            try {
                hookRet = actAfter(this, arguments, ret, locals);
            }
            catch(e) {
                self.logError(e);
            }

            return hookRet || ret;
        };
    });
};

Wrap.prototype.enableShellWrap = function () {

    this.disableShellWrap();
    this.isShellWrapEnabled = true;
};

Wrap.prototype.disableShellWrap = function () {
    this.isShellWrapEnabled = false;
    this.shellIndex = -1;
    this.shellMap = {};
    this.shellContextMap = {};
};

Wrap.prototype.getShellContext = function (shellId) {
    var self = this;
    return self.isShellWrapEnabled && self.shellContextMap[shellId];
};

Wrap.prototype.shellWrap = function (func) {
    var self = this;
    var threadId = self.agent.thread.current();
    if(threadId !== undefined)
    {
        var shellId = self.shellMap[threadId];
        if(shellId >=0)
        {
            return shells[shellId](func);
        }
        else if (this.shellIndex++ < 255)
        {
            shellId = this.shellIndex;
            self.shellContextMap[shellId] = {
                threadId: threadId,
            };
            self.shellMap[threadId] = shellId;
            return shells[shellId](func);
        }
    }
    return undefined;

};

Wrap.prototype.callback = function(args, pos, actBefore, actAfter,actErr) {
    var self = this;

    if(!args) return false;

    if(args.length <= pos) return false;
    if(pos === -1) pos = args.length - 1;

    var orig = (typeof args[pos] === 'function') ? args[pos] : undefined;
    if(!orig) return false;

    args[pos] = function bonr_wrap() {
        var locals = new Locals();
        if(actBefore) {
            try {
                actBefore(this, arguments);
            }
            catch(e) {
                self.logError(e);
            }
        }

        // var ret = orig.apply(this, arguments);
        var ret;
        try {
            ret = orig.apply(this, arguments);
        }catch(e)
        {
            locals.error = e;
            if(actErr)
            {
                try {
                    actErr(this, arguments, locals);
                }catch (err)
                {
                    self.logError(err);
                }
            }
            throw e;
        }
        if(actAfter) {
            try {
                actAfter(this, arguments, ret);
            }
            catch(e) {
                self.logError(e);
            }
        }
        return ret;
    };

    if(self.isShellWrapEnabled)
    {
        var shell = self.shellWrap(args[pos]);
        if(shell!==undefined)
        {
            args[pos] = shell;
        }
    }

    // this is needed for removeListener
    orig.__bonreeWrap__ = args[pos];
    return true;
};

function isPromiseSupported() {
    if (parseInt(process.versions.node.split('.')[0], 10) === 0 && parseInt(process.versions.node.split('.')[1], 10) < 12)
        return false;
    return true;
}

/**
 * Helper function that chains promiseHook after returnVal if returnVal is a promise.
 */
function promise(returnVal, promiseHook, obj, methodArgs, locals) {
    // For unsupported node versions return.
    if (!isPromiseSupported())
        return;
    // A Promise is always 'thenable'.
    // "thenable" is an object or function that defines a then method.
    // Here is thread, with a discussion on how to determine if an Object is a Promise
    // https://stackoverflow.com/questions/27746304/how-do-i-tell-if-an-object-is-a-promise
    if (!returnVal || typeof (returnVal.then) !== 'function')
        return;

    // For Promises
    return returnVal.then(function resolve(data) {
        let promiseHookRet = promiseHook(obj, methodArgs, {
            __appdynamicsIsPromiseResult__: true,
            error: null,
            data: data
        }, locals);

        // If promiseHook returns a promise, chain it.
        if (promiseHookRet && typeof (promiseHookRet.then) === 'function') {
            return promiseHookRet.then(() => {
                return data;
            }).catch(promiseHookErr => {
                //TODO: is it okay to use the logger instance directly or should I pass this as one of the arguments?
                self.logError(promiseHookErr);
                return data;
            });
        }

        return data;
    }).catch(function (err) {
        let promiseHookRet = promiseHook(obj, methodArgs, {
            __appdynamicsIsPromiseResult__: true,
            error: err || 'AppDPromiseError', // Put the default value for the error, so that check on error property passes
            data: null
        }, locals);

        // If promiseHook returns a promise, chain it.
        if (promiseHookRet && typeof (promiseHookRet.then) === 'function') {
            let promiseHookSucceeded = false;

            return promiseHookRet.then(() => {
                promiseHookSucceeded = true;
                return Promise.reject(err);
            }).catch(promiseHookErr => {
                // This check is needed so that we don't extraneously log the error thrown in the previous then block
                if (!promiseHookSucceeded) {
                    self.logError(promiseHookErr);
                }
                return Promise.reject(err);
            });
        }
        return Promise.reject(err);
    });
};
function callConstructorFunction(functionLogic, fnArgs) {
    switch (fnArgs.length) {
        case 0:
            return new functionLogic();
        case 1:
            return new functionLogic(fnArgs[0]);
        case 2:
            return new functionLogic(fnArgs[0], fnArgs[1]);
        case 3:
            return new functionLogic(fnArgs[0], fnArgs[1], fnArgs[2]);
        case 4:
            return new functionLogic(fnArgs[0], fnArgs[1], fnArgs[2], fnArgs[3]);
        case 5:
            return new functionLogic(fnArgs[0], fnArgs[1], fnArgs[2], fnArgs[3], fnArgs[4]);
        case 6:
            return new functionLogic(fnArgs[0], fnArgs[1], fnArgs[2], fnArgs[3], fnArgs[4], fnArgs[5]);
        case 7:
            return new functionLogic(fnArgs[0], fnArgs[1], fnArgs[2], fnArgs[3], fnArgs[4], fnArgs[5], fnArgs[6]);
        case 8:
            return new functionLogic(fnArgs[0], fnArgs[1], fnArgs[2], fnArgs[3], fnArgs[4], fnArgs[5], fnArgs[6], fnArgs[7]);
        case 9:
            return new functionLogic(fnArgs[0], fnArgs[1], fnArgs[2], fnArgs[3], fnArgs[4], fnArgs[5], fnArgs[6], fnArgs[7], fnArgs[8]);
        case 10:
            return new functionLogic(fnArgs[0], fnArgs[1], fnArgs[2], fnArgs[3], fnArgs[4], fnArgs[5], fnArgs[6], fnArgs[7], fnArgs[8], fnArgs[9]);
        default:
            // logger.warn('Experienced a high arity function with arity of ', fnArgs.length);
            console.log('Experienced a high arity function with arity of ', fnArgs.length);
            return new functionLogic(fnArgs);
    }
}

function callOrigFunction(context, origFn, modifiedFn, fnInvokeArgs) {
    if (context && context.constructor && context.constructor === modifiedFn)
        return callConstructorFunction(origFn, fnInvokeArgs);
    else
        return origFn.apply(context, fnInvokeArgs);
}

Wrap.prototype.aroundpromise = function(obj, meths, hookBefore, hookAfter) {
  var self = this;
  if (!obj) return false;

    if (!Array.isArray(meths)) meths = [meths];

    meths.forEach(function (meth) {
        var orig = obj[meth];
        if (!orig) return;

        obj[meth] = function () {
            try {
                var locals = new Locals();
                locals.methodName = meth;
                try {
                    hookBefore(this, arguments, locals);
                }
                catch (e) {
                    self.logError(e);
                }

                try {
                    var ret = callOrigFunction(this, orig, obj[meth], arguments);
                } catch (e) {
                    var ret = {
                        "error": e,
                        "isLambdaError": true
                    };

                }

                var promiseRet = promise(ret, hookAfter, this, arguments, locals);
                if (promiseRet) {
                    return promiseRet;
                }

                var hookRet;
                try {
                    hookRet = hookAfter(this, arguments, ret, locals);
                }
                catch (e) {
                    self.logError(e);
                }

                if (hookRet) {
                    return hookRet;
                }

                if (ret) {
                    if (ret.error && ret.isLambdaError) {
                        throw ret
                    }
                    return ret;
                }
            } catch (e) {
                self.logError(e);
                //If the originating lambda threw an error, we still need to make the lambda throw an error instead of swallowing it in the around hook.
                if (e.isLambdaError && e.error) {
                    self.logError("Error occurred in lambda. This will cause the Node.js process to exit and hence Appdynamics will not able to report metrics.");
                    throw e.error;
                }
            }
        };

        obj[meth].__appdynamicsProxyInfo__ = {
            obj: obj,
            meth: meth,
            orig: orig
        };
    });
};


Wrap.prototype.callbackPromiseChain = function (args, pos, actBefore, actAfter) {
    if (!args) return false;

    if (args.length <= pos) return false;
    if (pos === -1) pos = args.length - 1;

    // orig is the callback function that is passed to handler
    let orig = (typeof args[pos] === 'function') ? args[pos] : undefined;
    if (!orig) return false;

    args[pos] = function bonr_wrap() {
        let callbackThis = this;
        let callbackArgs = arguments;
        let hbPromise = new Promise((resolve, reject) => {
            let hbReturnsPromise = false;
            if (actBefore) {

                try {
                    let hookBeforeRet = actBefore(callbackThis, callbackArgs);

                    // Check that hookBefore returns a promise
                    if (hookBeforeRet && typeof (hookBeforeRet.then) === 'function') {
                        hbReturnsPromise = true;
                        hookBeforeRet.then(() => {
                            resolve();
                        }).catch(hookBeforeErr => {
                            self.logError(hookBeforeErr);
                            resolve();
                        });
                    }

                } catch (e) {
                    self.logError(e);
                }
            }

            if (!hbReturnsPromise) {
                resolve();
            }
        }).then(() => {
            return Promise.resolve(orig.apply(callbackThis, callbackArgs));
        }).then(origRet => {
            if (actAfter) {
                try {
                    let hookAfterRet = actAfter(callbackThis, callbackArgs, origRet);

                    if (hookAfterRet && typeof (hookAfterRet.then) === 'function') {
                        return hookAfterRet.then(() => {
                            return origRet;
                        }).catch(hookAfterErr => {
                            self.logError(hookAfterErr);
                            return origRet;
                        });
                    }
                } catch (e) {
                    self.logError(e);
                }
            }
            return origRet;
        });

        return hbPromise;
    };

    // this is needed for removeListener
    orig.__bonreeWrap__ = args[pos];
    return true;
}

Wrap.prototype.getter = function(obj, props, hook) {
    var self = this;

    if(!Array.isArray(props)) props = [props];

    props.forEach(function(prop) {
        var orig = obj.__lookupGetter__(prop);
        if(!orig) return;

        obj.__defineGetter__(prop, function() {
            var ret = orig.apply(this, arguments);

            try {
                hook(this, ret);
            }
            catch(e) {
                self.logError(e);
            }

            return ret;
        });
    });
};

Wrap.prototype.getErrorObject = function(args) {
    if(args && args.length > 0 && args[0]) {
        if(typeof(args[0]) === 'object' || typeof(args[0]) === 'string') {
            return args[0];
        }
        else {
            return 'unspecified';
        }
    }

    return undefined;
};

Wrap.prototype.logError = function(err) {
    this.agent.logger.error(err.stack);
};
