'use restrict';

var util = require('util');
var URL = require('url');
var BackendType =  require('../common/constant').BackendType;
var MQDIRECTION = require('../common/constant').MQDirection;


var channelMethodsWithNoExitcall = [
    'close',
    'open',
    'waitForConfirms'
];
var channelMethodsWithExitcall = [
    'publish',
    'assertQueue',
    'checkQueue',
    'deleteQueue',
    'purgeQueue',
    'bindQueue',
    'unbindQueue',
    'assertExchange',
    'checkExchange',
    'deleteExchange',
    'bindExchange',
    'unbindExchange',
    'sendToQueue',
    'consume',
    'cancel',
    'get',
    'reject',
    'prefetch',
    'recover'
];
  

exports.AmqplibHook = AmqplibHook;

function parseConnection(connOpts) {
    var url = connOpts || 'amqp://localhost';
    var protocol,host,user,passwd;
    if (typeof url === 'object') {
        protocol = (url.protocol || 'amqp:');
        host = url.hostname;
        port = url.port || ((protocol === 'amqp:') ? 5672 : 5671);
        if (url.username === undefined && url.password === undefined) {
            user = 'guest'; pass = 'guest';
        } else {
            user = url.username || '';
            pass = url.password || '';
        }
    }
    else
    {
        var parts = URL.parse(url, true);
        protocol = parts.protocol;
        host = parts.hostname;
        port = parseInt(parts.port) || ((protocol === 'amqp:') ? 5672 : 5671);
    }
    return {
        'HOST': host,
        'PORT': port,
        'VENDER': 'AMQP',
        'PROTOCOL': protocol.substring(0,protocol.lastIndexOf(':')),
        'USER': user || 'guest',
        'PASSWD': passwd || 'guest',
        'URL': url,
    };
}

//
function AmqplibHook(agent) {
    this.agent = agent;
    this.baseHook = agent.baseHook;
    this.keys = ['./lib/connect','./lib/channel_model','./lib/callback_model'];
    this.parentPackage = ['amqplib/channel_api','amqplib/callback_api'];
}
//
exports.AmqplibHook = AmqplibHook;
AmqplibHook.prototype.match = function (parentid) {
    var self = this;
    for(var pkg in self.parentPackage)
    {
        if(parentid.replace(/\\/g, '\/').indexOf(self.parentPackage[pkg]) !== -1)
        {
            return true;
        }
    }
    return false;
};

AmqplibHook.prototype._attachconnect = function (obj) {
    var self = this;

    if(obj.__bonrAttached__) return;
    obj.__bonrAttached__ = true;

    var logger = self.agent.logger;
    var wrap = self.agent.wrap;
    var profiler = self.agent.profiler;
    var localBaseHook = self.baseHook;

    localBaseHook.wrapAsyncMethod(obj,'amqplib.connect',null,function (obj,args,locals) {

        if(args.length>=3)
        {
            var txn = self.agent.tracer.getTransaction();
            if(!txn) {
                return;
            }
            locals.parentSegment = txn.getActiveSegment();
            var supportedProperties = parseConnection(args[0]);
            wrap.callback(args,2,function (obj,cbargs) {
                    if (cbargs[0] && txn) {
                        var err = cbargs[0];
                        if(err){
                            err.name='amqplib';
                        }
                        txn.addException(err);
                        return;
                    }
                    if(cbargs.length>1)
                    {
                        cbargs[1]._bonrConOption = supportedProperties;
                    }
                }
            );
            locals.exitCallInfo = {
                backendType: BackendType.BACKEND_TYPE_RABBITMQ,
                name: supportedProperties.URL,
                host: supportedProperties.HOST,
                port: supportedProperties.PORT,
                operation: 'connect',
                url: supportedProperties.URL,
                connType: supportedProperties.PROTOCOL,
                is_async: false
            };
        }
    });

};

AmqplibHook.prototype._attachchannel = function (obj) {
    var self = this;
    
    if(obj.__bonrAttached__) return;
    obj.__bonrAttached__ = true;
    
    var localBaseHook = self.baseHook;
    var createMethods = ['createChannel','createConfirmChannel'];
    createMethods.forEach(function(value) {
        localBaseHook.wrapSyncMethod(obj.prototype, 'ChannelModel.'+value, null, function(obj, args, ret) {
            if (ret.__proto__._bonreeThenWrap) return;
            self.agent.wrap.before(ret.__proto__, 'then', function(obj, args) {
                if (args.length < 1) return;
                var wrapCb = {};
                wrapCb.wrapBefore = function(obj, args) {
                    if (args.length < 1) return;
                    var ch = args[0];
                    if (!ch || !ch.connection || !ch.connection._bonrConOption) return;
                    channelMethodsWithNoExitcall.forEach(function(method) {
                        localBaseHook.wrapAsyncMethod(ch, 'Channel.' + method);
                    });
                    channelMethodsWithExitcall.forEach(function(method) {
                        localBaseHook.wrapAsyncMethod(ch, 'Channel.'+method, null, function(obj, args, locals) {
                            var txn = self.agent.tracer.getTransaction();
                            if (txn) {
                                locals.parentSegment = txn.getActiveSegment();
                            }

                            var bonrConOption = obj.connection._bonrConOption;
                            if (txn && localBaseHook.needExitCall(txn)) {
                                var mqInfo;

                                if(method == 'sendToQueue') {
                                    let content = args[1];
                                    mqInfo = {
                                        size: content && Buffer.byteLength(content) || 0,
                                        direction: MQDIRECTION.PRODUCER
                                    }
                                }
                                locals.exitCallInfo = {
                                    backendType: BackendType.BACKEND_TYPE_RABBITMQ,
                                    name: bonrConOption.URL,
                                    host: bonrConOption.HOST,
                                    port: bonrConOption.PORT,
                                    operation: method,
                                    url: bonrConOption.URL,
                                    connType: bonrConOption.PROTOCOL,
                                    is_async: false,
                                    mqInfo: mqInfo
                                };
                                //locals.exitCall = self.agent.profiler.createExitCall(exitCallInfo, txn);

                                if (method == 'consume') {
                                    if(args.length <1) {
                                        return;
                                    }
                                    // var wrapCb = {};
                                    locals.wrapBefore = function(obj, args) {
                                        if(!!locals.curExitCall) {
                                            let content = args[0].content;
                                            mqInfo = {
                                                size: content && Buffer.byteLength(content) || 0,
                                                direction: MQDIRECTION.CONSUMER
                                            }
                                            locals.curExitCall.setMQInfo(mqInfo);
                                        }
                                    }
                                    // locals.hasCallback = true;
                                    // args[1] = localBaseHook.bindFunction(args[1], null, true, wrapCb);
                                }
                            }
                        }, function(obj, args, ret, locals) {
                            if (!ret  || !ret.__proto__ || ret.__proto__.constructor.name !== 'Promise') {
                                return;
                            }
                            var origThen = ret.__proto__.then;
                            self.agent.wrap.before(ret.__proto__, 'then', function(obj, args) {
                                if (args.length < 1) return;
                                var wrapCb = {};
                                wrapCb.wrapBefore = function(obj, args) {//mark 此处存在疑惑
                                    if (locals.curExitCall) {
                                        locals.curExitCall.duration_time = (self.agent.system.hrtime() - locals.curExitCall.ts) / 1000;
                                    }
                                };
                                args[0] = localBaseHook.bindFunction(args[0], null, true, wrapCb);
                                ret.__proto__.then = origThen;
                            });
                            self.agent.wrap.before(ret.__proto__, 'catch', function(obj, args) {
                                if (args.length < 1) return;
                                var txn = self.agent.tracer.getTransaction();
                                var wrapCb = {};
                                wrapCb.wrapBefore = function(obj, args) {
                                    if (txn && args.length > 0 && args[0]) {
                                        var err = args[0];
                                        if(err){
                                            err.name='amqplib';
                                        }
                                        txn.addException(err);
                                    }
                                };
                                args[0] = localBaseHook.bindFunction(args[0], null, true, wrapCb);
                            });
                        });
                    });
                };
                args[0] = localBaseHook.bindFunction(args[0], null, true, wrapCb);
            });
            self.agent.wrap.before(ret.__proto__, 'catch', function(obj, args) {
                if (args.length < 1) return;
                var txn = self.agent.tracer.getTransaction();
                var wrapCb = {};
                wrapCb.wrapBefore = function(obj, args) {
                    if (txn && args.length > 0 && args[0]) {
                        var err = args[0];
                        if(err){
                            err.name='amqplib';
                        }
                        txn.addException(err);
                    }
                };
                args[0] = localBaseHook.bindFunction(args[0], null, true, wrapCb);
            });
            ret.__proto__._bonreeThenWrap = true;
        });
    });
};

AmqplibHook.prototype._attachCallbackChannel = function (obj) {
    var self = this;

    if(obj.__bonrAttached__) return;
    obj.__bonrAttached__ = true;

    var localBaseHook = self.baseHook;
    var createMethods = ['createChannel','createConfirmChannel'];
    createMethods.forEach(function(value) {
        localBaseHook.wrapAsyncMethod(obj.prototype, 'CallbackModel.'+value, null, null, function(obj, args, ret) {
            var ch = ret;
            if (!ch || !ch.connection || !ch.connection._bonrConOption) return;
            channelMethodsWithNoExitcall.forEach(function(method) {
                localBaseHook.wrapAsyncMethod(ch, 'Channel.' + method);
            });
            channelMethodsWithExitcall.forEach(function(method) {
                var cbId = (method == 'consume' ? 1 : null);
                localBaseHook.wrapAsyncMethod(ch, 'Channel.'+method, null, function(obj, args, locals) {
                    var txn = self.agent.tracer.getTransaction();
                    locals.parentSegment = txn.getActiveSegment();

                    obj._txn = txn;
                    var bonrConOption = obj.connection._bonrConOption;
                    if (txn && localBaseHook.needExitCall(txn)) {
                        var content;
                        var mqInfo;
                        if(method == 'sendToQueue') {
                            content = args[1];
                            mqInfo = {
                                size: content && Buffer.byteLength(content) || 0,
                                direction: MQDIRECTION.PRODUCER
                            }
                        }
                        if(method == 'consume') {
                            if (args.length >= 2 && typeof  args[1] === 'function') {
                                var cbWrap = {};
                                cbWrap.wrapBefore = function(obj, args) { //hook fun(msg)
                                    if (args.length >= 1 && !!args[0] ) {
                                        var content = args[0].content;
                                        mqInfo = {
                                            size: content && Buffer.byteLength(content) || 0,
                                            direction: MQDIRECTION.CONSUMER
                                        }
                                        if(!!locals.curExitCall) {
                                            locals.curExitCall.setMQInfo(mqInfo);
                                        }
                                    }
                                };
                                args[1] = localBaseHook.bindFunction(args[1], null, true, cbWrap);
                            };
                        }

                        locals.exitCallInfo = {
                            backendType: BackendType.BACKEND_TYPE_RABBITMQ,
                            name: bonrConOption.URL,
                            host: bonrConOption.HOST,
                            port: bonrConOption.PORT,
                            operation: method,
                            url: bonrConOption.URL,
                            connType: bonrConOption.PROTOCOL,
                            is_async: false,
                            mqInfo: mqInfo
                        };

                    }
                    if (method != 'consume') {
                        locals.wrapBefore = function(obj, args) {
                            if (args.length > 1 && args[0] && txn) {
                                var err = args[0];
                                if(err){
                                    err.name='amqplib';
                                }
                                txn.addException(err);
                            }
                        };
                    } else {
                        var cbId = args.length - 1;
                        if (cbId > 1 && typeof args[cbId] === 'function') {
                            var cbWrap = {};
                            cbWrap.wrapBefore = function(obj, args) {
                                if (args.length > 1 && args[0] && txn) {
                                    var err = args[0];
                                    if(err){
                                        err.name='amqplib';
                                    }
                                    txn.addException(err);
                                }
                            };
                            args[cbId] = localBaseHook.bindFunction(args[cbId], null, true, cbWrap);
                        }
                    }
                });
            });
        });
    });
};

AmqplibHook.prototype.attach = function (obj,module) {
    var self = this;
    switch (module)
    {
        case './lib/connect':
        {
            self._attachconnect(obj);
            break;
        }
        case './lib/channel_model':
        {
            self._attachchannel(obj.ChannelModel);
            break;
        }
        case './lib/callback_model':
        {
            self._attachCallbackChannel(obj.CallbackModel);
            break;
        }
    }
};