'use strict';

function BluebirdHook(agent) {
    this.agent = agent;
    this.baseHook = agent.baseHook;
    this.packages = ['bluebird'];
}
exports.BluebirdHook = BluebirdHook;

var syncMethods = [
    'PromiseInspection',
    'isFulfilled',
    'isRejected',
    'isPending',
    'isCancelled',
    'value',
    'reason',
    'resolve',
    'reject',
    'all',
    'props',
    'any',
    'some',
    'delay',
    'return',
    'thenReturn',
    'throw',
    'thenThrow',
    'reflect',
    'noConflict',
    'call',
    'get',
    'reflect'
];

var asyncMethods = [
    'then',
    'spread',
    'error',
    'catch',
    'finally',
    'map',
    'filter',
    'each',
    'mapSeries',
    'disposer',
    'catchReturn',
    'catchThrow'
];

var promiseSyncMethods = [
    'resolve',
    'reject',
    'race',
    'delay',
    'timeout',
    'promisify',
    'promisifyAll',
    'noConflict'
];

var promiseAsyncMethods = [
    'join',
    'try',
    'method',
    'map',
    'reduce',
    'filter',
    'each',
    'mapSeries',
    'using',
    'fromCallback',
    'asCallback',
    'setScheduler'
];

BluebirdHook.prototype.attach = function(obj) {
    var baseHook = this.baseHook;
    syncMethods.forEach(function(method) {
        baseHook.wrapSyncMethod(obj.prototype, 'bluebird.' + method);
    });
    asyncMethods.forEach(function(method) {
        baseHook.wrapAllCallback(obj.prototype, 'bluebird.' + method);
    });

    promiseSyncMethods.forEach(function(method) {
        baseHook.wrapSyncMethod(obj, 'Promise.' + method);
    });
    promiseAsyncMethods.forEach(function(method) {
        baseHook.wrapAllCallback(obj, 'Promise.' + method);
    });
};
