'use strict';

function BaseHook(agent) {
    this.agent = agent;
}

exports.BaseHook = BaseHook;

//创建后端与trace节点
BaseHook.prototype.dealWithLocalsBefore = function(locals, methodName,parentSegment) {
    var localAgent = this.agent;
    var segment = localAgent.tracer.createSegment(methodName, parentSegment, false, locals.needTrace);
    if (!segment) return null;
    segment.reuse = locals.reuseParent;

    if(!segment.getNodeInfo()) return null;

    segment.start();
    localAgent.tracer.curSegment = segment;
    var hasExitCall;
    if (!!locals.exitCallInfo) {
        if (!!locals.shouldIgnore && locals.shouldIgnore(segment)) {//segment与后端过于耦合，影响性能，展示屏蔽，后续寻找其他途径获取屏蔽mangodb的方案
            hasExitCall = false;
        } else {
            hasExitCall = true;
            locals.curExitCall = localAgent.profiler.createExitCall(locals.exitCallInfo, locals.transaction);
        }
        locals.exitCallInfo = null;
    } else if (locals.exitCallInfo === null) {
        hasExitCall = false;
    }

    //In view of the delayed creation of the backend,
    // exitCall tags can be reserved in the local parameters used at hook.
    segment._hasExitCall = locals.hasExitCall || hasExitCall;

    return segment;
};

BaseHook.prototype.dealWithLocalsBeforewithoutSeg = function(locals, methodName) {

    var hasExitCall = false;
    if (!!locals.exitCallInfo) {
        var curStackInfo = __stack;
        for (var elem in curStackInfo) {

            if (process.version < 'v7.0.0') {
                if (!curStackInfo[elem] || !curStackInfo[elem].getFileName) {
                    continue;
                }
            } else {
                if (!curStackInfo[elem] || !curStackInfo[elem].__proto__ ||
                    !curStackInfo[elem].__proto__.getFileName) {
                    continue;
                }
            }
            var filename = curStackInfo[elem].getFileName();
            if (!!locals.shouldIgnore
                && (filename.indexOf('/node_modules/mongodb/') != -1
                    || filename.indexOf('/node_modules/mongodb-core/') != -1))
            {
                continue;
            }
            hasExitCall = true;
            locals.curExitCall = this.agent.profiler.createExitCall(locals.segTime, locals.exitCallInfo, locals.transaction);
        }
    } else if (locals.exitCallInfo === null) {
        hasExitCall = false;
    }
    locals.exitCallInfo = null;
}

BaseHook.prototype.dealWithLocalsAfter = function(locals) {
    var localAgent = this.agent;
    if (!!locals.exitCallInfo) {
        locals.curExitCall = localAgent.profiler.createExitCall(locals.exitCallInfo, locals.transaction);
        locals.exitCallInfo = null;
    }

    if (!!locals.curExitCall && !locals.hasCallback) {
        this.agent.profiler.addExitCall(null, locals.curExitCall, locals.transaction);
        locals.curExitCall._added = true;
    }
};

BaseHook.prototype.dealWithLocalsError = function(locals) {
    var localAgent = this.agent;
    if (!!locals.exitCallInfo) {
        locals.curExitCall = localAgent.profiler.createExitCall(locals.exitCallInfo, locals.transaction);
        locals.exitCallInfo = null;
    }
    if (!!locals.curExitCall) {
        this.agent.profiler.addExitCall(locals.error,locals.curExitCall,locals.transaction);
    }
};

BaseHook.prototype.dealWithLocalsInCb = function(locals) {
    var localAgent = this.agent;
    if (!!locals.exitCallInfo) {
        locals.curExitCall = localAgent.profiler.createExitCall(locals.exitCallInfo, locals.transaction);
        locals.exitCallInfo = null;
    }
    if (!!locals.curExitCall && !!locals.hasCallback) {
            if (!locals.curExitCall._added) {
                this.agent.profiler.addExitCall(null,locals.curExitCall,locals.transaction);
            }
            locals.curExitCall.duration_time = (this.agent.system.hrtime() - locals.curExitCall.ts) / 1000;
            locals.curExitCall = null;
    }
    if (!!locals.error) {
        locals.transaction.addException(locals.error);
    }
};

BaseHook.prototype.needExitCall = function(txn) {
    var parentSegment = this.agent.tracer.getParentSegment(txn);
    if (!parentSegment) {
        //9.20  add exitCall yi hou tiao shi you wen ti zai gai wei false;
        return true;
    }
    if (!parentSegment.hasOwnProperty('_hasExitCall') || parentSegment._hasExitCall == undefined) {
        return true;
    }

    return false;
};

BaseHook.prototype.wrapSyncMethod = function(obj, methodName, wrapBefore, wrapAfter, wrapErr) {
    var wrapKey = '_bonrWraped_' + methodName + '_sync';
    if((typeof obj !== 'object' && typeof obj !== 'function') ||
        obj.hasOwnProperty(wrapKey)) {
        return;
    }
    var self = this;
    var localAgent = self.agent;
    var segment;
    var parentSegment;
    var clsMethod = methodName.split('.');
    var realMethodNm = clsMethod.length > 1 ? clsMethod[1] : methodName;

    localAgent.wrap.around(obj, [realMethodNm], function(obj, args, locals) {//actBefore
        if(!!wrapBefore) wrapBefore(obj, args, locals);
        parentSegment = locals.parentSegment || localAgent.tracer.getParentSegment();
        if (!parentSegment) {
            return;
        }
        segment = self.dealWithLocalsBefore(locals, methodName, parentSegment);
    }, function(obj, args, ret, locals) {//actAfter
        if(!!wrapAfter) wrapAfter(obj, args, ret, locals);
        if(!!segment) self.dealWithLocalsAfter(locals);
        if(!!segment) {
            segment.done();
            if (!locals.reuseParent && !!parentSegment)
                localAgent.tracer.curSegment = parentSegment;
        }
    }, function(obj, args, locals) { //actErr
        if (!!wrapErr) wrapErr(obj, args, locals);
        self.dealWithLocalsError(locals);
    });
    obj[wrapKey] = true;
};

BaseHook.prototype.wrapAsyncMethod = function(obj, methodName, cbId, wrapBefore, wrapAfter, wrapErr) {
    var wrapKey = '_bonrWraped_' + methodName + '_async';
    if((typeof obj !== 'object' && typeof obj !== 'function') ||
        obj.hasOwnProperty(wrapKey)) {
        return;
    }

    var localAgent = this.agent;
    var parentSegment, segment;
    var self = this;
    var clsMethod = methodName.split('.');
    var realMethodNm = clsMethod.length > 1 ? clsMethod[1] : methodName;

    localAgent.wrap.around(obj, [realMethodNm], function(obj, args, locals) {//actbefore
        if(!!wrapBefore) wrapBefore(obj, args, locals);
        parentSegment = locals.parentSegment || localAgent.tracer.getParentSegment();
        if(!parentSegment) {
            return;
        }
        segment = self.dealWithLocalsBefore(locals, methodName, parentSegment);
        if (!segment) return;

        if (args.length < 1) return;
        var orig, curCbId;
        if(cbId == 0 || !!cbId) {
            curCbId = cbId;
        } else {
            curCbId = args.length - 1;
        }
        orig = args[curCbId];
        if (typeof orig === 'function') {
            locals.hasCallback = true;
            args[curCbId] = self.bindFunction(orig, segment, true, locals, wrapErr);
        }

    }, function(obj, args, ret, locals) {//actafter
        if (!!wrapAfter) wrapAfter(obj, args, ret, locals);
        if (!!segment) self.dealWithLocalsAfter(locals);
        if (!!segment) {
            segment.done();
            if (!locals.reuseParent && !!parentSegment)
                localAgent.tracer.curSegment = parentSegment;
        }
    }, function(obj, args, locals) {
        if (!!wrapErr) wrapErr(obj, args, locals);
        self.dealWithLocalsError(locals);
    });
    obj[wrapKey] = true;
};

BaseHook.prototype.wrapAsyncMethodWithNoSegment = function(obj, methodName, cbId) {
    var wrapKey = '_bonrWraped_' + methodName + '_noseg';
    if((typeof obj !== 'object' && typeof obj !== 'function') ||
        obj.hasOwnProperty(wrapKey)) {
        return;
    }
    var localAgent = this.agent;

    localAgent.wrap.before(obj, [methodName], function(obj, args) {

        if (args.length < 1) return;
        var parentSegment = localAgent.tracer.getParentSegment();//localAgent.tracer.curSegment;
        if (!parentSegment) return;
        var orig, curCbId;
        if(cbId == 0 || !!cbId) {
            curCbId = cbId;
        } else {
            curCbId = args.length - 1;
        }
        orig = args[curCbId];
        var cbWrapKey = '_bonrWraped_' + methodName + '_noseg_callback';
        if (typeof orig !== 'function' || orig.hasOwnProperty(cbWrapKey)) {
            return;
        }

        args[curCbId] = function bonr_wrap() {
            var ret;
            try {
                ret = orig.apply(this, arguments);
            }catch(e)
            {
                //time.done();
                //if (!!time.ms && !!parentSegment.traceNode) {
                    //parentSegment.traceNode.exeTime += time.ms;
                //}
                throw e;
            }
            //time.done();
            // if (!!time.ms && !!parentSegment.traceNode) {
            //     parentSegment.traceNode.exeTime += time.ms;
            // }
            return ret;
        };
        args[curCbId][cbWrapKey] = true;
    });

    obj[wrapKey] = true;
};

BaseHook.prototype.bindFunction = function(obj, parent, isCallback, locals) {
    var wrapKey = '_bonrWraped_bindfunc';
    if (typeof obj !== 'function' || obj.hasOwnProperty(wrapKey)) return obj;
    var orig = obj;
    var self = this;
    var localAgent = self.agent;

    var wrapped = function() {
        var segment;
        var methodName;

        if (!!locals && typeof locals === 'object' && locals.hasOwnProperty('segmentName') && !!locals.segmentName) {
            methodName = locals.segmentName;
        } else {
            methodName = obj.name || '<anonymouse>';
        }

        var args = arguments;
        var parentSegment = parent || localAgent.tracer.getParentSegment(obj._txn);
        if (args.length > 0 && args[0] && args[0]._txn && args[0]._tag === 'koa') {
            var txn = args[0]._txn;
            parentSegment = txn.baseSegment;
        }

        if (!!parentSegment && !!parentSegment.transaction) {
            segment = localAgent.tracer.createSegment(methodName, parentSegment, isCallback);
            if (!!segment) {
                segment.reuse = !!locals && typeof locals === 'object' && locals.reuseParent;
                if(segment.getNodeInfo()) {
                    segment.start();
                    localAgent.tracer.curSegment = segment;
                } else {
                    segment = null;
                }
                if (orig.__proto__.constructor.name == 'AsyncFunction') {
                    localAgent.tracer.curAsyncSegment = segment;
                }
            }
        }
        if (!!locals && typeof locals === 'object' && locals.hasOwnProperty('wrapBefore') && !!locals.wrapBefore) {
            locals.wrapBefore(this, arguments);
        }
        if (!!locals && typeof locals === 'object') self.dealWithLocalsInCb(locals);

        var ret;
        try {
            ret = orig.apply(this, arguments);
        } catch (e) {
            if (!!segment && !!segment.transaction) segment.transaction.addException(e);
            if (!!segment) segment.done();
            if(!!parentSegment) {
                localAgent.tracer.curSegment = parentSegment;
            }
            throw e;
        }

        if (!!segment) segment.done();
        if (!!parentSegment) localAgent.tracer.curSegment = parentSegment;

        if (!!locals && typeof locals === 'object' && locals.hasOwnProperty('wrapAfter') && !!locals.wrapAfter) {
            locals.wrapAfter(this, arguments);
        }

        return ret;
    };
    wrapped[wrapKey] = true;


    return wrapped;
};

BaseHook.prototype.wrapAllCallback = function(obj, methodName, wrapBefore, wrapAfter, wrapErr) {
    var wrapKey = '_bonrWraped_' + methodName + '_callbacks';
    if((typeof obj !== 'object' && typeof obj !== 'function') ||
        obj.hasOwnProperty(wrapKey)) {
        return;
    }

    var localAgent = this.agent;
    var parentSegment, segment;
    var self = this;
    var clsMethod = methodName.split('.');
    var realMethodNm = clsMethod.length > 1 ? clsMethod[1] : methodName;

    localAgent.wrap.around(obj, [realMethodNm], function(obj, args, locals) {
        if (args.length < 1) return;
        if(!!wrapBefore) wrapBefore(obj, args, locals);
        parentSegment = localAgent.tracer.getParentSegment();
        if (!parentSegment) return;
        var cbIds = [];
        for (var i = 0; i < args.length; i++) {
            if (typeof args[i] === 'function') {
                cbIds.push(i);
            }
        }

        if(!!parentSegment && !!parentSegment.transaction) {
            segment = localAgent.tracer.createSegment(methodName, parentSegment, true);
            if (!segment) return;
            if (!segment.getNodeInfo()) {
                segment = null;
                return;
            }
            segment.start();
        }
        cbIds.forEach(function(cbId) {
            args[cbId] = self.bindFunction(args[cbId], segment, true, locals, wrapErr);
        });
    }, function(obj, args, ret, locals) {
        if (!!segment) {
            segment.done();

            if(!!parentSegment) localAgent.tracer.curSegment = parentSegment;
        }
        if (!!wrapAfter) wrapAfter(obj, args, ret, locals);
    }, function(obj, args, locals) {
        if (!!wrapErr) wrapErr(obj, args, locals);
    });
    obj[wrapKey] = true;
};

BaseHook.prototype.wrapAllCallbackWithArr = function(obj, methodName, wrapBefore, wrapAfter, wrapErr) {
    var wrapKey = '_bonrWraped_' + methodName + '_callbacks_with_arr';
    if((typeof obj !== 'object' && typeof obj !== 'function') ||
        obj.hasOwnProperty(wrapKey)) {
        return;
    }

    var localAgent = this.agent;
    var parentSegment, segment;
    var self = this;
    var clsMethod = methodName.split('.');
    var realMethodNm = clsMethod.length > 1 ? clsMethod[1] : methodName;

    localAgent.wrap.around(obj, [realMethodNm], function(obj, args, locals) {
        if (args.length < 1) return;
        if(!!wrapBefore) wrapBefore(obj, args, locals);
        parentSegment = localAgent.tracer.getParentSegment();
        if (!parentSegment) return;

        segment = localAgent.tracer.createSegment(methodName, parentSegment, true);
        if (!segment) return;
        if (!segment.getNodeInfo()) {
            segment = null;
            return;
        }
        segment.start();

        for (var i = 0; i < args.length; i++) {
            if (typeof args[i] === 'function') {
                args[i] = self.bindFunction(args[i], segment, true, locals, wrapErr);
            } else if (Array.isArray(args[i])) {
                for (var j = 0; j < args[i].length; j++) {
                    if (typeof args[i][j] === 'function') {
                        args[i][j] = self.bindFunction(args[i][j], segment, true, locals, wrapErr);
                    }
                }
            }
        }
    }, function(obj, args, ret, locals) {
        if (!!segment) {
            segment.done();
            if (!!parentSegment) localAgent.tracer.curSegment = parentSegment;
        }
        if (!!wrapAfter) wrapAfter(obj, args, ret, locals);
    }, function(obj, args, locals) {
        if (!!wrapErr) wrapErr(obj, args, locals);
    });
    obj[wrapKey] = true;
};

BaseHook.prototype.wrapWithTaskObj = function(obj, methodName, taskObjId, wrapBefore, wrapAfter, wrapErr) {
    var wrapKey = '_bonrWraped_' + methodName + '_taskobj';
    if((typeof obj !== 'object' && typeof obj !== 'function') ||
        obj.hasOwnProperty(wrapKey)) {
        return;
    }

    var localAgent = this.agent;
    var parentSegment, segment;
    var self = this;
    var clsMethod = methodName.split('.');
    var realMethodNm = clsMethod.length > 1 ? clsMethod[1] : methodName;

    localAgent.wrap.around(obj, [realMethodNm], function(obj, args, locals) {
        if (!taskObjId) taskObjId = 0;
        if (args.length < 1 || taskObjId >= args.length) return;
        if(!!wrapBefore) wrapBefore(obj, args, locals);
        parentSegment = localAgent.tracer.getParentSegment();
        if (!parentSegment) return;

        segment = localAgent.tracer.createSegment(methodName, parentSegment, true);
        if (!segment) return;
        if (!segment.getNodeInfo()) {
            segment = null;
            return;
        }
        segment.start();

        if (typeof args[taskObjId] === 'object') {
            for (var key in args[taskObjId]) {
                if (typeof args[taskObjId][key] === 'function') {
                    args[taskObjId][key] = self.bindFunction(args[taskObjId][key], segment, true, locals, wrapErr);
                } else if(Array.isArray(args[taskObjId][key])) {
                    var inerArr = args[taskObjId][key];
                    for (var i = 0; i < inerArr.length; i++) {
                        if (typeof inerArr[i] === 'function') {
                            inerArr[i] = self.bindFunction(inerArr[i], segment, true, locals, wrapErr);
                        }
                    }
                }
            }
        }
        for (let i = 0; i < args.length; i++) {
            if (i !== taskObjId && typeof args[i] === 'function') {
                args[i] = self.bindFunction(args[i], segment, true, locals, wrapErr);
            }
        }
    }, function(obj, args, ret, locals) {
        if (!!segment) {
            segment.done();
            if (!! parentSegment) localAgent.tracer.curSegment = parentSegment;
        }
        if (!!wrapAfter) wrapAfter(obj, args, ret, locals);
    }, function(obj, args, locals) {
        if (!!wrapErr) wrapErr(obj, args, locals);
    });
    obj[wrapKey] = true;
};

BaseHook.prototype.wrapWithTaskObjOrArr = function(obj, methodName, tasksId, wrapBefore, wrapAfter, wrapErr) {
    var wrapKey = '_bonrWraped_' + methodName + '_taskobj_or_arr';
    if((typeof obj !== 'object' && typeof obj !== 'function') ||
        obj.hasOwnProperty(wrapKey)) {
        return;
    }

    var localAgent = this.agent;
    var parentSegment, segment;
    var self = this;
    var clsMethod = methodName.split('.');
    var realMethodNm = clsMethod.length > 1 ? clsMethod[1] : methodName;

    localAgent.wrap.around(obj, [realMethodNm], function(obj, args, locals) {
        if (!tasksId) tasksId = 0;
        if (args.length < 1 || tasksId >= args.length) return;
        if(!!wrapBefore) wrapBefore(obj, args, locals);
        parentSegment = localAgent.tracer.getParentSegment();
        if (!parentSegment) return;

        segment = localAgent.tracer.createSegment(methodName, parentSegment, true);
        if (!segment) return;
        if (!segment.getNodeInfo()) {
            segment = null;
            return;
        }
        segment.start();

        if (typeof args[tasksId] === 'object') {
            for (var key in args[tasksId]) {
                if (typeof args[tasksId][key] === 'function') {
                    args[tasksId][key] = self.bindFunction(args[tasksId][key], segment, true, locals, wrapErr);
                }
            }
        } else if(Array.isArray(args[tasksId])) {
            for (var j = 0; j < args[tasksId].length; j++) {
                if (typeof args[tasksId][j] === 'function') {
                    args[tasksId][j] = self.bindFunction(args[tasksId][j], segment, true, locals, wrapErr);
                }
            }
        }
        for (var i = 0; i < args.length; i++) {
            if (i !== tasksId && typeof args[i] === 'function') {
                args[i] = self.bindFunction(args[i], segment, true, locals, wrapErr);
            }
        }
    }, function(obj, args, ret, locals) {
        if (!!segment) {
            segment.done();
            localAgent.tracer.curSegment = parentSegment;
        }
        if (!!wrapAfter) wrapAfter(obj, args, ret, locals);
    }, function(obj, args, locals) {
        if (!!wrapErr) wrapErr(obj, args, locals);
    });
    obj[wrapKey] = true;
};
