'use strict';

function NetHook(agent) {
    this.agent = agent;
    this.baseHook = agent.baseHook;
    this.packages = ['net'];
}

exports.NetHook = NetHook;

var servSyncMethods = [
    'address',
    'ref',
    'unref'
];
var servAsyncMethods = [
    'close',
    'getConnections',
    'listen'
];
var socketSyncMethods = [
    'address',
    'destroy',
    'end',
    'pause',
    'ref',
    'resume',
    'setEncoding',
    'setKeepAlive',
    'setNoDelay',
    'unref'
];
var socketAsyncMethods = [
    'connect',
    'setTimeout',
    'write'
];

NetHook.prototype.attach = function(obj, moduleName) {
    var self = this;
    var baseHook = this.baseHook;
    var methods = ['connect', 'createConnection'];
    methods.forEach(function(method) {
        baseHook.wrapAsyncMethod(obj, 'net.' + method, null, null, function(obj, args, ret, locals) {
            baseHook.wrapSyncMethod(ret, 'emit', function(obj, args, locals) {
                var txn = self.agent.tracer.getTransaction();
                if (!!txn) {
                    if (!txn._socket_emit_reptnum) txn._socket_emit_reptnum = 1;
                    else locals.reuseParent = true;
                }
            });//The asynchronous callback has been encapsulated, and is it necessary?

            socketSyncMethods.forEach(function(value) {
                baseHook.wrapSyncMethod(ret, 'Socket.' + value, function(obj, args, locals) {
                    var filterRepeats = ['setKeepAlive', 'setNoDelay'];
                    if (filterRepeats.indexOf(value) == -1) return;
                    var txn = self.agent.tracer.getTransaction();
                    if (!!txn) {
                        if (!txn['_socket_'+value+'_reptnum']) txn['_socket_'+value+'_reptnum'] = 1;
                        else locals.reuseParent = true;
                    }
                });
            });
            // socketAsyncMethods.forEach(function (value) {
            //     baseHook.wrapAsyncMethod(ret, 'Socket.' + value, null, function(obj, args, locals) {
            //         if (value !== 'setTimeout') return;
            //         var txn = self.agent.tracer.getTransaction();
            //         if (!!txn) {
            //             if (!txn['_socket_'+value+'_reptnum']) txn['_socket_'+value+'_reptnum'] = 1;
            //             else locals.reuseParent = true;
            //         }
            //     });
            // });
        });
    });

    servSyncMethods.forEach(function(value) {
        baseHook.wrapSyncMethod(obj.Server.prototype, 'Server.' + value);
    });
    servAsyncMethods.forEach(function (value) {
        baseHook.wrapAsyncMethod(obj.Server.prototype, 'Server.' + value);
    });
};