'use strict';

var util = require('util');
var BackendType =  require('../common/constant').BackendType;

function FullgoalRedisHook(agent) {
    this.agent = agent;
    this.packages = ['meap_redis_connection'];
    this.baseHook = agent.baseHook;
    this.connections = undefined;
}
exports.FullgoalRedisHook = FullgoalRedisHook;
FullgoalRedisHook.prototype.finishExitCall = function(socketObj,error,exitCalls) {
    var self = this;
    if(socketObj && exitCalls.length > 0)
    {
        var exitcall = exitCalls.shift();
        if(!exitcall || !exitcall.txn) return;
        var txn = exitcall.txn;
        exitcall.txn = null;
        self.agent.profiler.addExitCall(error,exitcall,txn);
    }
};
FullgoalRedisHook.prototype.attach = function(obj,name) {
    var self = this;

    if(obj.__bonrAttached__) return;
    obj.__bonrAttached__ = true;

    //var logger = self.agent.logger;
    var wrap = self.agent.wrap;
    var profiler = self.agent.profiler;
    var localBaseHook = self.baseHook;
    var exitcalls = [];
    self.connections = [];
    if (name == 'meap_redis_connection') {
        localBaseHook.wrapSyncMethod(obj, 'meap_redis_connection.createConnection', function (obj, args, locals) {
            locals._txn = profiler.agent.tracer.getTransaction();
            if (!!locals._txn) locals.parentSegment = locals._txn.getActiveSegment();
            locals._host = args.length > 1 ? args[1] : (args[0].host || 'unknown');
            locals._port = args.length > 1 ? args[0] : (args[0].port || 'unknown');
            if (locals._txn) locals._conExitCall = profiler.createExitCall({
                backendType: BackendType.BACKEND_TYPE_REDIS,
                name: util.format('Redis://%s:%s', locals._host, locals._port),
                connType: 'Redis',
                dbVendor: 'Redis',
                is_async: false,
                params: undefined,
                operation: 'connect',
                host: locals._host,
                port: locals._port,
                url: util.format('Redis://%s:%s', locals._host, locals._port),
            }, locals._txn);
        }, function (obj, args, socketObj, locals) {
            if (!socketObj)
                return;
            socketObj._bonrConOption = {HOST: locals._host, PORT: locals._port, VENDOR: 'redis'};

            if (socketObj.__bonrAttached__) return;
            socketObj.__bonrAttached__ = true;
            var asyncMethods = ['on', 'once'];
            asyncMethods.forEach(function (method) {
                localBaseHook.wrapAsyncMethod(socketObj, 'socket.' + method);
            });
            var syncMethods = ['end', 'destroy'];
            syncMethods.forEach(function (method) {
                localBaseHook.wrapSyncMethod(socketObj, 'socket.' + method);
            });
            localBaseHook.wrapSyncMethod(socketObj, 'socket.write', function(obj, args, locals) {
                if (!obj.hasOwnProperty('_bonrConOption'))
                    return;
                var supportedProperties = obj._bonrConOption;
                var txn = profiler.agent.tracer.getTransaction();
                if (!!txn) locals.parentSegment = txn.getActiveSegment();
                var exitCall = profiler.createExitCall({
                    backendType: BackendType.BACKEND_TYPE_REDIS,
                    name: util.format('%s://%s:%s', supportedProperties.VENDOR, supportedProperties.HOST,
                        supportedProperties.PORT),
                    connType: 'Redis',
                    dbVendor: 'Redis',
                    is_async: false,
                    params: Object.keys(args).reduce(function (finalArray, item) {
                        finalArray.push(args[item]);
                        return finalArray;
                    }, []).join(' '),
                    operation: args[0],
                    host: supportedProperties.HOST,
                    port: supportedProperties.PORT,
                    url: util.format('%s://%s:%s', supportedProperties.VENDOR, supportedProperties.HOST,
                        supportedProperties.PORT),
                }, txn);
                exitCall.txn = txn;
                //exitCall.time = time;
                exitcalls.push(exitCall);
            });
            wrap.before(socketObj, 'emit', function (obj, args) {
                var error = null;
                if (args[0] === 'reply') {
                    var reply = args[1];
                    if (reply.name === 'Error') {
                        error = reply;
                    }
                    self.finishExitCall(socketObj, error, exitcalls);
                }
                if (args[0] === 'error') {
                    error = args[1];
                    self.finishExitCall(socketObj, error, exitcalls);
                }
            });

            if (locals._conExitCall) {
                profiler.addExitCall(null, locals._conExitCall, locals._txn);
            }
        }, function (obj, args, locals) {
            if (locals.conExitCall) {
                profiler.addExitCall(locals.error, locals._conExitCall, locals._txn);
            }
        });
    }
};
