'use strict';
var BackendType =  require('../common/constant').BackendType;

var util = require('util');
function HiRedisHook(agent) {
    this.agent = agent;
    this.packages = ['hiredis'];
    this.baseHook = agent.baseHook;
    this.connections = undefined;
}
exports.HiRedisHook = HiRedisHook;
HiRedisHook.prototype.finishExitCall = function(socketObj,error,exitCalls) {
    var self = this;
    if(socketObj && exitCalls.length > 0)
    {
        var exitcall = exitCalls.shift();
        if(!exitcall || !exitcall.txn) return;
        var txn = exitcall.txn;
        exitcall.txn = null;
        if(error) error.name='hiredis';
        self.agent.profiler.addExitCall(error,exitcall,txn);
    }
};
HiRedisHook.prototype.attach = function(obj) {
    var self = this;

    if(obj.__bonrAttached__) return;
    obj.__bonrAttached__ = true;

    //var logger = self.agent.logger;
    var wrap = self.agent.wrap;
    var profiler = self.agent.profiler;
    var localBaseHook = self.baseHook;
    var exitcalls = [];
    self.connections = [];
    localBaseHook.wrapSyncMethod(obj,'hiredis.createConnection',function (obj, args, locals) {
        locals._time = profiler.time();
        locals._txn = profiler.agent.tracer.getTransaction();
        if (!!locals._txn) locals.parentSegment = locals._txn.getActiveSegment();
        locals._host = args.length > 1 ? args[1] : (args[0].host || 'unknown');
        locals._port = args.length > 1 ? args[0] : (args[0].port || 'unknown');
        if (locals._txn) locals._conExitCall = profiler.createExitCall(locals._time, {
            backendType: BackendType.BACKEND_TYPE_REDIS,
            name: util.format('Redis://%s:%s',locals._host,locals._port),
            connType: 'Redis',
            dbVendor: 'Redis',
            is_async: false,
            params: undefined,
            operation: 'connect',
            host: locals._host,
            port: locals._port,
            url: util.format('redis://%s:%s',locals._host,locals._port)
        },locals._txn);
    },function (obj, args, socketObj, locals) {
        if(!socketObj)
            return;
        socketObj._bonrConOption = {HOST:locals._host,PORT:locals._port,VENDOR:'redis'};

        if(socketObj.__bonrAttached__) return;
        socketObj.__bonrAttached__ = true;
        var asyncMethods = ['on', 'once'];
        asyncMethods.forEach(function(method) {
            localBaseHook.wrapAsyncMethod(socketObj, 'socket.' + method);
        });
        var syncMethods = ['write', 'end', 'destroy'];
        syncMethods.forEach(function(method) {
            localBaseHook.wrapSyncMethod(socketObj, 'socket.' + method);
        });
        wrap.before(socketObj,'emit',function (obj, args) {
            var error;
            if(args[0] === 'reply')
            {
                var reply = args[1];
                error = null;
                if(reply && reply.name === 'Error')
                {
                    error = reply;
                }
                self.finishExitCall(socketObj,error,exitcalls);
            }
            if(args[0] === 'error')
            {
                error = args[1];
                self.finishExitCall(socketObj,error,exitcalls);
            }
        });

        if (locals._conExitCall) {
            profiler.addExitCall(null,locals._conExitCall,locals._txn);
        }
    }, function (obj,args,locals) {
        if (locals._conExitCall) {
            profiler.addExitCall(locals.error,locals._conExitCall,locals._txn);
        }
    });
    localBaseHook.wrapAsyncMethod(obj,'hiredis.writeCommand',null,function (obj,args,locals) {
        if(!obj.hasOwnProperty('_bonrConOption'))
            return;
        var supportedProperties = obj._bonrConOption;
        var txn = profiler.agent.tracer.getTransaction();
        if (!!txn) locals.parentSegment = txn.getActiveSegment();
        var exitCall = profiler.createExitCall({
            backendType: BackendType.BACKEND_TYPE_REDIS,
            name: util.format('%s://%s:%s',supportedProperties.VENDOR,supportedProperties.HOST,
                supportedProperties.PORT),
            connType: 'Redis',
            dbVendor: 'Redis',
            is_async: false,
            params: Object.keys(args).reduce(function (finalArray,item) {
               finalArray.push(args[item]);
               return finalArray;
            },[]).join(' '),
            operation: args[0],
            host: supportedProperties.HOST,
            port: supportedProperties.PORT,
            url: util.format('%s://%s:%s',supportedProperties.VENDOR,supportedProperties.HOST,
                supportedProperties.PORT),
        },txn);
        exitCall.txn = txn;
        exitcalls.push(exitCall);

    },null);

    localBaseHook.wrapSyncMethod(obj.Reader.prototype, 'Reader.feed');
    localBaseHook.wrapSyncMethod(obj.Reader.prototype, 'Reader.get');
};
