'use strict';

var url = require('url');
var BACKENDTYPE =  require('../common/constant').BackendType;
var BTTYPE =  require('../common/constant').BTType;
var HEADERKEY =  require('../common/constant').HeaderKey;


var util = require('util');
var remoteBackendName = require('../models/exitcall').remoteBackendName;
var BACKENDTYPE =  require('../common/constant').BackendType;
var ERRORCODE =  require('../common/constant').ErrCode;


function HttpHook(agent) {
    this.agent = agent;
    this.baseHook = agent.baseHook;
    this.packages = ['http', 'https'];
    this.traceConfig = agent.config.config.traceConfig || {
        enable : false,
        defaultGathererPercent : 10,
        enableCollectHttpParams : false,
        collectHttpReqHeaders : undefined

    };
    this.jsConfig=agent.config.config.jsConfig || {
        enable:false
    };
    //this.delayedCallbackQueue = [];
    this.enable_datafusion = agent.config.config.enable_datafusion;
}
exports.HttpHook = HttpHook;

//modify param time to segment
HttpHook.prototype.finalizeTransaction = function(err, profiler, segment, transaction, res, prevSegment) {
    //结束业务、结束事务、处理错误
    var self = this;
    if(err)
    {
        transaction.addException(err);
    }
    if(res && res.err)
    {
        transaction.addException(res.err);
    }

    if((!!segment && !segment.done())) return;
    transaction.agent.tracer.curSegment = prevSegment;
    if(!res) {
        self.agent.logger.error('response is undefined '+ res);
        return;
    }
    if(self.traceConfig.enableCollectHttpReqHeaders) {
        //transaction.responseHeaders = res.getHeaders();

        if(res.hasOwnProperty('getHeaders'))
        {
            // Starts from Nodejs 7.7
            transaction.responseHeaders = res.getHeaders();
        }
        else
        {
            transaction.responseHeaders = res._headers;
        }
    }
    transaction.statusCode = res.statusCode;

    transaction.addHttpError(res.statusCode);

    self.agent.tracer.endTransaction(transaction);
    transaction = null;
};

HttpHook.prototype.setHttpDefaults = function(locals, spec, protocol) {
    var params;
    if(typeof(spec) === 'string') {
        params = url.parse(spec);
    } else if (!spec) {
        return;
    } else {
        params = spec;
    }
    locals.opts = {
        hostname:params.hostname || params.host || 'localhost',
        port:params.port || (protocol === 'https' ? 443 : 80),
        path:params.pathname || params.path || '/'
    };
};

HttpHook.prototype.setDataHeaders = function(traceId, res) {
    var self=this;
    if (!traceId) {
        self.agent.logger.error('null traceID, will not set data header!');
        return;
    }
    res.setHeader('br-resp-key',traceId);
    // var cookVal = util.format('br-resp-key="g:%s"', traceId);
    // if(typeof (res.cookie)=="undefined"){
    //     const oldSetCookie = res.getHeader('Set-Cookie');
    //     if (oldSetCookie) {
    //         oldSetCookie.push(cookVal);
    //         res.setHeader('Set-Cookie',oldSetCookie);
    //     } else {
    //         res.setHeader('Set-Cookie', cookVal);
    //     }
    // }else{
    //     res.cookie('br-resp-key','g:'+traceId);
    // }
};


HttpHook.prototype.attach = function(obj, moduleName) {
    var self = this;

    if(obj.__bonrAttached__) return;
    obj.__bonrAttached__ = true;

    // server hook
    // You don't have to create a segmen here, so use the original wrap
    self.agent.wrap.before(obj.Server.prototype, ['on', 'addListener'], function(obj, args, locals) {
        if(args[0] !== 'request') return;

        if(obj.__httpProbe__) return;
        obj.__httpProbe__ = true;
        var cbIndex = args.length -1 ;
        // args[cbIndex] = self.__createRequestHandler(args[cbIndex], moduleName === 'https');
        //Using the original callback wrap
        self.agent.wrap.callback(args,cbIndex,function (obj,args,locals) {
            if(!self.agent.config.config.enable) {
                return;
            }
            var req = args[0];
            var res = args[1];
            var preSegment = self.agent.tracer.curSegment;
            var txn = self.agent.tracer.startTransaction(req,'BT_NODEJS_WEB', moduleName);

            if (!txn || typeof txn === "number") {
                if(typeof txn === "number" && txn < 0 ) {
                    self.agent.logger.error('failed to startTransaction!');
                }
                return;
            }

            //设置调用链
            var segment = self.agent.tracer.curSegment || txn.baseSegment;
            if(!self.agent.tracer.curSegment){
                self.agent.tracer.curSegment=txn.baseSegment;
            }
            if(!!segment && segment.getNodeInfo()) {
                segment.start();
            } else {
                segment = null;
            }

            self.baseHook.wrapSyncMethod(res,'response.writeHead',function(obj, args, locals) {
                if (res._bonrset) return;
                if (!!txn && !txn.isFinished){

                    locals.parentSegment = txn.getActiveSegment();

                    //设置跨容器响应头
                    var cxhead = txn.agent.reporter.CrossHead();
                    var ret = txn.agent.reporter.so.gen_cross_resp_head(txn.instance.ref(),cxhead.ref());
                    if (ret!=0){
                        self.agent.logger.error('failed to gen_cross_resp_head.');
                    }
                    var arr = cxhead.data.toArray();
                    arr.length = arr.indexOf(0x0);
                    if(arr.length){
                        res.setHeader(HEADERKEY.CXRESP, Buffer.from(arr).toString());
                    }
                    //数据融合设置cookie
                    if(txn.agent.config.dataAggregateEnabled()) {
                        var traceId = txn.agent.reporter.TraceId();
                        var ret = txn.agent.reporter.so.get_trace_id(txn.tracer.ref(), traceId.ref());
                        if (ret!=0){
                            self.agent.logger.error('failed to get_trace_id.');
                        }
                        arr = traceId.data.toArray();
                        arr.length = arr.indexOf(0x0);
                        if(arr.length) {
                            self.setDataHeaders(Buffer.from(arr).toString(),res);
                        }
                    }
                    res._bonrset = true;
                }
            },function(obj,args){
                if(req.socket._isHapiProcessing){
                    txn.component='Hapi';
                }
            });

            self.baseHook.wrapAsyncMethod(res,'response.end',null,function(obj, args, locals) {    //wrapBefore()

                if(!!txn) locals.parentSegment = txn.getActiveSegment();

            },function(obj, args, ret, locals) {    //wrapAfter()
                if(!!txn)
                {
                    self.finalizeTransaction(null,self.agent.profiler,segment,txn,res,preSegment); //add new

                }

            },function (obj,args,locals) {
                if(locals.error && !!txn)
                {
                    self.finalizeTransaction(locals.error,self.agent.profiler,segment,txn,res,preSegment); //add new
                }

            });
        });
    });

    //client hook
    function clientCallback(locals) {

        //if(!locals.time || !locals.time.done()) return;
        if(!locals.hasOwnProperty('txn')) locals.txn = null;
        var exitCall = locals.exitCall;
        var error = locals.error;

        if(exitCall) {
            if(locals.res) {
                //获取server端回复的br-cs-rsp响应头
                exitCall.responseHeaders = locals.res.headers;

                var headers=locals.res.headers;

                if(headers[HEADERKEY.CXRESP]) {
                    exitCall.crossResp=headers[HEADERKEY.CXRESP];
                }

                exitCall.statusCode = ~~locals.res.statusCode;


                if((!error) && ((exitCall.statusCode < 200) || (exitCall.statusCode >= 400))) {
                    error = new Error("Error code, " + exitCall.statusCode + ", no further details");
                }
            }
            //结束后端调用
            self.agent.profiler.addExitCall(error,exitCall,locals.txn);
        }
    }

    // support 0.11.x and further
    if(obj.globalAgent && obj.globalAgent.request) {
        obj = obj.globalAgent;
    }

    var httpClientMethod = ['get','request'];//后端调用
    httpClientMethod.forEach(function(method) {                        //wrapBefore()
        self.baseHook.wrapAsyncMethod(obj, 'http.' + method, null, function(obj, args, locals) {
                if( process.versions.modules <= 57 && moduleName == "https" && method == 'request') {
                    return;
                }
                var txn = self.agent.tracer.getTransaction();
                if(!txn) return;
                locals.txn = txn;
                self.setHttpDefaults(locals, args[0], moduleName);//设置http host，port，path

                var req;
                if (typeof(args[0]) === 'string') {
                    var params = url.parse(args[0]);
                    var options = {
                        hostname: params.hostname,
                        port: parseInt(params.port),
                        path: params.path,
                        method: 'GET',
                        headers: {}
                    };
                    // if(method=='get'){
                        args[0]= options;
                    // }
                    req = options;
                } else {
                    req = args[0];
                }

                var supportedProperties = {
                    'HOST': locals.opts.hostname,
                    'PORT': locals.opts.port,
                    'URL':  locals.opts.href? locals.opts.href : util.format('%s://%s:%s%s',moduleName,locals.opts.hostname,locals.opts.port,locals.opts.path),
                    'TYPE': moduleName,
                    'PATH': locals.opts.path,
                    'METHOD': locals.opts.method? locals.opts.method : 'GET',
                };
                var backendNamePath = remoteBackendName(self.agent,url.parse(supportedProperties.URL).pathname);
                if (self.baseHook.needExitCall(txn)) {//判断has-exitcall
                    if (!!txn) locals.parentSegment = txn.getActiveSegment();
                    locals.exitCall = self.agent.profiler.createExitCall({
                        backendType: BACKENDTYPE.BACKEND_TYPE_HTTP,
                        name: util.format('%s://%s:%s%s',moduleName,supportedProperties.HOST,supportedProperties.PORT,backendNamePath || ''),
                        port: supportedProperties.PORT,
                        operation: backendNamePath,
                        // params: supportedProperties.URL,
                        host: supportedProperties.HOST,
                        url: supportedProperties.URL,
                        connType: supportedProperties.TYPE,
                        is_async: false,
                        dbVendor: 'Http',
                        callMethod: supportedProperties.METHOD
                    }, txn);
                    locals.hasExitCall = true;
                    //设置发送给server端的br-cs-req请求头
                    if(!!locals.exitCall.crossReq) {
                        if(!req.headers) {
                            req.headers = {};
                        }
                        req.headers[HEADERKEY.CXREQ] = locals.exitCall.crossReq;
                    }
                }else{
                    console.log("exit");
                }

                self.agent.wrap.callback(args, -1, function(obj, args) {
                    var res = locals.res = args[0];
                    clientCallback(locals);
                    self.agent.wrap.before(res, ['on', 'addListener'], function(obj, args) {
                        // workaround for end event
                        if(args[0] === 'data' && obj.on !== undefined) {
                            obj.on('end', function() {});
                        }
                        if(args[0] !== 'end') return;
                        //clientCallback(locals, txn);
                    });
                });
            },
            //wrapAfter();
            function(obj, args, ret, locals) {
                var writeOnce = false;
                var methods = ['write', 'end'];
                methods.forEach(function(method) {
                    self.baseHook.wrapAsyncMethod(ret, 'ClientRequest.' + method, null, function(obj, args) {
                        if(!writeOnce) {
                            writeOnce = true;
                        }
                        else {
                            return;
                        }

                    });
                });

                self.agent.wrap.before(ret, ['on', 'addListener'], function(obj, args) {
                    var txn = self.agent.tracer.getTransaction();
                    if(args[0] == 'response') {
                        self.agent.wrap.callback(args, -1,function(obj, args) {
                            locals.res = args[0];
                            clientCallback(locals);
                        });
                    }
                    else if(args[0] == 'error') {
                        self.agent.wrap.callback(args, -1, function(obj, args) {
                            locals.error = args[0];
                            clientCallback(locals);
                        });
                    }
                });
            },function (obj,args,locals) {
                clientCallback(locals);
            });
    });
};
