'use strict';
var util = require('util');
var BackendType =  require('../common/constant').BackendType;

function IoredisHook(agent) {
  this.agent = agent;
  this.baseHook = agent.baseHook;
  this.packages = ['ioredis'];
  this.connectExitCalls = [];
}
exports.IoredisHook = IoredisHook;

var syncMethods = [
    'disconnect',
    'end'
];

IoredisHook.prototype.attach = function(obj) {
  var self = this;

  if(obj.__bonrAttached__) return;
  obj.__bonrAttached__ = true;

  var logger = self.agent.logger;
  var wrap = self.agent.wrap;
  var profiler = self.agent.profiler;
  var localBaseHook = self.baseHook;


  var clusters = {};
  syncMethods.forEach(function(method) {
        localBaseHook.wrapSyncMethod(obj.prototype, "ioredis." + method);
  });

  localBaseHook.wrapAsyncMethod(obj.prototype, "ioredis.connect", null, function(obj, args, locals) {
      var txn = self.agent.tracer.getTransaction();
      if (!!txn) locals.parentSegment = txn.getActiveSegment();
      var exitCall = profiler.createExitCall({
          backendType: BackendType.BACKEND_TYPE_REDIS,
          name: util.format('Redis://%s:%s',obj.options.host,obj.options.port),
          connType: 'Redis',
          dbVendor: 'Redis',
          is_async: false,
          params: undefined,
          operation: 'connect',
          host: obj.options.host,
          port: obj.options.port,
          url: util.format('Redis://%s:%s',obj.options.host,obj.options.port)
      }, txn);
      if (!exitCall) return;
      exitCall._txn = txn;
      //exitCall._time = time;
      self.connectExitCalls.push(exitCall);
  });

  localBaseHook.wrapAsyncMethod(obj.prototype, "ioredis.on", null, function(obj, args, locals) {
      if (args[0] == 'connect') {
          var exitCall = self.connectExitCalls.shift();
          if (!exitCall) return;
          //var time = exitCall._time;
          var txn = exitCall._txn;
          exitCall._txn = exitCall._time = null;
          profiler.addExitCall(null, exitCall,txn);
      }
  });
  localBaseHook.wrapAsyncMethod(obj.Cluster.prototype, "ioredis.sendCommand", null,function(obj, args) {
    var serverPool = [];
    if(Array.isArray(obj.startupNodes)) {
      obj.startupNodes.forEach(function(node) {
        var address = node.host + ':' + node.port;
        serverPool.push(address);
        clusters[address] = serverPool;
      });
    }
  },null);


  localBaseHook.wrapAsyncMethod(obj.prototype, "ioredis.sendCommand", null,function(obj, args, locals) {
    var redis = obj;
    var command = args[0];
    var commandName = command.name;
    var params = command.args;
    var address = redis.options.host + ':' + redis.options.port;
    var txn = profiler.agent.tracer.getTransaction();
    if (!!txn) locals.parentSegment = txn.getActiveSegment();
    var serverPool = clusters[address];
    if(serverPool) {
      address = serverPool.join('\n');
    }

    var supportedProperties = {
      'HOST': redis.options.host,
      'PORT': redis.options.port,
      'VENDOR': 'Redis'
    };

    var exitCall = profiler.createExitCall({
        backendType: BackendType.BACKEND_TYPE_REDIS,
        name: util.format('%s://%s:%s',supportedProperties.VENDOR,supportedProperties.HOST,
            supportedProperties.PORT),
        connType: 'Redis',
        dbVendor: 'Redis',
        is_async: false,
        params: params,
        operation: commandName,
        host: supportedProperties.HOST,
        port: supportedProperties.PORT,
        url: util.format('%s://%s:%s',supportedProperties.VENDOR,supportedProperties.HOST,
            supportedProperties.PORT),
    },txn);

    if(command.callback && typeof(command.callback) === 'function') {
      wrap.before(command, 'callback', function(obj, args) {

        if(exitCall) {
          var error = wrap.getErrorObject(args);
          if(error) error.name='ioredis';
          profiler.addExitCall(error, exitCall,txn);
        }
      });

      wrap.before(command.promise, '_promise0', function(obj, args) {
        if(exitCall) {
          var error = wrap.getErrorObject(args);
          if(error) error.name='ioredis';
          profiler.addExitCall(error , exitCall,txn);
        }
      });
    }
    else if(exitCall) {
        profiler.addExitCall(null, exitCall,txn);
    }
  },null);
};
