'use strict';

var util = require('util');
var BackendType =  require('../common/constant').BackendType;
function LoopbackMongodbHook(agent) {
  this.agent = agent;
  this.baseHook = agent.baseHook;
  this.packages = ['loopback-connector-mongodb'];
}
exports.LoopbackMongodbHook = LoopbackMongodbHook;

LoopbackMongodbHook.prototype.makeExitCallInfo = function(obj, command, colName, option) {
  var host = 'unknown';
  var port = 'unknown';
  var database = 'unknown';

  if (obj && obj.settings) {
    if (obj.settings.host) host = obj.settings.host;
    if (obj.settings.port) port = obj.settings.port.toString();
    if (obj.settings.database) database = obj.settings.database;
  }
  if (!colName) colName = 'unknown';
  var queryOptions = (!option ? {} : option);
  var commandArgs = {databaseName:database, collectionName:colName, queryOptions:queryOptions};
  return {
    'backendType': BackendType.BACKEND_TYPE_MONGODB,
    'name':util.format('Mongo://%s/%s',host+':'+port,colName),
    'connType': 'Mongo',
    'dbVendor': 'Mongo',
    'dbVersion': 'unknown',
    'dbSchema': database,
    'is_async': false,
    'params': JSON.stringify(commandArgs),
    'operation': command,
    'host': host,
    'port': port,
    'url':util.format('Mongo://%s/%s',host+':'+port,colName)
  };
};

LoopbackMongodbHook.prototype.checkAndaddErrorInfo = function(args, txn) {
  if (!args || !txn) return;
  var error = args[0];
  var reply = args[1];
  if(!error && reply && reply.result && reply.result.writeErrors)
  {
    error = reply.result.writeErrors[0];
  }
  if ( error && error.errmsg)
  {
    error.stack = error.errmsg;
  }
  if(error){
      error.name="loopback-connector-mongodb";
      txn.addException(error);
  }
};

LoopbackMongodbHook.prototype.attach = function(obj, name) {
  var self = this;
  var localBaseHook = self.baseHook;
  var tracer = self.agent.tracer;

  if(obj.__bonrAttached__) return;
  obj.__bonrAttached__ = true;

  var mongoDb = obj.MongoDB;

  function wrapMongoDb(mongoDb) {
    var methods = ['connect', 'execute'];
    methods.forEach(function(method) {
      var fullname = 'MongoDb.'+method;
      localBaseHook.wrapAsyncMethod(mongoDb, fullname, null, function (obj, args, locals) {
        var commandArgs;
        if (method == 'connect') {
          commandArgs = {};
          commandArgs[1] = 'connect';
        } else {
          commandArgs = args;
        }
        locals.transaction = tracer.getTransaction();
        if (!!locals.transaction) locals.parentSegment = locals.transaction.getActiveSegment();
        locals.wrapBefore = function(obj, args) {
          self.checkAndaddErrorInfo(args, locals.transaction);
        };
        locals.exitCallInfo = self.makeExitCallInfo(obj, commandArgs[1], commandArgs[0],
          commandArgs[1] !== 'find' ? null : commandArgs[2]);
      });
    });

    var methodsWithNoExitCall = ['collectionName','collection','fromDatabase','toDatabase'];
    methodsWithNoExitCall.forEach(function(method) {
      var fullname = 'MongoDb.' + method;
      localBaseHook.wrapSyncMethod(mongoDb, fullname);
    });
  }

  if (!mongoDb) {
    localBaseHook.wrapSyncMethod(obj, 'Connector.initialize', null, function(obj, args) {
      mongoDb = !!args[0] ? args[0].connector : null;
      if (!mongoDb) return;
	    wrapMongoDb(mongoDb);
	  });
  } else {
    wrapMongoDb(mongoDb.prototype);
  }
};

