'use strict';

function LoopbackDatasourceHook(agent) {
  this.agent = agent;
  this.baseHook = agent.baseHook;
  this.packages = ['loopback-datasource-juggler'];
}
exports.LoopbackDatasourceHook = LoopbackDatasourceHook;

LoopbackDatasourceHook.prototype.attach = function(obj, name) {
  var self = this;
  var localBaseHook = self.baseHook;

  if(obj.__bonrAttached__) return;
  obj.__bonrAttached__ = true;

  var datasource = obj.DataSource;
  var protoTxnMethod = ['commit','rollback'];
  var defineMethods = ['createModel', 'define'];
  var daoMethods = ['count', 'create', 'deleteAll', 'deleteById', 'destroyAll', 'destroyById', 'find',
    'findById', 'findByIds', 'findOne', 'findOrCreate', 'remove', 'removeById', 'replaceById',
    'replaceOrCreate', 'update', 'updateAll', 'updateOrCreate', 'upsert', 'upsertWithWhere',
    'patchOrCreate', 'patchOrCreateWithWhere'];
  var daoProtoMeths = ['delete','destroy','patchAttributes','reload','remove','replaceAttributes','save',
    'updateAttributes','updateAttribute'];

  daoMethods.forEach(function(method) {
    if (method == 'create') {
      localBaseHook.wrapAsyncMethod(datasource.DataAccessObject, 'DataAccessObject.'+method, null, function(obj, args, locals) {
        locals.wrapBefore = function(obj, args) {
          if (args.length < 2 || args[0]) return;
          var modelObj = args[1];
          daoProtoMeths.forEach(function(method) {
            localBaseHook.wrapAsyncMethod(modelObj.__proto__, 'DataAccessObject.'+method);
          });
        };
      });
    } else {
      localBaseHook.wrapAsyncMethod(datasource.DataAccessObject, 'DataAccessObject.'+method);
    }
  });

  localBaseHook.wrapAsyncMethod(datasource.DataAccessObject.Transaction, 'Transaction.begin');
  protoTxnMethod.forEach(function (method) {
    localBaseHook.wrapAsyncMethod(datasource.DataAccessObject.Transaction.prototype, 'Transaction.'+method);
  });
  defineMethods.forEach(function(defMethod) {
    localBaseHook.wrapSyncMethod(datasource.prototype, 'DataSource.' + defMethod, null, function(obj, args, ret) {
      var model = ret;
      daoMethods.forEach(function(method) {
        if (method == 'create') {
          localBaseHook.wrapAsyncMethod(model, 'DataAccessObject.'+method, null, function(obj, args, locals) {
            locals.wrapBefore = function(obj, args) {
              if (args.length < 2 || args[0]) return;
              var modelObj = args[1];
              daoProtoMeths.forEach(function(method) {
                localBaseHook.wrapAsyncMethod(modelObj.__proto__, 'DataAccessObject.'+method);
              });
            };
          });
        } else {
          localBaseHook.wrapAsyncMethod(model, 'DataAccessObject.'+method);
        }
      });
      daoProtoMeths.forEach(function(method) {
        localBaseHook.wrapAsyncMethod(model.prototype, 'DataAccessObject.'+method);
      });
    });
  });
};
