'use strict';
var Segment = require('../models/segment').Segment;

function ModuleWrapper(agent) {
    this.agent = agent;
    this.baseHook = agent.baseHook;
}

exports.ModuleWrapper = ModuleWrapper;
//同步调用
//异步调用
ModuleWrapper.prototype.attach = function(obj, usercallback) {
    let self = this;
    var localAgent = self.agent;

    let exportsObj;
    if ('exports' in obj) {
        //This is the case where tracer.mainModule() is called within the lambda handler file
        exportsObj = obj.exports;
    } else {
        //This is the case where the lambda handler file is imported to another script (for layers/extensions)
        exportsObj = obj;
    }
    if (obj.__bonrAttached__) return;
    obj.__bonrAttached__ = true;

    let exportKeyFns = [];
    let exportKeys = Object.keys(exportsObj);
    for (let i = 0; i < exportKeys.length; i++) {
        if (typeof (exportsObj[exportKeys[i]]) === 'function')
            exportKeyFns.push(exportKeys[i]);
    }
    localAgent.wrap.aroundpromise(exportsObj, exportKeyFns, function (obj, args, locals) {
        self.agent.logger.info('[SCF Probe] Intercepted the exports call of the module');
        var trans_name;
        if(typeof usercallback === 'function') {
            try {
                trans_name = usercallback(args);
            } catch (e) {
                self.agent.logger.error('failed to get transaction name through user callback.Exception:' + e);
            }
        }
        if (!trans_name &&args.length >= 2 && args[1]
                && args[1].request_id
                && args[1].function_name
                && args[1].function_version) {
            trans_name = args[1].function_name + '_' + args[1].function_version
        }
        if (!trans_name) {
            trans_name = locals.methodName? locals.methodName: 'main_handler'
        }

        var txn = self.agent.tracer.startBGTaskTransaction(trans_name, 'BT_NODEJS_BG');
        if (!txn) {
            return;
        }

        locals.txn = txn;
        txn.segmentNum = 1;
        var segment = new Segment(locals.methodName, txn);
        segment.isRoot = true;
        txn.baseSegment = segment;
        txn.curSegment = segment;
        locals.segment = segment;
        if (!!locals.segment && locals.segment.getNodeInfo()) {
            locals.segment.start();
        } else {
            locals.segment = null;
        }
        //异步调用的情况
        locals.hasCallback = localAgent.wrap.callbackPromiseChain(args, -1, function (obj, args, locals) {
            let flushEventsPromise;
            if (!locals.promiseBasedLambda) {
                if (locals.txn && locals.segment) {
                    locals.callbackInvoked = true;
                    self.finalizeTransaction(args[0], self.agent.profiler, locals.segment, locals.txn); //add new
                    return flushEventsPromise.then(() => {
                        locals.callbackEmitter.emit('done');
                    });
                }
            }
        })
    }, function (obj, args, ret, locals) {
        if(!locals.txn) return;
        if (ret && (ret.__appdynamicsIsPromiseResult__ || locals.hasCallback)) {

            // If stopTransaction returns a promise then the around() wrapper will ensure that the
            // pending downstream events are flushed.
            let flushEventsPromise;
            if (ret.__appdynamicsIsPromiseResult__) {
                locals.promiseBasedLambda = true;
            }
            /* For the case where customers use an async handler with a callback.
             * If the callback is called before the async handler returns we need
             * to add a delayed promise to the async handler so that the callback has
             * time to flush the events.
             */
            if (locals.callbackInvoked) {
                self.logger.debug('[Lambda Probe] Callback invoked, will wait for events to flush before resolving promise.');
                return new Promise(resolve => {
                    locals.callbackEmitter.on('done', () => {
                        /* Push this to the end of event queue to ensure that
                         * the callback gets called first. This is because the 'done'
                         * event is emitted before the original callback is called.
                         * setTimeout(() => resolve(), 0) pushes the resolve() call
                         * to the end of the event queue.
                         */
                        setTimeout(() => resolve(), 0);
                    });
                })
            }

            if (locals.segment) {
                self.finalizeTransaction(null, self.agent.profiler, locals.segment, locals.txn); //add new
            }

            self.agent.lambdaFnRunning = false;
            //Note: locals.promiseBasedLambda is used instead of locals.hasCallback
            //      because from testing it appears that locals.hasCallback is set to true
            //      even for promise based handlers. This is because the lambda runtime
            //      passes the callback object regardless of the type of handler (async or non-async)
            if (!locals.promiseBasedLambda && ret.error && ret.isLambdaError) {
                // If callback based lambda threw an error then we need to rethrow it
                if (flushEventsPromise) {
                    // Ensure events flush before throwing error
                    return flushEventsPromise.then(function () {
                        return Promise.reject(ret.error);
                    }).catch(function (err) {
                        return Promise.reject(ret.error);
                    });
                } else {
                    throw ret.error;
                }
            }
        }
    })
}
//没有看到错误处理
ModuleWrapper.prototype.finalizeTransaction = function(err, profiler, segment, transaction) {
    //结束业务、结束事务、处理错误
    var self = this;
    if(err) {
        transaction.addException(err);
    }
    if((!!segment && !segment.done())) return;
    self.agent.profiler.endBGTaskTransaction(transaction);
    transaction = null;
};

