'use strict';
var BackendType =  require('../common/constant').BackendType;
var util = require('util');
var BaseHook = require('../hook/core/base_hook').BaseHook;
function Mysql2Hook(agent) {
    this.agent = agent;
    this.baseHook = new BaseHook(agent);
    this.packages = ['mysql2'];
}
exports.Mysql2Hook = Mysql2Hook;

function hookStream(hookHd, query) {
  hookHd.wrapSyncMethod(query, 'stream', null, function (obj, args, stream) {
        hookHd.wrapSyncMethod(stream, 'emit');
        hookHd.wrapSyncMethod(stream, 'pipe');
    });
}

function prepareExitCall(config, client, wrap, profiler,baseHook) {
    var supportedProperties = {
      'HOST': config.host,
      'PORT': config.port,
      'DATABASE': config.database,
      'VENDOR': 'MySQL'
    };
    
    if(client === undefined) {
        return;
    }

    if( client.__bonrAttach__ ) return;
    client.__bonrAttach__ = true;
    var orig = client['query'];

    baseHook.wrapAsyncMethod(client, 'query',null, function(obj, args, locals) {
        var command, params;

        var txn;
        if (!client.hasOwnProperty('_txn')) {
          txn = profiler.agent.tracer.getTransaction();
        } else {
          txn = client._txn;
          client._txn = null;
        }
        var exitCall = profiler.createExitCall({
            'backendType': BackendType.BACKEND_TYPE_MYSQL,
            'name':util.format('%s://%s:%s/%s',supportedProperties.VENDOR,supportedProperties.HOST,
                supportedProperties.PORT,supportedProperties.DATABASE),
            'connType': 'MySQL',
            'dbVendor': supportedProperties.VENDOR,
            'dbVersion': version,
            'dbSchema': supportedProperties.DATABASE,
            'is_async': false,
            'operation': command,
            //'params': JSON.stringify(params),
            'host': supportedProperties.HOST,
            'port': supportedProperties.PORT,
            'url':util.format('%s://%s@%s:%s/%s',supportedProperties.VENDOR,config.user,supportedProperties.HOST,
                supportedProperties.PORT,supportedProperties.DATABASE),
        },txn);

        var explain_args = [];
        for(var i = 0;i < args.length; i++) {
            explain_args[i] = args[i];
        }
        explain_args[0] = 'explain ' + args[0];
        var Client = obj;
        function processQueryCb(obj, args) {
            var version;
            if (obj !== undefined && obj._connection && obj._connection._protocol &&
                obj._connection._protocol._handshakeInitializationPacket &&
                obj._connection._protocol._handshakeInitializationPacket.serverVersion
            ) {
              version = obj._connection._protocol._handshakeInitializationPacket.serverVersion;
            } else {
                version = '';
            }
        
            var error = wrap.getErrorObject(args);
            if (!txn) return;
            if (!exitCall) {
              return;
            }
            //后端调用时间大于阈值时，获取执行计划
            if(!error && !!exitCall && exitCall.needExplainPlan(txn.agent)) {
                var callback = function (err, results, fields){
                    if(!err && results && fields) {
                        exitCall.collectMySqlExplainPlan(fields, results);
                    }
                };
                explain_args[explain_args.length - 1] = callback;
                orig.apply(Client,explain_args);
            }
            profiler.addExitCall(error, exitCall,txn);
        }

        if (args[0] && args[0].__bonreeCb) {
            command = args[0].sql;
            params = args[0].values;
            wrap.before(args[0], 'onResult', processQueryCb);
        } else {
            command = args.length > 0 ? args[0] : undefined;
            params = args.length > 1 && Array.isArray(args[1]) ? args[1] : undefined;
            var callback;
            if (typeof args[0] === 'function') {
                callback = 0;
            } else if (typeof args[1] === 'function') {
                callback = 1;
            } else if (typeof args[2] === 'function') {
                callback = 2;
            }
            wrap.callback(args, callback, processQueryCb);
        }

    },function(obj, args, query) {
        hookStream(baseHook, query);
    });
}
  
Mysql2Hook.prototype.attach = function(obj) {
    var self = this;
    var wrap = self.agent.wrap;
    var localBaseHook = self.baseHook;
    var profiler = self.agent.profiler;
    var cmds = ['createClient', 'createConnection', 'createPool'];
  
    if(obj.__bonrAttached__) return;
    obj.__bonrAttached__ = true;
 
    localBaseHook.wrapSyncMethod(obj, "createPoolCluster");
    cmds.forEach(function(createCmd) {
        localBaseHook.wrapSyncMethod(obj, createCmd,null, function(obj, args, ret) {
            if (createCmd === 'createPool') {
                localBaseHook.wrapAsyncMethod(ret, 'getConnection', null,function(obj, args) {
                    wrap.callback(args, -1, function(obj, args) {
                        if (args[0]) return;
                        if (args[1]) {
                            var connection = args[1], config = connection.config;
                            prepareExitCall(config, connection, wrap, profiler,localBaseHook);
                        }
                        });
                      },null);

                localBaseHook.wrapAsyncMethod(ret, 'query', null, null, function (obj, args, query) {
                    hookStream(localBaseHook, query);
                });
  
                wrap.after(ret, 'query', function(obj, args, query) {
                    var callback;
                    if (typeof args[0] === 'function') {
                      callback = args[0];
                    } else if (typeof args[1] === 'function') {
                      callback = args[1];
                    } else {
                      callback = args[2];
                    }
                    query.__bonreeCb = callback;
                });
        } else {
            var client = ret;
            var config = (createCmd === 'createClient' ? client : client.config);
            if (!config) {
                return;
            }
            prepareExitCall(config, client, wrap, profiler,localBaseHook);
        }
    });
  });
};
  