'use strict';

var util = require('util');
var BackendType =  require('../common/constant').BackendType;

function PgHook(agent) {
  this.agent = agent;
  this.baseHook = agent.baseHook;
  this.packages = ['pg','pg-pool'];
}
exports.PgHook = PgHook;

function makeExitCallInfo(client, supportedProperties, args) {
    if (!args || args.length == 0 || (typeof args[0] == 'string' && args[0].length == 0)) return null;
    var command = args[0];

    return {
        'backendType': BackendType.BACKEND_TYPE_POSTGRESQL,
        'name':util.format('%s://%s:%s/%s',supportedProperties.VENDOR,supportedProperties.HOST,
            supportedProperties.PORT,supportedProperties.DATABASE),
        'connType': 'PostgreSQL',
        'dbVendor': supportedProperties.VENDOR,
        'dbVersion': supportedProperties.VERSION,
        'dbSchema': supportedProperties.DATABASE,
        'is_async': false,
        'operation': command,
        'host': supportedProperties.HOST,
        'port': supportedProperties.PORT,
        'url':util.format('%s://%s@%s:%s/%s',supportedProperties.VENDOR,client.user,supportedProperties.HOST,
            supportedProperties.PORT,supportedProperties.DATABASE),
    };
}

PgHook.prototype.attach = function(obj, name) {
  var self = this;

  if(obj.__bonrAttached__) return;
  obj.__bonrAttached__ = true;

  var logger = self.agent.logger;
  var wrap = self.agent.wrap;
  var tracer = self.agent.tracer;
  var localBaseHook = self.baseHook;

  function probePool(obj) {
      localBaseHook.wrapAsyncMethod(obj.prototype, 'Pool.connect', null, function(obj, args,locals) {
          var txn = tracer.getTransaction();
          var clientOptions = obj.options;
          if (!!txn) {
              locals.transaction = obj._txn = txn;
              locals.parentSegment = txn && txn.getActiveSegment();
              var cmd = ['connect'];
              locals.exitCallInfo = makeExitCallInfo(clientOptions, {
                  'HOST': clientOptions.host || 'unknown',
                  'PORT': clientOptions.port,
                  'DATABASE': clientOptions.database,
                  'VERSION': undefined,
                  'VENDOR': 'PostgreSQL'
              }, cmd);
              locals.wrapBefore = function(obj, args) {
                  if (args.length < 1) return;
                  if (args.length > 1) {
                      var client = args[1];
                      var origQuery = client['query'];
                      localBaseHook.wrapAsyncMethod(client, 'client.query',null,function(obj, args, locals) {
                          var connection = obj;
                          locals.transaction = tracer.getTransaction();
                          locals.parentSegment = locals.transaction && locals.transaction.getActiveSegment();
                          locals.exitCallInfo = makeExitCallInfo(clientOptions, {
                              'HOST': clientOptions.host || 'unknown',
                              'PORT': clientOptions.port,
                              'DATABASE': clientOptions.database,
                              'VERSION': undefined,
                              'VENDOR': 'PostgreSQL'
                          }, args);
                          wrapQueryCb(connection, origQuery,locals);
                          var explain_args = [];
                          for (var i=0; i < args.length; i++) {
                              explain_args[i] = args[i];
                          }
                          explain_args[0] = 'explain ' + args[0];
                          locals.wrapBefore = function(obj, args) {
                              locals.error = wrap.getErrorObject(args);
                              if(locals.error){
                                  locals.error.name='postgresql';
                              }
                              //后端执行时间超过阈值的情况，采集执行计划
                              if(!locals.error && !!locals.exitCall && locals.exitCall.needExplainPlan(locals.transaction.agent)) {
                                  var callback = function (err,result) {
                                      if(!err && result) {
                                          locals.exitCall.collectPGExplainPlan(result.rows);
                                          localBaseHook.agent.profiler.addExitCall(locals.error,locals.exitCall,locals.transaction);
                                      }
                                  };
                                  explain_args[explain_args.length - 1] = callback;
                                  query.apply(connection,explain_args);
                              } else {
                                  localBaseHook.agent.profiler.addExitCall(locals.error,locals.exitCall,locals.transaction);
                              }
                          };

                      });
                  }
              };
          }
      });
      localBaseHook.wrapAsyncMethod(obj.prototype, 'Pool.query');
      localBaseHook.wrapAsyncMethod(obj.prototype, 'Pool.end');
  }

  function probe(obj) {
    if(obj.__bonrAttached__) return;
    obj.__bonrAttached__ = true;

    var origQuery = obj['query'];

    localBaseHook.wrapAsyncMethod(obj, 'pg.connect', null, function(obj,args,locals) {
         locals.transaction = tracer.getTransaction();
         if (!locals.transaction) return;
        locals.parentSegment = locals.transaction.getActiveSegment();
        var cmd = ['connect'];
        if (localBaseHook.needExitCall(locals.transaction)) locals.exitCallInfo = makeExitCallInfo(obj, {
            'HOST': obj.host,
            'PORT': obj.port,
            'DATABASE': obj.database,
            'VERSION': undefined,
            'VENDOR': 'PostgreSQL'
        }, cmd);
        locals.wrapBefore = function(obj, args) {
            locals.error = wrap.getErrorObject(args);
            if(locals.error){
                locals.error.name="PostgreSQL";
            }
        };
    }, function(obj, args, ret, locals) {
        var supportedProperties = {
        	'HOST': obj.host,
        	'PORT': obj.port,
        	'DATABASE': obj.database,
        	'VERSION': undefined,
        	'VENDOR': 'PostgreSQL'
        };
        obj.connection.on('parameterStatus', function(msg) {
            if (msg.parameterName !== 'server_version') {
                return;
            }
            supportedProperties.VERSION = msg.parameterValue;
        });
        var explain_args = [];

        localBaseHook.wrapAsyncMethod(obj, 'pg.query',null,function(obj, args, locals) {
            var self = this;
            locals.transaction = tracer.getTransaction();
            if (!locals.transaction) return;

            locals.parentSegment = locals.transaction && locals.transaction.getActiveSegment();
            if (localBaseHook.needExitCall(locals.transaction)) {
                var exitCallInfo = makeExitCallInfo(obj, supportedProperties,args);
                locals.exitCall = localBaseHook.agent.profiler.createExitCall(exitCallInfo, locals.transaction);
                locals.hasExitCall = true;
            }
            var connection = obj;
            for(var i = 0;i < args.length; i++) {
                explain_args[i] = args[i];
            }
            explain_args[0] = 'explain ' + args[0];

            locals.wrapBefore = function(obj, args) {
                locals.error = wrap.getErrorObject(args);
                if(locals.error){
                    locals.error.name='PostgreSQL';
                }
                //后端执行时间超过阈值的情况，采集执行计划
                if(!locals.error && !!locals.exitCall && locals.exitCall.needExplainPlan(locals.transaction.agent)) {
                    var callback = function (err,result) {
                        if(!err && result) {
                            locals.exitCall.collectPGExplainPlan(result.rows)
                            localBaseHook.agent.profiler.addExitCall(locals.error,locals.exitCall,locals.transaction);
                        }
                    };
                    explain_args[explain_args.length - 1] = callback;
                    origQuery.apply(connection,explain_args);
                } else {
                    localBaseHook.agent.profiler.addExitCall(locals.error,locals.exitCall,locals.transaction);
                }
            };

      }, function(obj, args, ret, locals) { // Evented API
        // If has a callback, ignore
        if (args.length > 0 && typeof args[args.length - 1] === 'function') {
          return;
        }
        wrap.before(ret, 'on',function(obj, args) {
          var event = args[0];
          if(event !== 'end' && event !== 'error') return;

          var error;
          var connection = obj;
          wrap.callback(args, -1, function(obj, args) {
              if(event === 'error') {
                  error = wrap.getErrorObject(args);
                  if (error) {
                      error.name = 'PostgreSQL';
                  }
                  locals.transaction.addException(error);
                  localBaseHook.agent.profiler.addExitCall(error,locals.exitCall,locals.transaction);
              }
              if (!error &&!!locals.exitCall && locals.exitCall.needExplainPlan(locals.transaction.agent)) {
                  var callback = function (err, result) {
                      if (!err && result) {
                          locals.exitCall.collectPGExplainPlan(result.rows);
                          localBaseHook.agent.profiler.addExitCall(error,locals.exitCall,locals.transaction);
                      }
                  };
                  explain_args[explain_args.length - 1] = callback;
                  origQuery.apply(connection, explain_args);
              } else {
                  localBaseHook.agent.profiler.addExitCall(error,locals.exitCall,locals.transaction);
              }
              return;
          });
        });
      });
    });
    localBaseHook.wrapAsyncMethod(obj, 'pg.end');
  }

  // Native, reinitialize probe
  wrap.getter(obj, 'native', function(obj, ret) {
    localBaseHook.wrapSyncMethod(ret, 'Client',null, function(obj, args, ret) {
      probe(ret.__proto__);
    });
  });
  if (name == 'pg-pool') {
      probePool(obj);
  } else {
      probe(obj.Client.prototype);
  }
};



