'use strict';
var util = require('util');
function PromiseHook(agent) {
    this.agent = agent;
    this.baseHook = agent.baseHook;
    this.packages = ['promise'];
}
exports.PromiseHook = PromiseHook;

var syncMethods = [
    'isPending',
    'isFulfilled',
    'isRejected',
    'getValue',
    'getReason',
    'getState',
    'all'
];
var asyncMethods = [
    'catch',
    'finally'
];

PromiseHook.prototype.attach = function(obj) {
    var baseHook = this.baseHook;
    syncMethods.forEach(function(method) {
       baseHook.wrapSyncMethod(obj.prototype, 'promise.' + method);
    });
    asyncMethods.forEach(function(method) {
        baseHook.wrapAsyncMethod(obj.prototype, 'promise.' + method);
    });
    baseHook.wrapAllCallback(obj.prototype, 'promise.then');
};

//If you pay attention to the callback function introduced in the constructor, you need to do this
PromiseHook.prototype.hookConstructor = function(obj) {
    var baseHook = this.baseHook;
    var orig = obj;
    function PromiseWithHook() {
        var locals = {};
        locals.wrapBefore = function(obj, args) {
            var argNum = args.length;
            var curName = {};
            if (argNum < 1) return;

            if (typeof args[0] === 'function') {
                curName.segmentName = 'promise.resolve';
                args[0] = baseHook.bindFunction(args[0], null, true, curName);
            }
            if (argNum > 1 && typeof args[1] === 'function') {
                curName.segmentName = 'promise.reject';
                args[1] = baseHook.bindFunction(args[1], null, true, curName);
            }
        };
        arguments[0] = baseHook.bindFunction(arguments[0], null, true, locals);
        orig.call(this, arguments[0]);
    }
    util.inherits(PromiseWithHook, orig);
    for (var attr in PromiseWithHook.super_) {
        PromiseWithHook[attr] = PromiseWithHook.super_[attr];
    }
    return PromiseWithHook;
};