'use strict';
var URL = require('url');
var remoteBackendName = require('../models/exitcall').remoteBackendName;
var BackendType = require('../common/constant').BackendType;

function RabbitJsHook(agent) {
    this.agent = agent;
    this.baseHook = agent.baseHook;
    this.packages = ['rabbit.js'];
    // this.contextProperties = {};
}

exports.RabbitJsHook = RabbitJsHook;

RabbitJsHook.prototype._createExitcall = function (options,txn) {
    if (!this.baseHook.needExitCall(txn)) return null;
    var self = this;
    var profiler = self.agent.profiler;
    var fullurl = options.basicProp.URL.replace(/\/$/,'') + '/' + options.actor +
            '/' + options.action + 'to' + options.target;
    var parts = URL.parse(fullurl);
    return profiler.createExitCall({
        backendType: BackendType.BACKEND_TYPE_RABBITMQ,
        name: fullurl,
        host: parts.hostname,
        port: options.basicProp.PORT,
        operation: remoteBackendName(self.agent,parts.path) || options.action,
        // params: fullurl,
        url: fullurl,
        connType: options.basicProp.PROTOCOL,
        is_async: false,
    },txn);
};

RabbitJsHook.prototype._addExitCall = function (error,exitcall,txn) {
    if(!exitcall){return;}
    var self = this;
    var profiler = self.agent.profiler;
    //var time = exitcall.time;
    if(!exitcall) {return;}
    profiler.addExitCall(error,exitcall,txn);
};

RabbitJsHook.prototype.__SocketWrap = function (targetSockets,meth) {
    var self = this;
    var wrap = self.agent.wrap;
    var profiler = self.agent.profiler;
    var localBaseHook = self.baseHook;
    targetSockets.map(function(socketClass){
       localBaseHook.wrapAsyncMethod(socketClass,meth,null,function (obj,args,locals) {
            if(!obj && !obj.hasOwnProperty('ch')){return;}
            var ch = obj.ch;
            if(!ch || !ch.hasOwnProperty('connection')){return;}
            var connection = ch.connection;
            var txn = profiler.agent.tracer.getTransaction();
            locals.parentSegment = txn && txn.getActiveSegment();
            if(!connection || !connection.hasOwnProperty('_bonrConOption')) {return;}
            var queue = '',exchange;
            if(obj.hasOwnProperty('queues'))
            {
                // queue = JSON.stringify(obj.queues);
                queue = obj.queues.join('_');
            }
            if(obj.hasOwnProperty('consumers'))
            {
                // queue = JSON.stringify(Object.keys(obj.consumers));
                queue = Object.keys(obj.consumers).join('_');
            }
            var meths = [meth];
            var exitcallOption = {
                basicProp: connection._bonrConOption,
                actor: obj.constructor.name,
                action: meths.reduce(function(finalStr,meth){finalStr=finalStr+meth; return finalStr;},''),
                target: queue
            };
            locals.exitCall = self._createExitcall(exitcallOption,txn);
        },function (obj,args,ret,locals) {
            var txn = profiler.agent.tracer.getTransaction();
            self._addExitCall(locals.error,locals.exitCall,txn);
        });
    });
};
RabbitJsHook.prototype.attach = function (modobj) {
    var self = this;

    if(modobj.__bonrAttached__) return;
    modobj.__bonrAttached__ = true;

    self.baseHook.wrapSyncMethod(modobj, 'Socket.createContext', null, function(obj, args, ret) {
        var context = ret;
        self.baseHook.wrapAsyncMethod(context, 'Context.on');
    });
    var objsHasConnect = [modobj.PUB.prototype,modobj.PUSH.prototype,modobj.REQ.prototype,modobj.REP.prototype,modobj.TASK.prototype,
                          modobj.PULL.prototype,modobj.SUB.prototype,modobj.WORKER.prototype];
    objsHasConnect.forEach(function(curObj) {
        self.baseHook.wrapAsyncMethod(curObj, 'connect');
    });
    self.__SocketWrap([modobj.PUB.prototype,modobj.PUSH.prototype,modobj.REQ.prototype,modobj.REP.prototype,modobj.TASK.prototype],'write');
    self.__SocketWrap([modobj.PULL.prototype,modobj.SUB.prototype,modobj.REQ.prototype,modobj.REP.prototype,modobj.WORKER.prototype],'read');
    self.__SocketWrap([modobj.PUB.prototype],'publish');
    self.__SocketWrap([modobj.REP.prototype,modobj.WORKER.prototype],'requeue');
    self.__SocketWrap([modobj.WORKER.prototype],'ack');
    self.__SocketWrap([modobj.REP.prototype,modobj.WORKER.prototype],'discard');
    self.__SocketWrap([modobj.TASK.prototype],'post');
};