'use strict';
var BackendType =  require('../common/constant').BackendType;

var util = require('util');
function RedisHook(agent) {
    this.agent = agent;
    this.baseHook = agent.baseHook;
    this.packages = ['redis'];
    this.connectExitCalls = [];
}
exports.RedisHook = RedisHook;

var asyncMethods = [
    'get',
    'set',
    'hgetall',
    'hmset',
    'hget',
    'hset',
    'on',
    'on_connect',
    'on_error',
    'on_ready'
];

RedisHook.prototype.attach = function(obj) {
    var self = this;

    if(obj.__bonrAttached__) return;
    obj.__bonrAttached__ = true;

    var logger = self.agent.logger;
    var wrap = self.agent.wrap;
    var profiler = self.agent.profiler;
    var localBaseHook = self.baseHook;
    var tracer = self.agent.tracer;

    function getCommandParams(command,args) {
        switch(command)
        {
            case 'info':
                return 'info';
            default:
            {
                var keyValues = [];
                if(args.length > 0)
                {
                    if(typeof args[args.length-1] !== 'function')
                    {
                        keyValues = args.slice(0,args.length);
                    }
                    else
                    {
                        keyValues = args.slice(0,args.length-1);
                    }
                }
                return command+' '+keyValues.join(' ');
            }
        }
    }
    function proxyCommand(client) {
        var method = client.internal_send_command ? "internal_send_command" : "send_command";
        localBaseHook.wrapAsyncMethod(client, method,null, function(obj, args, locals) {
            if (args && args[0] && args[0]._marked) return;
            //locals.time = profiler.time();
            var address = client.address || (client.host + ':' + client.port);
            var hostport = address.split(":");
            var command, params,cmdobj;

            if (typeof(args[0]) == 'string') {
                command = args[0];
                params = args[1];
            } else {
                command = args[0].command;
                params = args[0].args;
                cmdobj = args[0];
            }
            var supportedProperties = {
                'HOST': hostport[0],
                'PORT': hostport[1],
                'VENDOR': 'Redis'
            };

            if (!client.hasOwnProperty('_txn')) {
                locals.txn = tracer.getTransaction();
            } else {
                locals.txn = client._txn;
                client._txn = null;
            }
            locals.parentSegment = locals.txn && locals.txn.getActiveSegment();
            locals.exitCall = profiler.createExitCall({
                backendType: BackendType.BACKEND_TYPE_REDIS,
                name: util.format('%s://%s:%s',supportedProperties.VENDOR,supportedProperties.HOST,
                    supportedProperties.PORT),
                connType: 'Redis',
                dbVendor: 'Redis',
                is_async: false,
                params: params,
                operation: command,
                host: supportedProperties.HOST,
                port: supportedProperties.PORT,
                url: util.format('%s://%s:%s',supportedProperties.VENDOR,supportedProperties.HOST,
                    supportedProperties.PORT),
                segmentUsed:false
                }, locals.txn);
            if (args && args[0]) args[0]._marked = true;

            if(cmdobj && cmdobj.hasOwnProperty('callback') && cmdobj.callback)
            {
                var orgin = cmdobj.callback;
                cmdobj.callback = function (err,reply) {
                    if (locals.exitCall && locals.txn) {
                        if(err) err.name='Redis';
                        profiler.addExitCall(err, locals.exitCall, locals.txn);
                    }
                    orgin(err,reply);
                };
            }
            else{
                if(typeof args[args.length-1] === 'function')
                {
                    wrap.callback(args, -1, function(obj, args) {
                    if (locals.exitCall && locals.txn) {
                        // var error = proxy.getErrorObject(args);
                        profiler.addExitCall(null, locals.exitCall, locals.txn);
                    }
                    });
                }
                else
                {
                    locals._nocb = true;
                }
            }
        }, function(obj, args, ret, locals) {
            if (locals._nocb) {
                //End exitcall now if there is no callback for the command
                if (locals.exitCall && locals.txn) {
                    // var error = proxy.getErrorObject(args);
                    profiler.addExitCall(null, locals.exitCall, locals.txn);
                }
            }
        });
    }

    localBaseHook.wrapSyncMethod(obj, 'redis.createClient',function(obj, args, locals) {
        locals.txn = tracer.getTransaction();
        locals.parentSegment = locals.txn && locals.txn.getActiveSegment();
    }, function(obj, args, ret, locals) {
        var address = ret.address || (ret.host + ':' + ret.port);
        var hostport = address.split(":");
        var redisUrl = util.format('Redis://%s:%s',hostport[0], hostport[1]);
        var exitCall = profiler.createExitCall({
            backendType: BackendType.BACKEND_TYPE_REDIS,
            name: redisUrl,
            connType: 'Redis',
            dbVendor: 'Redis',
            is_async: false,
            params: '',
            operation: 'connect',
            host: hostport[0],
            port: hostport[1],
            url: redisUrl,
            segmentUsed:false
        }, locals.txn);
        //profiler.addExitCall(locals.time,  null, exitCall, locals.txn);
        if (!!exitCall) {
            exitCall._txn = locals.txn;
            self.connectExitCalls.push(exitCall);
        }

        var client = ret;
        localBaseHook.wrapAsyncMethod(client, "redis.on", null, function(obj, args, locals) {
            if (args[0] == 'connect') {
                var exitCall = self.connectExitCalls.shift();
                if (!exitCall) return;
                var txn = exitCall._txn;
                exitCall._txn = exitCall._time = null;
                exitCall.method='on';
                profiler.addExitCall(null, exitCall,txn);
            }
        });
        asyncMethods.forEach(function(method) {
            localBaseHook.wrapAsyncMethod(client, 'RedisClient.' + method);
        });
        localBaseHook.wrapAsyncMethod(client, 'RedisClient.end');
        proxyCommand(client);
    }, function(obj, args, locals) {
        var exitCall = profiler.createExitCall({
            backendType: BackendType.BACKEND_TYPE_REDIS,
            name: 'Redis://unknown:unknown',
            connType: 'Redis',
            dbVendor: 'Redis',
            is_async: false,
            params: '',
            operation: 'connect',
            host: 'unknown',
            port: 'unknown',
            url: 'Redis://unknown:unknown',
            segmentUsed:false
        }, locals.txn);
        //profiler.addExitCall(locals.time, locals.error, exitCall, locals.txn);
        if (!!exitCall) {
            exitCall._txn = locals.txn;
            //exitCall._time = locals.time;
            self.connectExitCalls.push(exitCall);
        }
    });
};
