'use strict';
var loggerfuncs = require('../core/logger').Logger_funcs;
var BackendType = require("../common/constant").BackendType;
var MQDIRECTION = require("../common/constant").MQDirection;
var MQDETAILPREFIX = require("../common/constant").MQDetailPrefix;


var GETURISEGMENTTYPE = require("../common/constant").GetURISegmentType;

function ExitCall(exitCallInfo, transaction) {
    if(!exitCallInfo || !transaction || transaction.isFinished) {
        return;
    }
    this.isFinished = false;
    this.reporter = transaction.agent.reporter;
    this.transaction = transaction;
    this.backendType = exitCallInfo.backendType;
    this.name = exitCallInfo.name;
    this.connType = exitCallInfo.connType;
    this.host = exitCallInfo.host;
    this.port = exitCallInfo.port;
    this.duration_time = 0;
    this.id = transaction.exitCallCnt++;
    this.start_time = Date.now();
    if(this.backendType >= BackendType.BACKEND_TYPE_MONGODB && this.backendType <= BackendType.BACKEND_TYPE_SSDB)
    if(transaction.agent.config.config.backendConfig.enableCaptureNosqlParams) {
        if(exitCallInfo.params instanceof Array) {
            this.params = exitCallInfo.params.join(' ');
        } else {
            this.params =exitCallInfo.params;
        }
        this.params = this.params.substr(transaction.agent.config.config.nosqlMaxParamLength)
    }
    this.type = undefined;
    this.operation = exitCallInfo.operation;
    this.duration_time = 0;
    this.have_error = false;
    this.bt = undefined;
    this.is_async = exitCallInfo.is_async;
    this.agentGUID = undefined;
    this.agentId=-1;

    //Sql specific
    this.dbVendor = exitCallInfo.dbVendor;
    this.dbVersion = exitCallInfo.dbVersion;
    this.dbSchema = exitCallInfo.dbSchema;

    if(this.backendType >= BackendType.BACKEND_TYPE_MYSQL && this.backendType <= BackendType.BACKEND_TYPE_GENERIC_JDBC) {
        this.dbVendor = this.dbVendor || 'unknown';
        this.dbVersion = this.dbVersion ||'unknown';
        this.dbSchema = this.dbSchema ||'unknown';
    }

    //remote call specific
    this.crossReq = undefined;
    this.crossResp = undefined;
    this.callMethod = exitCallInfo.callMethod || exitCallInfo.operation ||'unknown';

    this.url = exitCallInfo.url;
    if(this.backendType  === BackendType.BACKEND_TYPE_MYSQL) {
        this.detail = this.operation;
    } else if (this.params !== undefined && this.params!=""){
        this.detail = this.operation + ':' + this.params;
    } else {
        this.detail = this.operation;
    }

    var be_call_options = new this.reporter.BeCallOptions();
    be_call_options.class_method.class_name.value = this.reporter.ref.NULL;
    be_call_options.class_method.class_name.length = 0;

    be_call_options.class_method.method_name.value = this.reporter.ref.NULL;
    be_call_options.class_method.method_name.length = 0;

    be_call_options.call_detail.detail.value = this.reporter.ref.NULL;
    be_call_options.call_detail.detail.length = 0;

    be_call_options.be.db_schema.value = this.reporter.ref.NULL;
    be_call_options.be.db_schema.length = 0;

    be_call_options.be.db_vendor.value = this.reporter.ref.NULL;
    be_call_options.be.db_vendor.length = 0;

    be_call_options.be.db_version.value = this.reporter.ref.NULL;
    be_call_options.be.db_version.length = 0;

    be_call_options.be.conn_type.value = this.reporter.ref.NULL;
    be_call_options.be.conn_type.length = 0;

    be_call_options.be.host.value = this.reporter.ref.NULL;
    be_call_options.be.host.length = 0;

    be_call_options.be.name.value = this.reporter.ref.NULL;
    be_call_options.be.name.length = 0;

    be_call_options.be.url.value = this.reporter.ref.NULL;
    be_call_options.be.url.length = 0;



    if(BackendType.BACKEND_TYPE_MYSQL <= this.backendType && this.backendType <= BackendType.BACKEND_TYPE_SSDB) {
        this.vendor_tmp = this.reporter.ref.allocCString(this.dbVendor);
        this.version_tmp = this.reporter.ref.allocCString(this.dbVersion);
        this.schema_tmp = this.reporter.ref.allocCString(this.dbSchema);
        be_call_options.be.db_vendor.value = this.vendor_tmp;
        be_call_options.be.db_version.value = this.version_tmp;
        be_call_options.be.db_schema.value = this.schema_tmp;
    }
    this.connType_tmp = this.reporter.ref.allocCString(this.connType);
    this.name_tmp = this.reporter.ref.allocCString(this.name);
    this.host_tmp = this.reporter.ref.allocCString(this.host);
    this.url_tmp = this.reporter.ref.allocCString(this.url);

    be_call_options.be.conn_type.value = this.connType_tmp;
    be_call_options.be.host.value = this.name_tmp;
    be_call_options.be.name.value = this.host_tmp;
    be_call_options.be.url.value = this.url_tmp;
    be_call_options.be.port = this.port;
    be_call_options.be.type = this.backendType;

    this.detail_tmp = this.reporter.ref.allocCString(this.detail);
    be_call_options.call_detail.detail.value = this.detail_tmp;
    be_call_options.call_detail.is_async = this.is_async;

    this.methodName_tmp = this.reporter.ref.allocCString(this.callMethod);
    be_call_options.class_method.method_name.value = this.methodName_tmp;
    be_call_options.start_time_stamp = 0;

    this.be_call = this.reporter.Instance();
    var cxReq = this.reporter.CrossHead();

    var ret = -1;
    if(BackendType.BACKEND_TYPE_HTTP <= this.backendType && this.backendType <= BackendType.BACKEND_TYPE_ES) {
        ret = this.reporter.so.start_rpc(this.transaction.instance.ref(), be_call_options.ref(), cxReq.ref(),this.be_call.ref());
        if (ret != 0) {
            this.agent.logger.error('failed to so.start_rpc.');
        }
        var arr = cxReq.data.toArray();
        arr.length = arr.indexOf(0x0);
        if(arr.length) {
            this.crossReq = Buffer.from(arr).toString();
        }
    } else {
        ret =this.reporter.so.start_be_call(this.transaction.instance.ref(), be_call_options.ref(), this.be_call.ref());
        if (ret != 0) {
            this.agent.logger.error('failed to so.start_be_call.');
        }
    }
    if(!!exitCallInfo.mqInfo) {
        this.setMQInfo(exitCallInfo.mqInfo);
    }

}

ExitCall.prototype.stop = function(options) {
    var self = this;
    var reporter = this.reporter;
    var txn = this.transaction;
    var be_call_finish_options = reporter.BeFinishOptions();

    self.error_detail = reporter.Str({value:'', length:0});
    self.error_name = reporter.Str({value:'', length:0});
    self.sql_explain_plan = reporter.Str({value:'', length:0});

    be_call_finish_options.flags = options.flags;
    be_call_finish_options.error_detail = self.error_detail;
    be_call_finish_options.error_name = self.error_name;
    be_call_finish_options.sql_explain_plan = self.sql_explain_plan;
    be_call_finish_options.finish_time_stamp = 0;

    if(options.flags) {
        if(options.errorDetail) {
            self.error_detail.value = options.errorDetail;
            be_call_finish_options.error_detail = self.error_detail;

        }
        if(options.errorName) {
            self.error_name.value = options.errorName;
            be_call_finish_options.error_name = self.error_name;
        }
    }
    if(self.sqlExpPlan) {
        self.sql_explain_plan.value = self.sqlExpPlan;
        be_call_finish_options.sql_explain_plan = self.sql_explain_plan;
    }

    var ret;
    if(BackendType.BACKEND_TYPE_HTTP <= self.backendType && self.backendType <= BackendType.BACKEND_TYPE_ES) {
        var crossRespHead = reporter.CrossHead();
        crossRespHead.data[0]=0;
        for(var i=0;this.crossResp && i<self.crossResp.length;i++) {
            crossRespHead.data[i]=self.crossResp.charAt(i);
        }
        ret = reporter.so.finish_rpc(txn.instance.ref(), self.be_call.ref(),be_call_finish_options.ref(),crossRespHead.ref());
        if (ret != 0) {
            self.agent.logger.error('failed to so.finish_rpc.');
        }
    } else {
        ret = reporter.so.finish_be_call(self.be_call.ref(), be_call_finish_options.ref());
        if (ret != 0) {
            self.agent.logger.error('failed to so.finish_be_call.');
        }
    }
};

ExitCall.prototype.detailWithoutParam = function () {
    var self = this;
    return self.operation;
};

ExitCall.prototype.needExplainPlan = function (agent) {
    if(!agent) {
        return false;
    }
    var self = this;
    var queryEndTime = Date.now();
    if(agent.config.config.backendConfig.enableSqlExplain
        && (queryEndTime - self.start_time) > agent.config.config.backendConfig.sqlExeTimeExplainThreshold) {
        return true;
    }
    return false;
}

ExitCall.prototype.collectPGExplainPlan = function (rows) {
    var self = this;
    var contents = '';
    rows.forEach(function (row) {
        contents += row['QUERY PLAN'];
        contents += '\n';
    });
    self.sqlExpPlan = 'QUERY PLAN\n'+contents;
}

ExitCall.prototype.collectMySqlExplainPlan = function (fields,results) {
    var self = this;
    var keys = [];
    fields.forEach(function (field, index){
        keys[index] = field.name;
    })
    var title = keys.join('\t');
    var content='';
    results.forEach(function (result) {
        let row = '';
        for(let i=0; i < keys.length; i++) {
            row = row +result[keys[i]];
            if( i == (keys.length -1) ) {
                row += '\n';
            } else {
                row += '\t';
            }
        }
        content +=row;
    })
    self.sqlExpPlan = title + '\n' + content;
}

ExitCall.prototype.setMQInfo = function (mqInfo) {
    var self =this;
    if(!!mqInfo.size && mqInfo.size <= 0) {
        return;
    }
    if (mqInfo.direction == MQDIRECTION.PRODUCER) {
        this.detail =MQDETAILPREFIX.PRODUCER + this.detail;
    }
    if (mqInfo.direction == MQDIRECTION.CONSUMER) {
        this.detail = MQDETAILPREFIX.CONSUMER + this.detail;
    }

    var mq =self.reporter.MQInfo();
    mq.direction = mqInfo.direction;
    mq.mq_size = mqInfo.size;
    self.reporter.so.set_mq_info(self.be_call.ref(),mq.ref());
    var be_call_options = new this.reporter.BeCallOptions();
    be_call_options.class_method.class_name.value = this.reporter.ref.NULL;
    be_call_options.class_method.class_name.length = 0;

    be_call_options.class_method.method_name.value = this.reporter.ref.NULL;
    be_call_options.class_method.method_name.length = 0;

    be_call_options.call_detail.detail.value = this.reporter.ref.NULL;
    be_call_options.call_detail.detail.length = 0;

    be_call_options.be.db_schema.value = this.reporter.ref.NULL;
    be_call_options.be.db_schema.length = 0;

    be_call_options.be.db_vendor.value = this.reporter.ref.NULL;
    be_call_options.be.db_vendor.length = 0;

    be_call_options.be.db_version.value = this.reporter.ref.NULL;
    be_call_options.be.db_version.length = 0;

    be_call_options.be.conn_type.value = this.reporter.ref.NULL;
    be_call_options.be.conn_type.length = 0;

    be_call_options.be.host.value = this.reporter.ref.NULL;
    be_call_options.be.host.length = 0;

    be_call_options.be.name.value = this.reporter.ref.NULL;
    be_call_options.be.name.length = 0;

    be_call_options.be.url.value = this.reporter.ref.NULL;
    be_call_options.be.url.length = 0;

    this.connType_tmp = this.reporter.ref.allocCString(this.connType);
    this.name_tmp = this.reporter.ref.allocCString(this.name);
    this.host_tmp = this.reporter.ref.allocCString(this.host);
    this.url_tmp = this.reporter.ref.allocCString(this.url);

    be_call_options.be.conn_type.value = this.connType_tmp;
    be_call_options.be.host.value = this.name_tmp;
    be_call_options.be.name.value = this.host_tmp;
    be_call_options.be.url.value = this.url_tmp;
    be_call_options.be.port = this.port;
    be_call_options.be.type = this.backendType;

    this.detail_tmp = this.reporter.ref.allocCString(this.detail);
    be_call_options.call_detail.detail.value = this.detail_tmp;
    be_call_options.call_detail.is_async = this.is_async;

    this.methodName_tmp = this.reporter.ref.allocCString(this.callMethod);
    be_call_options.class_method.method_name.value = this.methodName_tmp;
    be_call_options.start_time_stamp = 0;
    
    var ret =self.reporter.so.update_be_call(self.be_call.ref(), be_call_options.ref());
}
function remoteBackendName(agent,path) {
    var config = agent.config.config;
    var cut_path = path.replace(/^\//,'').split('/');
    switch (config.appConfig.remoteCallGetURISegmentType)
    {
        case GETURISEGMENTTYPE.GET_URI_SEGMENT_TYPE_FULL:
        {
            return path;
        }
        case GETURISEGMENTTYPE.GET_URI_SEGMENT_TYPE_START:
        {
            return '/'+cut_path.slice(0,config.appConfig.remoteCallURISegments).join('/');
        }
        case GETURISEGMENTTYPE.GET_URI_SEGMENT_TYPE_END:
        {
            return '/'+cut_path.slice(cut_path.length-config.appConfig.remoteCallURISegments,cut_path.length).join('/');
        }
    }

}

exports.remoteBackendName = remoteBackendName;
exports.ExitCall = ExitCall;
