var path = require('path');
var loggerfuncs = require('../core/logger').Logger_funcs;


Object.defineProperty(global, '__stack', {
    get: function(){
        var orig = Error.prepareStackTrace;
        var origLimit = Error.stackTraceLimit;
        Error.prepareStackTrace = function(_, stack){ return stack; };
        Error.stackTraceLimit = 5;
        var err = new Error();
        Error.captureStackTrace(err, arguments.callee);
        var stack = err.stack;
        Error.prepareStackTrace = orig;
        Error.stackTraceLimit = origLimit;
        return stack;
    }
});

//time come from profiler
function Segment(segmentName, parent, isCallback, needTrace) {

    this.isCallback = isCallback;
    this.spanId = undefined;
    this.span = undefined;
    var transaction;
    this.parent = null;
    if (parent.hasOwnProperty('entryType')) {
        transaction = parent;
    } else {
        var tmp= parent;
        while(tmp.reuse) {
            tmp = tmp.parent;
        }
        transaction = tmp.transaction;
        this.parent = tmp;
    }
    if(!transaction || transaction.isFinished) {
        return null;
    }
    this.isAlived = undefined;
    this.childs = [];
    this.transaction = transaction;
    transaction.lastSegment = this;
    this.id = undefined;
    this.childIds = [];
    this.isRoot = false;

    var clsMethod = segmentName.split('.');
    this.segmentName = clsMethod.length > 1 ? clsMethod[1] : segmentName;
    this.className = clsMethod.length > 1 ? clsMethod[0] : undefined;
    this.filename = undefined;
    this.lineNumber = 0;
    this.methodName = this.segmentName;

    this.tags = [];
    this.errors = [];
    transaction.segmentNum++;
}

exports.Segment = Segment;

Segment.prototype.isEliminate = function(filePath) {
    var bonree_path = __filename.replace(/lib\/models.*$/,'');
    var bonree_filter = new RegExp(bonree_path);
    var bonree_nativefilter = new RegExp(/bonree/);
    return !!bonree_filter.exec(filePath) || !!bonree_nativefilter.exec(filePath);
};

Segment.prototype.SpliteFuncName = function(fileName, methodNm) {
    var funcInfo = {clsNm:null, funcNm:null};
    if(!fileName) return funcInfo;
    if(methodNm == '<anonymouse>') this.className = 'Function';
    var clasfun = methodNm.split('.');
    var shortpath = fileName.split(path.sep).slice(-2).join(path.sep);
    var len = clasfun.length;
    if(len > 1) {
        funcInfo.clsNm = shortpath + '-' + (!!this.className ? this.className : clasfun[len - 2]);
        funcInfo.funcNm = clasfun[len - 1];
    } else {
        if (!this.className) this.className = path.basename(shortpath, '.js');
        funcInfo.clsNm = shortpath + '-' + this.className;
        funcInfo.funcNm = methodNm;
    }

    return funcInfo;
};

Segment.prototype.getNodeInfo = function() {

    if (!this.transaction) return false;

    if (!this.reuse && this.transaction.needTrace) {

        var curStackInfo = __stack;
        for (var elem in curStackInfo) {
            if(process.version<'v7.0.0'){
                if(!curStackInfo[elem] || !curStackInfo[elem].getFileName){
                    continue;
                }
            }else{
                if (!curStackInfo[elem] || !curStackInfo[elem].__proto__ ||
                    !curStackInfo[elem].__proto__.getFileName) {
                    continue;
                }
            }

            var filename = curStackInfo[elem].getFileName();
            if (!this.isEliminate(filename)) {
                var methodNm = this.segmentName;
                var funcInfo = this.SpliteFuncName(filename, methodNm);

                if (funcInfo.funcNm == null) continue;
                this.className = funcInfo.clsNm;
                this.methodName = funcInfo.funcNm;
                this.filename = filename;
                this.lineNumber = curStackInfo[elem].getLineNumber();
                break;
            }
        }
    }

    // !!this.parent&&this.parent.add(this);

    return true;
};

Segment.prototype.setBeTags = function (exitcall) {
    if(!exitcall || !this.span ||!this.transaction || !this.transaction.needTrace) {
        return;
    }

    var self = this;
    var reporter = self.transaction.agent.reporter;
    var tags = [];

    if(!!exitcall.host && exitcall.host !== 'unknown' && !!exitcall.port && !!exitcall.connType) {
        tags.push({
            key: 'peer.address', value: exitcall.connType +'://' + exitcall.host + ':' + exitcall.port
        }, {
            key: 'peer.hostname', value: exitcall.host
        });
    }
    if(!!exitcall.host) {
        tags.push({
            key: 'peer.hostname', value: exitcall.host
        });
    }
    if(!!exitcall.dbSchema) {
        tags.push({key:'db.instance', value:exitcall.dbSchema});
    }
    if(!!exitcall.detail) {
        tags.push({key:'db.statement', value:exitcall.detail});
    }
    if(!!exitcall.connType) {
        tags.push({key: 'db.type', value: exitcall.connType});
        tags.push({key: 'component', value: exitcall.connType});
    }


    tags.forEach(function (tag) {
        if(!tag.key || !tag.value) {
            return;
        }
        if(tag.value==="unknown") {
            return;
        }
        var key = reporter.Str({value:'', length:0});
        var val = reporter.Str({value:'', length:0});

        key.value = tag.key;
        val.value = tag.value.toString();
        key.length = tag.key.length;
        val.length = tag.value.toString().length;
        var ret = reporter.so.set_tag(self.span.ref(), key.ref(), val.ref());
        if( ret !=0) {
            self.transaction.agent.logger.error("faild to so.set_tag.")
        }
    });
};

Segment.prototype.setError = function (summary,detail) {
    if(!summary || !detail ||!this.span || !this.transaction||!this.transaction.needTrace) {
        return;
    }
    var self = this;
    var reporter = self.transaction.agent.reporter;
    var errSummary = reporter.Str({value:'', length:0});
    var errDetail = reporter.Str({value:'', length:0});
    var spanErr = reporter.TraceError();
    spanErr.error_details = errDetail;
    spanErr.error_summary = errSummary;


    if(summary) {
        spanErr.error_summary.value = summary;
    }
    if(detail) {
        spanErr.error_details.value = detail;
    }
    var logger = this.transaction.agent.logger;

    var ret = reporter.so.set_trace_error(self.span.ref(), spanErr.ref());
    if(ret != 0 ) {
        logger.error('failed to so.set_trace_error.');
    }
};

Segment.prototype.start = function() {
    if(!this.transaction) {
        return;
    }
    self = this;
    self.isAlived = true;

    if(!self.transaction.needTrace || !self.transaction.tracer || self.reuse) {
        return;
    }

    var logger = self.transaction.agent.logger;
    var reporter = self.transaction.agent.reporter;
    var tracer = self.transaction.tracer;
    var startSpanOptions = new reporter.StartSpanOptions();

    startSpanOptions.thread_info.thread_name.value = reporter.ref.NULL;
    startSpanOptions.thread_info.thread_name.length = 0;
    startSpanOptions.thread_info.thread_id = 0;

    startSpanOptions.method_info.class_name.value = reporter.ref.NULL;
    startSpanOptions.method_info.class_name.length = 0;

    startSpanOptions.method_info.method_name.value = reporter.ref.NULL;
    startSpanOptions.method_info.method_name.length = 0;

    startSpanOptions.method_info.ret_value.value = reporter.ref.NULL;
    startSpanOptions.method_info.ret_value.length = 0;

    startSpanOptions.method_info.params.value = reporter.ref.NULL;
    startSpanOptions.method_info.params.length = 0;

    //将引用保存，防止被自动回收内存
    self.tmp1 = reporter.ref.allocCString(self.className);
    self.tmp2 = reporter.ref.allocCString(self.methodName);
    startSpanOptions.method_info.class_name.value = self.tmp1;
    startSpanOptions.method_info.method_name.value = self.tmp2;
    startSpanOptions.method_info.line_number = self.lineNumber;


    startSpanOptions.start_time_stamp = 0;

    var ret = -1;
    var span = reporter.Instance();
    if(this.parent &&!!this.parent.spanId) {
        ret = reporter.so.start_child_span(tracer.ref(), startSpanOptions.ref(), this.parent.spanId.ref(), span.ref());
        if(ret != 0) {
            logger.error('failed to so.start_child_span.');
            return;
        }

        var Id = reporter.SpanId();
        ret = reporter.so.get_span_id(span.ref(), Id.ref());
        if(ret != 0) {
            logger.error('failed to so.get_span_id.');
            return;
        }
        this.spanId = Id;
        this.span = span;

    } else {
        ret = reporter.so.start_span(tracer.ref(), startSpanOptions.ref(), span.ref());
        if(ret != 0) {
            logger.error('failed to so.start_span');
            return;
        }
        var Id = reporter.SpanId();
        ret = reporter.so.get_span_id(span.ref(), Id.ref());
        if(ret != 0) {
            logger.error('failed to so.get_span_id.');
            return;
        }
        self.span = span;
        self.spanId = Id;
    }
    if(!self.reuse && !!self.className) {
        !!self.parent&&self.parent.add(self);
    }
};

Segment.prototype.done = function() {
    var self = this;
    if(!self.transaction) return true;
    if(!self.isAlived) return true;
    self.isAlived = false;

    if(!self.span || !this.transaction.needTrace) return true;

    var ret = -1;
    var reporter = self.transaction.agent.reporter;

    ret = reporter.so.finish_span(this.span.ref(), null);
    if (ret != 0) {
        return false;

    }
    return true;
};

Segment.prototype.add = function(child) {
    var self = this;
    var txn = this.transaction;
    txn.lastSegment = child;
    self.childs.push(child);
};

Segment.prototype.updateAllTime = function() {
    var segments = [this];
    while(segments.length > 0) {
        var segment = segments.pop();
        if (segment.isAlived && !segment.done()) {
            if (!!this.transaction) {
                this.transaction.agent.logger.error('segment failed @ updateTime');
            }
            return false;
        }
        segments = segments.concat(segment.childs);
    }
    return true;
};

Segment.prototype.linkBeCall = function (exitcall) {
    var self = this;
    if(!self.span || !exitcall || exitcall.segmentUsed ||!self.transaction || !self.transaction.needTrace) {
        return;
    }
    var reporter = this.transaction.agent.reporter;
    var ret = -1;
    this.setBeTags(exitcall);
    if(!!exitcall.be_call) {
        ret = reporter.so.link_to_be_call(self.span.ref(), exitcall.be_call.ref());
        if(ret != 0) {
            this.transaction.agent.logger.error('failed to so.link_to_be_call.');
        }
    }
    exitcall.segmentUsed = true;
};