var Segment = require('../models/segment').Segment;
var AgentType = require('../common/constant').AgentType;
var TRACEMODE = require('../common/constant').TraceMode;
var ERRORCODE = require('../common/constant').ErrCode;

//add new
function Tracer(agent) {
    this.agent = agent;
    this.curSegment = null;
    this.lastSegment = null;
    this.nextTransactionId = 1;
}
exports.Tracer = Tracer;

Tracer.prototype.startTransactionWithoutBe= function() {
    var self = this;
    var ret = -1;
    if(!this.agent.initialized) {
        self.agent.logger.error('startTransactionWithoutBe failed, agent is not initialized!');
        return null;
    }

    var transaction = self.agent.profiler.createTransaction(null,undefined,false,undefined,true);
    if(!transaction) {
        self.agent.logger.error('failed to createTransaction!');
        return null;
    }
    if (!!this.agent.asyncTracer) {
        this.agent.asyncTracer.setTransactionId(transaction.id);
    }
    this.nextTransactionId = this.nextTransactionId + 1;
    return transaction;
};

Tracer.prototype.startBGTaskTransaction = function (trans_name,btType) {
    var self = this;
    var ret = -1;
    var reporter = self.agent.reporter;
    if(!this.agent.initialized) {
        self.agent.logger.error('in startTransaction, agent not initialized!');
        console.log('in startTransaction, agent not initialized!');

        return null;
    }
    var transaction = this.agent.profiler.createBGTaskTranaction(trans_name, btType);
    if(!transaction) {
        self.agent.logger.error('createTransaction failed!');
        return null;
    }

    if (!!this.agent.asyncTracer) {
        this.agent.asyncTracer.setTransactionId(transaction.id);
    }

    if(transaction.needTrace) {
        transaction.tracer = reporter.Instance();
        ret = reporter.so.get_or_create_tracer(transaction.instance.ref(), transaction.tracer.ref());
        if(ret != 0) {
            self.agent.logger.error('failed to get_or_create_tracer failed!');
            return null;
        }
    }

    this.nextTransactionId = this.nextTransactionId + 1;
    return transaction;
}

Tracer.prototype.startTransaction = function(req,btType, moduleName) {
    var self = this;
    var ret = -1;
    var reporter = self.agent.reporter;
    if(!this.agent.initialized) {
        self.agent.logger.error('in startTransaction, agent not initialized!');
        return null;
    }
    var transaction = this.agent.profiler.createTransaction(req,btType,moduleName);

    if(!transaction) {
        self.agent.logger.error('createTransaction failed!');
        return null;
    }

    if (!!this.agent.asyncTracer) {
        this.agent.asyncTracer.setTransactionId(transaction.id);
    }

    transaction.segmentNum = 1;
    var segment = new Segment(btType, transaction);
    segment.isRoot = true;
    transaction.baseSegment = segment;
    this.curSegment = segment;

    if(transaction.needTrace) {
        transaction.tracer = reporter.Instance();
        ret = reporter.so.get_or_create_tracer(transaction.instance.ref(), transaction.tracer.ref());
        if(ret != 0) {
            self.agent.logger.error('failed to get_or_create_tracer failed!');
            return null;
        }
    }

    this.nextTransactionId = this.nextTransactionId + 1;
    return transaction;
};

Tracer.prototype.endTransaction = function (transaction) {
    this.agent.profiler.endTransaction(transaction);
    this.curSegment = null;
};

Tracer.prototype.removeTransaction = function(transaction) {
  if (!transaction) return;
  var id = transaction.id;
  this.curSegment = null;
  if (!this.agent.profiler.transactions[id]) return;
  transaction.clear();
  delete this.agent.profiler.transactions[id];
};

Tracer.prototype.getTransaction = function() {
    var returnValue1;
    var returnValue2;
    if(!!this.agent.asyncTracer){
        var txnId=this.agent.asyncTracer.getTransactionId();
        if(!txnId && !!this.nextTransactionId){
            txnId=this.nextTransactionId-1;
        }
        returnValue1=this.agent.profiler.transactions[txnId];
    }
    if(!! this.curSegment && this.curSegment.isAlived) {
        returnValue2=this.curSegment.transaction;
    }
    return returnValue1 || returnValue2;
};

Tracer.prototype.getParentSegment = function(txn) {
    var curSegment = this.curSegment;
    if (!!txn) {
        return txn.getActiveSegment();
    }

    if (!curSegment  || !curSegment.transaction || !curSegment.isAlived) {
        return null;
    }

    return curSegment;
};



Tracer.prototype.createSegment = function(segmentName, parent, isCallback, needTrace) {
    if(!parent || !parent.transaction) {
        return null;
    }
    if (parent.transaction.segmentNum > parent.transaction.maxSegmentNum) {
        this.agent.logger.error('current segment num:' + parent.transaction.segmentNum + ' has reached the maximum, max:' + parent.transaction.maxSegmentNum);
        return null;
    }
    var segment = new Segment(segmentName, parent, isCallback, needTrace);
    this.curSegment = segment;
    return segment;
};
