var BTTYPE =  require('../common/constant').BTType;
var nodeModuleVersion = process.versions.modules;
var napi_verson = process.versions.napi;
var ffi;
var ref;
var refStruct;
var refArray;

var version = process.version.split('.');

const {
    AgentType,
    LEN
} = require('../common/constant');

var on_agent_state_changed;

exports.Reporter = Reporter;

function Reporter(agent) {
	
    this.agent = agent;
    this.so = {};
    this.config = {};
    this.transaction = {};
	
	if (nodeModuleVersion > 64 && !!napi_verson && napi_verson > 3) {
                if(napi_verson >= 5) {
			ref = require('ref-napi3');
			ffi = require('ffi-napi3');
		} else {
			ref = require('ref-napi2');
			ffi = require('ffi-napi2');
		}

		refStruct = require('ref-struct-di')(ref);
		refArray = require('ref-array-di')(ref);
	} else if(nodeModuleVersion == 48 || nodeModuleVersion == 57 || nodeModuleVersion == 64) {
		var path = '../../ffi-ref/node_modules/';
		ffi = require(path + 'ffi');
		ref = require(path + 'ref');
		refStruct = require(path + 'ref-struct');
		refArray = require(path + 'ref-array');
	}
	this.ref = ref;
    this.Int = ref.alloc(ref.types.uint, 0);


    this.Str = refStruct({
        'value': 'string',
        'length': ref.types.uint,
    });

    this.Instance = refStruct({
        'ins': ref.refType('void'),
    });

    this.ContainerInfo = refStruct({
        'agent_version': this.Str,
        'agent_install_timestamp': 'ulonglong',
        'agent_install_dir': this.Str,
        'listens': this.Str,
        'container_version': this.Str,
        'container_install_dir': this.Str,
        'vm_info': this.Str,
        'vm_startup_options': this.Str,
        'vm_system_options': this.Str,
    });

    this.SdkLogCfg = refStruct({'level': ref.types.int, 'log_dir': this.Str,});

    this.SdkCacheCfg = refStruct({'cache_dir': this.Str,});

    this.SdkOptions = refStruct({
        "agent_type": ref.types.int,
        "pid": ref.types.ulong,
        "flags": ref.types.int,
        "log_cfg": this.SdkLogCfg,
        "container_info": ref.refType(this.ContainerInfo),
        "cfg_dir": this.Str,
    });

    this.BtOptions = refStruct({
       // "id": ref.types.uint,
        "type": ref.types.int,
        "name": this.Str,
        "internal_name": this.Str,
        "uri": this.Str,
        "start_time_stamp":'ulonglong'
    });

    this.HttpReqData = refStruct({
        "url": this.Str,
        "http_method": this.Str,
        "parameters": this.Str,
        "req_headers": this.Str,
        "req_content": this.Str,
    });

    this.HttpRespData = refStruct({
        "status_code": this.Str,
        "res_header": this.Str,
        "resp_content": this.Str,
    });

    this.BtError = refStruct({
        "time_stamp":'ulonglong',
        "error_type": ref.types.int,
        "error_name": this.Str,
        "error_type_str": this.Str,
        "error_sumary": this.Str,
        "error_details": this.Str,
    });


    this.BtFinishOptions = refStruct({
        "flags": ref.types.uint,
        "finish_time_stamp": 'ulonglong',
    });

    this.CrossHead = refStruct({
        "data":refArray('char',LEN.CROSS_HEAD),
    });

    this.CrossReqOptions = refStruct({
        "is_sampling": ref.types.int,
        "cross_req": this.CrossHead,
    });

    this.GenCrossReqStrOptions = refStruct({
        "be_type": ref.types.int,
        "is_async": ref.types.int,
        "net_rpc_id": this.Str,
    });

    this.CrossRspOptions = refStruct({
        "cross_rsp": this.Str,
    });

    this.Be = refStruct({
        "type": ref.types.int,
        "conn_type": this.Str,
        "name": this.Str,
        "host": this.Str,
        "port": ref.types.uint,
        "url": this.Str,
        "db_vendor": this.Str,
        "db_version": this.Str,
        "db_schema": this.Str,
    });

    this.CallDetail = refStruct({
        "is_async": ref.types.int,
        "detail": this.Str,
    });

    this.ClassMethod = refStruct({
        "class_name": this.Str,
        "method_name": this.Str,
    });

    this.BeCallOptions = refStruct({
        "be": this.Be,
        "call_detail": this.CallDetail,
        "class_method": this.ClassMethod,
        "start_time_stamp": 'ulonglong',
    });

    this.BeFinishOptions = refStruct({
        "flags": ref.types.int,
        "finish_time_stamp" : 'ulonglong',
        "error_name" : this.Str,
        "error_detail" : this.Str,
        "sql_explain_plan" : this.Str,
    });

    this.AgentState = refStruct({
        "state": ref.types.int,
    });

    this.BtNamingCfg = refStruct({
        "type": ref.types.int,
        "enable": ref.types.int,
        "properties": this.Str
    });

    this.BtThresholdConfig = refStruct({
        "bt_threshold": ref.types.uint,
        "bt_threshold_slow": ref.types.uint,
        "bt_threshold_very_slow": ref.types.uint
    });

    this.SpanThreadInfo = refStruct({
        'thread_name': this.Str,
        'thread_id': 'ulonglong'
    });

    this.SpanMethodInfo = refStruct({
        'class_name': this.Str,
        'method_name': this.Str,
        'ret_value': this.Str,
        'params': this.Str,
        'line_number': ref.types.uint
    });

    this.StartSpanOptions = refStruct({
        'thread_info': this.SpanThreadInfo,
        'method_info': this.SpanMethodInfo,
        'start_time_stamp': 'ulonglong',

    });

    this.FinishSpanOptions = refStruct({
        'finish_time_stamp' : 'ulonglong',
    });
    this.TraceError = refStruct({
        'error_summary': this.Str,
        'error_details': this.Str
    });

    this.TraceLog = refStruct({
        'level': this.Str,
        'log_msg': this.Str
    });

    this.SpanId = refStruct({
        'data': refArray(ref.types.char, LEN.SPAN_ID)
    });

    this.TraceId = refStruct({
        'data': refArray(ref.types.char, LEN.TRACE_ID+1)
    });
    this.MQInfo = refStruct({
        'mq_size': 'ulonglong',
        'direction': 'int'
    });
}

// 初始化sdk接口方法，数据结构
Reporter.prototype.init = function () {
    var self = this;


    self.sdk = self.Instance();
    self.cfg = self.Instance();
    self.app_cfg = self.Instance();
    self.bt_filter_cfg = self.Instance();
    self.bt_cfg = self.Instance();
    self.error_cfg = self.Instance();
    self.trace_cfg = self.Instance();
    self.js_cfg = self.Instance();
    self.backend_cfg = self.Instance();
    self.bt_naming_cfg = self.BtNamingCfg();

    var so_path;
    so_path = process.env.BONREE_SMARTAGENT_SDK_PATH;

    self.agent.logger.info('importing c sdk from ' + so_path);
    self.so = ffi.Library(so_path, {
        'open_sdk': ['int', [ref.refType(self.SdkOptions)]],
        'close_sdk': ['int', []],
        'get_config': ['int', [ref.refType(self.Instance)]],
        'open_transaction': ['int', [ref.refType(self.Instance)]],
        'close_transaction': ['int', [ref.refType(self.Instance)]],
        'get_or_create_bt': ['int', [ref.refType(self.Instance), ref.refType(self.BtOptions), ref.refType(self.HttpReqData),ref.refType(self.Instance)]],
        'finish_bt': ['int', [ref.refType(self.Instance), ref.refType(self.BtFinishOptions), ref.refType(self.HttpRespData)]],
        'get_or_create_cross_req': ['int', [ref.refType(self.Instance), ref.refType(self.CrossReqOptions), ref.refType(self.Instance)]],
        'gen_next_rpc_id': ['int', [ref.refType(self.Instance), ref.refType(self.Str)]],
        'free_next_rpc_id': ['int', [ref.refType(self.Str)]],
        'start_rpc': ['int', [ref.refType(self.Instance), ref.refType(self.BeCallOptions), ref.refType(self.CrossHead),ref.refType(self.Instance)]],
        'start_be_call': ['int', [ref.refType(self.Instance), ref.refType(self.BeCallOptions), ref.refType(self.Instance)]],
        'gen_cross_req_str': ['int', [ref.refType(self.Instance), ref.refType(self.GenCrossReqStrOptions), ref.refType(self.Str)]],
        'free_cross_req_str': ['int', [ref.refType(self.Str)]],
        'get_or_create_cross_resp': ['int', [ref.refType(self.Instance), ref.refType(self.CrossRspOptions), ref.refType(self.Instance)]],
        'get_target_agent_id': ['ulong', [ref.refType(self.Instance)]],
        'gen_cross_resp_str': ['int', [ref.refType(self.Instance), ref.refType(self.Str)]],
        'free_cross_resp_str': ['int', [ref.refType(self.Str)]],
        'gen_cross_resp_head': ['int', [ref.refType(self.Instance),ref.refType(self.CrossHead)]],
        'finish_be_call': ['int', [ref.refType(self.Instance), ref.refType(self.BeFinishOptions)]],
        'finish_rpc': ['int', [ref.refType(self.Instance), ref.refType(self.Instance), ref.refType(self.BeFinishOptions), ref.refType(self.CrossHead)]],
        'reg_agent_state_callback': ['int', ['pointer']],
        // config method
        'update_container_info': ['int', [ref.refType(self.ContainerInfo)]],
        'agent_enable': ['int', [ref.refType(self.Instance)]],
        'get_agent_id': ['int', [ref.refType(self.Instance)]],
        'get_app_config': ['int', [ref.refType(self.Instance), ref.refType(self.Instance)]],
        'get_error_config': ['int', [ref.refType(self.Instance), ref.refType(self.Instance)]],
        'get_trace_config': ['int', [ref.refType(self.Instance), ref.refType(self.Instance)]],
        'get_bt_filter_config': ['int', [ref.refType(self.Instance), ref.refType(self.Instance)]],
        'get_bt_configs': ['int', [ref.refType(self.Instance), ref.refType(self.Instance)]],
        'get_inject_js_config': ['int', [ref.refType(self.Instance), ref.refType(self.Instance)]],
        'get_backend_config': ['int', [ref.refType(self.Instance), ref.refType(self.Instance)]],
        // app_config method
        'get_instrument_level': ['int', [ref.refType(self.Instance)]],
        'data_aggregate_enable': ['int', [ref.refType(self.Instance)]],
        'get_bt_naming_cfg': ['int', [ref.refType(self.Instance), ref.types.int, ref.refType(self.BtNamingCfg)]],
        'free_bt_naming_cfg': ['int', [ref.refType(self.BtNamingCfg)]],
        // bt_filter_config method
        'in_bt_blacklist': ['int', [ref.refType(self.Instance), ref.refType(self.Str)]],
        'in_bt_whitelist': ['int', [ref.refType(self.Instance), ref.refType(self.Str)]],
        'in_snapshot_blacklist': ['int', [ref.refType(self.Instance), ref.refType(self.Str)]],
        'get_snapshot_blacklist': ['int', [ref.refType(self.Instance), ref.refType(ref.refType(self.Str)), ref.refType('int')]],
        'get_bt_blacklist': ['int', [ref.refType(self.Instance), ref.refType(ref.refType(self.Str)), ref.refType('int')]],
        'get_bt_whitelist': ['int', [ref.refType(self.Instance), ref.refType(ref.refType(self.Str)), ref.refType('int')]],
        // bt_configs method
        'get_default_bt_cfg': ['int', [ref.refType(self.Instance), ref.refType(self.Instance)]],
        'get_bt_cfg': ['int', [ref.refType(self.Instance), ref.refType(self.Str), ref.refType(self.BtThresholdConfig)]],
        'get_bt_cfg_list': ['int', [ref.refType(self.Instance), ref.refType(ref.refType(self.Str)), ref.refType('int')]],
        'free_bt_cfg_list': ['int', [ref.refType(self.Instance), 'int']],
        // call_graph_config method
        'get_method_time_threshold': ['uint', [ref.refType(self.Instance)]],
        'get_sql_query_time_threshold': ['uint', [ref.refType(self.Instance)]],
        'in_exclude_names': ['int', [ref.refType(self.Instance), ref.refType(self.Str)]],
        'in_include_names': ['int', [ref.refType(self.Instance), ref.refType(self.Str)]],
        'get_exclude_names': ['int', [ref.refType(self.Instance), ref.refType(ref.refType(self.Str)), ref.refType('int')]],
        'get_include_names': ['int', [ref.refType(self.Instance), ref.refType(ref.refType(self.Str)), ref.refType('int')]],
        // error_config method
        'ignore_exception': ['int', [ref.refType(self.Instance), ref.refType(self.Str)]],
        'ignore_err_msg': ['int', [ref.refType(self.Instance), ref.refType(self.Str)]],
        //'get_ignore_err_msg_list': ['int', [ref.refType(self.Instance), ref.types.uint, ref.refType(self.Str)]],
        //'get_ignore_exception_list': ['int', [ref.refType(self.Instance), ref.types.uint, ref.refType(self.Str)]],
        'get_custom_http_code_desc': ['int', [ref.refType(self.Instance), ref.types.uint, ref.refType(self.Str), 'uint *']],
        'free_custom_http_code_desc': ['int', [ref.refType(self.Str)]],
        'need_mark_trans_as_error_on_error_msglog': ['int', [ref.refType(self.Instance)]],
        'default_http_error_code_enable': ['int', [ref.refType(self.Instance)]],
        'detect_logging_enable': ['int', [ref.refType(self.Instance)]],
        'detect_log_book_enable': ['int', [ref.refType(self.Instance)]],
        'detect_eliot_enable': ['int', [ref.refType(self.Instance)]],
        // js_config method
        'get_inject_js': ['int', [ref.refType(self.Instance), ref.refType(self.Str)]],
        'free_inject_js': ['int', [ref.refType(self.Str)]],
        // trace_config method
        'trace_enable': ['int', [ref.refType(self.Instance)]],
        'get_trace_mode': ['int', [ref.refType(self.Instance)]],
        'get_default_gatherer_percent': ['uint', [ref.refType(self.Instance)]],
        'check_header_need_collect': ['int', [ref.refType(self.Instance), ref.refType(self.Instance)]],
        'get_need_collect_headers': ['int', [ref.refType(self.Instance), ref.refType(ref.refType(self.Instance)),ref.refType('int')]],
        'collect_http_req_headers_enable': ['int', [ref.refType(self.Instance)]],
        'enable_add_traceid_to_applog': ['int', [ref.refType(self.Instance)]],
        'collect_http_params_enable': ['int', [ref.refType(self.Instance)]],
        'get_collect_request_content_len': ['uint', [ref.refType(self.Instance)]],
        'get_collect_response_content_len': ['uint', [ref.refType(self.Instance)]],
        'get_smart_mode_level': ['uint', [ref.refType(self.Instance)]],
        //backend_config method
        'get_sql_record_type': ['int', [ref.refType(self.Instance)]],
        'get_sql_exe_time_threshold': ['uint', [ref.refType(self.Instance)]],
        'get_sql_exe_time_explain_threshold': ['uint', [ref.refType(self.Instance)]],
        'get_sql_max_length': ['uint', [ref.refType(self.Instance)]],
        'get_sql_max_records': ['uint', [ref.refType(self.Instance)]],
        'get_nosql_exe_time_threshold': ['uint', [ref.refType(self.Instance)]],
        'enable_capture_nosql_params': ['uint', [ref.refType(self.Instance)]],
        'get_nosql_max_param_length': ['uint', [ref.refType(self.Instance)]],
        'get_remote_call_exe_time_threshold': ['uint', [ref.refType(self.Instance)]],
        'get_remote_call_get_uri_seg_type': ['int', [ref.refType(self.Instance)]],
        'get_remote_call_uri_segments': ['uint', [ref.refType(self.Instance)]],
        'enable_sql_explain': ['uint', [ref.refType(self.Instance)]],
        // transaction method
        'get_or_create_tracer': ['int', [ref.refType(self.Instance), ref.refType(self.Instance)]],
        // tracer method
        'start_span': ['int', [ref.refType(self.Instance), ref.refType(self.StartSpanOptions), ref.refType(self.Instance)]],
        'start_child_span': ['int', [ref.refType(self.Instance),ref.refType(self.StartSpanOptions),ref.refType(self.SpanId),ref.refType(self.Str)]],
        'finish_tracer': ['int', [ref.refType(self.Instance)]],

        // span method
        'link_to_be_call': ['int', [ref.refType(self.Instance), ref.refType(self.Instance)]],
        'set_tag': ['int', [ref.refType(self.Instance), ref.refType(self.Str), ref.refType(self.Str)]],
        'set_trace_tag': ['int', [ref.refType(self.Instance), ref.refType(self.Str), ref.refType(self.Str)]],
        'set_trace_error': ['int', [ref.refType(self.Instance), ref.refType(self.TraceError)]],
        'finish_span': ['int', [ref.refType(self.Instance),ref.refType(self.FinishSpanOptions)]],
        'get_span_id': ['int', [ref.refType(self.Instance),ref.refType(self.SpanId)]],
        'add_bt_error':['int',[ref.refType(self.Instance),ref.refType(self.BtError)]],
        'get_trace_id': ['int', [ref.refType(self.Instance),ref.refType(self.TraceId)]],
        'set_mq_info': ['int', [ref.refType(self.Instance),ref.refType(self.MQInfo)]],
        'update_be_call': ['int', [ref.refType(self.Instance),ref.refType(self.BeCallOptions)]],
    });
    self.agent.logger.info('c sdk has been imported.');
    //初始化sdk
    return self.openSdk();
};

// 初始化sdk
Reporter.prototype.openSdk = function () {
    var self = this;
    var options = self.SdkOptions();
    options.agent_type = AgentType.AGENT_NODEJS;
    options.pid = self.agent.config.config.containerInfo.processId;
    options.flags = 0;
    var log_cfg = self.SdkLogCfg();
    options.log_cfg = log_cfg;
    var log_dir = self.Str({value:'', length:0});
    options.log_cfg.log_dir = log_dir;
    options.log_cfg.level = 0;
    var cfg_dir = self.Str({value:'', length:0});//为空默认取 sdkdir/../conf/bonree-sdk.yml
    options.cfg_dir = cfg_dir;

    var updateInfo = self.initContainerInfo();
    options.container_info = updateInfo.ref();

    var isOpen = self.so.open_sdk(options.ref());
    if(isOpen !=0) {
        self.agent.logger.error("faild to load sdk.")
        return false;
    }

    // var ret = self.so.update_container_info(updateInfo.ref());
    // if(ret !=0) {
    //     self.agent.logger.error("faild to update_container_info.")
    //     return false;
    // }

    var callback = onAgentStateChanged.bind(this);
    on_agent_state_changed = ffi.Callback('int',
        [ref.refType(self.AgentState)],
        callback);
    if (isOpen === 0) {
        ret = self.so.reg_agent_state_callback(on_agent_state_changed);
        if(ret !=0) {
            self.agent.logger.error("faild to reg_agent_state_callback.")
            return false;
        }
        return true;
    }
    return false;
};

var testCallback = function (state) {
    console.log('test callback'+state.deref().state);
};
var onAgentStateChanged = function (state) {
    var self = this;
    var ret;

    if (state.deref().state === 1) {
        if (self.so.get_config(self.cfg.ref())===0) {
            if (self.so.agent_enable(self.cfg.ref())===1) {
                self.agent.config.updateConfig();
            }
        }
    }
};


process.on('exit', function () {
    on_agent_state_changed;
});

// 获取容器信息
Reporter.prototype.initContainerInfo = function () {
    var self = this;
    var containerInfo = self.ContainerInfo();
    var container_info = self.agent.config.config.containerInfo;
    var agent_version = self.Str({value:'', length:0});
    var agent_install_timestamp = container_info.agentInstallTimestamp;
    var agent_install_dir = self.Str({value:'', length:0});
    var listens = self.Str({value:'', length:0});
    var container_version = self.Str({value:'', length:0});
    var container_install_dir = self.Str({value:'', length:0});
    var vm_info = self.Str({value:'', length:0});
    var vm_startup_options = self.Str({value:'', length:0});
    var vm_system_options = self.Str({value:'', length:0});
    agent_version.value = container_info.agentVersion;
    agent_install_dir.value = container_info.agentInstallDir;
    listens.value = container_info.listens[0].ip;
    container_version.value = container_info.containerVersion;
    container_install_dir.value = container_info.containerInstallDir;
    vm_info.value = container_info.VMInfo;
    vm_startup_options.value = container_info.VMStartupOptions;
    vm_system_options.value = container_info.VMSystemOptions;
    containerInfo.agent_version = agent_version;
    containerInfo.agent_install_timestamp = agent_install_timestamp;
    containerInfo.agent_install_dir = agent_install_dir;
    containerInfo.listens = listens;
    containerInfo.container_version = container_version;
    containerInfo.container_install_dir = container_install_dir;
    containerInfo.vm_info = vm_info;
    containerInfo.vm_startup_options = vm_startup_options;
    containerInfo.vm_system_options = vm_system_options;
    return containerInfo;
};

Reporter.prototype.getConfigResp = function () {
    var self = this;

    self.so.get_config(self.cfg.ref());
    var agent_enable = self.so.agent_enable(self.cfg.ref());
    self.config.enable = agent_enable === 1 ? true : false;
    if (agent_enable) {
        var agent_id = self.so.get_agent_id(self.cfg.ref());
        if (self.so.get_app_config(self.cfg.ref(), self.app_cfg.ref()) === 0) {
            self.bt_naming_cfg.properties = self.Str({value:'', length:0});
            let ret = self.so.get_bt_naming_cfg(self.app_cfg.ref(), BTTYPE.BT_NODEJS_WEB, self.bt_naming_cfg.ref());
            if(0 == ret) {
                var bt_naming_properties = self.bt_naming_cfg.properties.value;
                self.so.free_bt_naming_cfg(self.bt_naming_cfg.ref());
            }
            var data_aggregate_enable = self.so.data_aggregate_enable(self.app_cfg.ref());
        }
        if (self.so.get_backend_config(self.cfg.ref(), self.backend_cfg.ref()) === 0) {
            var sql_exe_time_explain_threshold = self.so.get_sql_exe_time_explain_threshold(self.backend_cfg.ref());
            var enable_capture_nosql_parms =  self.so.enable_capture_nosql_params(self.backend_cfg.ref()) === 0? false: true;
            var nosql_max_param_lenth = self.so.get_nosql_max_param_length(self.backend_cfg.ref());
            var sql_max_records = self.so.get_sql_max_records(self.backend_cfg.ref());
            var remote_call_get_uri_seg_type = self.so.get_remote_call_get_uri_seg_type(self.backend_cfg.ref());
            var remote_call_uri_segments = self.so.get_remote_call_uri_segments(self.backend_cfg.ref());
            var enable_sql_explain = self.so.enable_sql_explain(self.backend_cfg.ref()) === 0? false: true;
        }
        self.so.get_bt_filter_config(self.cfg.ref(), self.bt_filter_cfg.ref());
        self.so.get_error_config(self.cfg.ref(), self.error_cfg.ref());


        if (self.so.get_trace_config(self.cfg.ref(), self.trace_cfg.ref()) === 0) {
            var trace_enable = self.so.trace_enable(self.trace_cfg.ref())=== 0 ? false : true;
            var default_gatherer_percent = self.so.get_default_gatherer_percent(self.trace_cfg.ref());
            var collect_http_params_enable = self.so.collect_http_params_enable(self.trace_cfg.ref()) ===0 ? false : true;
            var trace_mode = self.so.get_trace_mode(self.trace_cfg.ref());
            var collect_http_req_headers_enable = self.so.collect_http_req_headers_enable(self.trace_cfg.ref()) ===0 ? false : true;
            var smart_mode_level = self.so.get_smart_mode_level(self.trace_cfg.ref());
        }

        if (self.so.get_inject_js_config(self.cfg.ref(), self.js_cfg.ref()) === 0) {
            var js = self.Str({value:'', length:0});
            var inject_js_enable = self.so.get_inject_js(self.js_cfg.ref(), js.ref());
            if (inject_js_enable === 0) {
                var inject_js = js.value;
                self.so.free_inject_js(js.ref());
            }
        }
        self.config = {
            enable: agent_enable,
            agentID: agent_id,
            appConfig: {
                dataAggregateEnable:data_aggregate_enable, //是否开启数据融合
                namingConfigs: { //命名规则
                    type : self.bt_naming_cfg.type,
                    enable: self.bt_naming_cfg.enable ===0? false: true,
                    properties: bt_naming_properties ? bt_naming_properties : '',
                }
            },
            backendConfig: {
                enableCaptureNosqlParams: enable_capture_nosql_parms,//nosql是否采集指令参数
                nosqlMaxParamLength: nosql_max_param_lenth,//nosql指令参数最大长度
                sqlMaxRecords: sql_max_records,//sql语句最大记录条数
                sqlExeTimeExplainThreshold: sql_exe_time_explain_threshold,//sql语句获取执行计划阀值，快照需要获取SQL执行计划
                remoteCallGetURISegmentType: remote_call_get_uri_seg_type,//截取外部调用URI类型，类型为FULL时，URI段数不生效
                remoteCallURISegments: remote_call_uri_segments,//截取外部调用URI段数
                enableSqlExplain: enable_sql_explain //启用sql执行计划采集
            },
            btConfig: {//业务黑/白名单设置
                btFilterCfg:self.bt_filter_cfg //通过调用in_bt_blacklist，in_bt_whitelist，in_snapshot_blacklist判断黑白名单
            },
            errorConfig: {
                hErrCfg: self.error_cfg
            },
            injectJsConfig: {
                injectJsEnable: inject_js_enable==0? true:false,
                injectJs: inject_js //嵌入js内容，<script>内容</script>
            },
            traceConfig: {
                enable: trace_enable, //是否启用trace功能
                defaultGathererPercent: default_gatherer_percent, //默认采样率， (范围1~1000)，单位：千分之一
                enableCollectHttpParams: collect_http_params_enable, //是否采集http参数
                traceMode: trace_mode, //调用链采集模式
                enableCollectHttpReqHeaders: collect_http_req_headers_enable, //是否采集http请求头，默认采集全部请求头字段
                smartModeLevel: smart_mode_level  //智能模式级别: 每个入口业务每5s的前n个请求抓取调用链，默认值为2，支持web配置强度低、中、高对应： 1、2、3
            },
        };
        return self.config;
    } else {
        return false;
    }

};