"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function createHooks(state) {
    let hooks = [];
    return {
        init(asyncId, provider, parentId, parentHandle) {
            state.childToParent.set(asyncId, parentId);
            for (const hook of hooks) {
                hook.init(asyncId, provider, parentId, parentHandle);
            }
        },
        pre(asyncId) {
            state.previousIds.push(state.currentId);
            state.currentId = asyncId;
            state.parentId = state.childToParent.get(asyncId) || 0;
            for (const hook of hooks) {
                hook.before(state.currentId);
            }
        },
        post(asyncId, didThrow) {
            state.currentId = state.previousIds.pop() || 0;
            state.parentId = state.childToParent.get(state.currentId) || 0;
            for (const hook of hooks) {
                hook.after(asyncId);
            }
        },
        destroy(asyncId) {
            state.childToParent.delete(asyncId);
            for (const hook of hooks) {
                hook.destroy(asyncId);
            }
        },
        add(hook) {
            hooks.push(hook);
        },
        remove(hook) {
            hooks = hooks.filter((next) => next !== hook);
        },
    };
}
exports.createHooks = createHooks;
//# sourceMappingURL=Hooks.js.map