"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const logger = require("../logger");
const patches_1 = require("./patches");
function startAsyncHooks(hooks, state) {
    const async_hooks = require('async_hooks');
    for (const key of Object.keys(patches_1.patches)) {
        patches_1.patches[key](hooks, state);
    }
    const ignoreUIDs = new Set();
    const idMap = new Map();
    async_hooks.createHook({
        init(uid, type, triggerId, resource) {
            try {
                if (type === 'PROMISE') {
                    ignoreUIDs.add(uid);
                }
                else {
                    const asyncId = state.getNextId();
                    const parentId = state.currentId;
                    idMap.set(uid, asyncId);
                    hooks.init(asyncId, type, parentId, resource);
                }
            }
            catch (err) {
                logger.error('[async_hooks]: init: ', err);
            }
        },
        before(uid) {
            try {
                if (!ignoreUIDs.has(uid)) {
                    const asyncId = idMap.get(uid);
                    if (asyncId !== undefined) {
                        hooks.pre(asyncId);
                    }
                }
            }
            catch (err) {
                logger.error('[async_hooks]: before: ', err);
            }
        },
        after(uid) {
            try {
                if (!ignoreUIDs.has(uid)) {
                    const asyncId = idMap.get(uid);
                    if (asyncId !== undefined) {
                        hooks.post(asyncId, false);
                    }
                }
            }
            catch (err) {
                logger.error('[async_hooks]: after: ', err);
            }
        },
        destroy(uid) {
            try {
                if (ignoreUIDs.has(uid)) {
                    ignoreUIDs.delete(uid);
                }
                else if (idMap.has(uid)) {
                    const asyncId = idMap.get(uid);
                    if (asyncId !== undefined) {
                        idMap.delete(uid);
                        hooks.destroy(asyncId);
                    }
                }
            }
            catch (err) {
                logger.error('[async_hooks]: destroy: ', err);
            }
        },
    }).enable();
}
exports.startAsyncHooks = startAsyncHooks;
//# sourceMappingURL=index.js.map