"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const logger = require("../logger");
const patches_1 = require("./patches");
function startAsyncWrap(hooks, state) {
    const asyncWrap = process.binding('async_wrap');
    for (const key of Object.keys(patches_1.patches)) {
        patches_1.patches[key](hooks, state);
    }
    const TIMERWRAP = asyncWrap.Providers.TIMERWRAP;
    const ignoreUIDs = new Set();
    const idMap = new Map();
    function providerToString(provider) {
        switch (provider) {
            case asyncWrap.Providers.NONE:
                return 'NONE';
            case asyncWrap.Providers.CRYPTO:
                return 'CRYPTO';
            case asyncWrap.Providers.FSEVENTWRAP:
                return 'FSEVENTWRAP';
            case asyncWrap.Providers.FSREQWRAP:
                return 'FSREQWRAP';
            case asyncWrap.Providers.GETADDRINFOREQWRAP:
                return 'GETADDRINFOREQWRAP';
            case asyncWrap.Providers.GETNAMEINFOREQWRAP:
                return 'GETNAMEINFOREQWRAP';
            case asyncWrap.Providers.HTTPPARSER:
                return 'HTTPPARSER';
            case asyncWrap.Providers.JSSTREAM:
                return 'JSSTREAM';
            case asyncWrap.Providers.PIPEWRAP:
                return 'PIPEWRAP';
            case asyncWrap.Providers.PIPECONNECTWRAP:
                return 'PIPECONNECTWRAP';
            case asyncWrap.Providers.PROCESSWRAP:
                return 'PROCESSWRAP';
            case asyncWrap.Providers.QUERYWRAP:
                return 'QUERYWRAP';
            case asyncWrap.Providers.SHUTDOWNWRAP:
                return 'SHUTDOWNWRAP';
            case asyncWrap.Providers.SIGNALWRAP:
                return 'SIGNALWRAP';
            case asyncWrap.Providers.STATWATCHER:
                return 'STATWATCHER';
            case asyncWrap.Providers.TCPWRAP:
                return 'TCPWRAP';
            case asyncWrap.Providers.TCPCONNECTWRAP:
                return 'TCPCONNECTWRAP';
            case asyncWrap.Providers.TIMERWRAP:
                return 'TIMERWRAP';
            case asyncWrap.Providers.TLSWRAP:
                return 'TLSWRAP';
            case asyncWrap.Providers.TTYWRAP:
                return 'TTYWRAP';
            case asyncWrap.Providers.UDPWRAP:
                return 'UDPWRAP';
            case asyncWrap.Providers.UDPSENDWRAP:
                return 'UDPSENDWRAP';
            case asyncWrap.Providers.WRITEWRAP:
                return 'WRITEWRAP';
            case asyncWrap.Providers.ZLIB:
                return 'ZLIB';
            default:
                return 'NONE';
        }
    }
    asyncWrap.setupHooks({
        init(uid, provider, parentUid, parentHandle) {
            try {
                if (provider === TIMERWRAP) {
                    ignoreUIDs.add(uid);
                }
                else {
                    const asyncId = state.getNextId();
                    const parentId = state.currentId;
                    const type = providerToString(provider);
                    idMap.set(uid, asyncId);
                    hooks.init(asyncId, type, parentId, parentHandle);
                }
            }
            catch (err) {
                logger.error('[async_wrap]: init: ', err);
            }
        },
        pre(uid) {
            try {
                if (!ignoreUIDs.has(uid)) {
                    const asyncId = idMap.get(uid);
                    if (asyncId !== undefined) {
                        hooks.pre(asyncId);
                    }
                }
            }
            catch (err) {
                logger.error('[async_wrap]: pre: ', err);
            }
        },
        post(uid, didThrow) {
            try {
                if (!ignoreUIDs.has(uid)) {
                    const asyncId = idMap.get(uid);
                    if (asyncId !== undefined) {
                        hooks.post(asyncId, didThrow);
                    }
                }
            }
            catch (err) {
                logger.error('[async_wrap]: post: ', err);
            }
        },
        destroy(uid) {
            try {
                if (ignoreUIDs.has(uid)) {
                    ignoreUIDs.delete(uid);
                }
                else if (idMap.has(uid)) {
                    const asyncId = idMap.get(uid);
                    if (asyncId !== undefined) {
                        idMap.delete(uid);
                        hooks.destroy(asyncId);
                    }
                }
            }
            catch (err) {
                logger.error('[async_wrap]: destroy: ', err);
            }
        },
    });
    asyncWrap.enable();
}
exports.startAsyncWrap = startAsyncWrap;
//# sourceMappingURL=index.js.map