"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class NextTickWrap {
}
function patchNextTick(hooks, state) {
    const oldNextTick = process.nextTick;
    function patchedTick() {
        if (!state.enabled) {
            return Function.prototype.apply.call(oldNextTick, process, arguments);
        }
        else {
            const args = Array.from(arguments);
            const callback = args[0];
            if (typeof callback !== 'function') {
                throw new TypeError('callback is not a function');
            }
            const handle = new NextTickWrap();
            const asyncId = state.getNextId();
            hooks.init(asyncId, 'NextTick', state.currentId, handle);
            args[0] = function () {
                hooks.pre(asyncId);
                let didThrow = true;
                try {
                    callback.apply(this, arguments);
                    didThrow = false;
                }
                finally {
                    if (didThrow && process.listenerCount('uncaughtException') > 0) {
                        process.once('uncaughtException', () => {
                            hooks.post(asyncId, true);
                            hooks.destroy(asyncId);
                        });
                    }
                }
                hooks.post(asyncId, false);
                hooks.destroy(asyncId);
            };
            return Function.prototype.apply.call(oldNextTick, process, args);
        }
    }
    oldNextTick.call = function (thisArg, ...args) {
        return Function.prototype.call(patchedTick, thisArg, ...args);
    };
    oldNextTick.apply = function (thisArg, args) {
        return Function.prototype.apply(patchedTick, [thisArg, ...args]);
    };
    process.nextTick = patchedTick;
}
exports.patchNextTick = patchNextTick;
//# sourceMappingURL=next-tick.js.map