"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class PromiseWrap {
}
function patchPromise(hooks, state) {
    const Promise = global.Promise;
    const oldThen = Promise.prototype.then;
    Promise.prototype.then = wrappedThen;
    function makeWrappedHandler(fn, asyncId, isOnFulfilled) {
        if ('function' !== typeof fn) {
            return isOnFulfilled
                ? makeUnhandledResolutionHandler(asyncId)
                : makeUnhandledRejectionHandler(asyncId);
        }
        return function wrappedHandler() {
            hooks.pre(asyncId);
            try {
                return fn.apply(this, arguments);
            }
            finally {
                hooks.post(asyncId, false);
                hooks.destroy(asyncId);
            }
        };
    }
    function makeUnhandledResolutionHandler(asyncId) {
        return function unhandledResolutionHandler(val) {
            hooks.destroy(asyncId);
            return val;
        };
    }
    function makeUnhandledRejectionHandler(asyncId) {
        return function unhandledRejectedHandler(val) {
            hooks.destroy(asyncId);
            throw val;
        };
    }
    function wrappedThen(onFulfilled, onRejected) {
        if (!state.enabled) {
            return oldThen.call(this, onFulfilled, onRejected);
        }
        const handle = new PromiseWrap();
        const asyncId = state.getNextId();
        hooks.init(asyncId, 0, state.currentId, handle);
        return oldThen.call(this, makeWrappedHandler(onFulfilled, asyncId, true), makeWrappedHandler(onRejected, asyncId, false));
    }
}
exports.patchPromise = patchPromise;
//# sourceMappingURL=promise.js.map