"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const timers = require("timers");
function TimeoutWrap() { }
function IntervalWrap() { }
function ImmediateWrap() { }
const timeoutMap = new Map();
const intervalMap = new Map();
const ImmediateMap = new Map();
let activeCallback = null;
let clearedInCallback = false;
function patchTimers(hooks, state) {
    patchTimer(hooks, state, 'setTimeout', 'clearTimeout', TimeoutWrap, timeoutMap, true);
    patchTimer(hooks, state, 'setInterval', 'clearInterval', IntervalWrap, intervalMap, false);
    patchTimer(hooks, state, 'setImmediate', 'clearImmediate', ImmediateWrap, ImmediateMap, true);
    global.setTimeout = timers.setTimeout;
    global.setInterval = timers.setInterval;
    global.setImmediate = timers.setImmediate;
    global.clearTimeout = timers.clearTimeout;
    global.clearInterval = timers.clearInterval;
    global.clearImmediate = timers.clearImmediate;
}
exports.patchTimers = patchTimers;
function patchTimer(hooks, state, setFn, clearFn, Handle, timerMap, singleCall) {
    const oldSetFn = timers[setFn];
    const oldClearFn = timers[clearFn];
    function patchedSetFunction() {
        if (!state.enabled) {
            return Function.prototype.call.apply(oldSetFn, [timers, ...arguments]);
        }
        else {
            const args = Array.from(arguments);
            const callback = args[0];
            if (typeof callback !== 'function') {
                throw new TypeError('"callback" argument must be a function');
            }
            const handle = new Handle();
            const asyncId = state.getNextId();
            let timerId;
            hooks.init(asyncId, 0, state.currentId, handle);
            args[0] = function () {
                activeCallback = timerId;
                hooks.pre(asyncId);
                let didThrow = true;
                try {
                    callback.apply(this, arguments);
                    didThrow = false;
                }
                finally {
                    if (didThrow && process.listenerCount('uncaughtException') > 0) {
                        process.once('uncaughtException', () => {
                            hooks.post(asyncId, true);
                            timerMap.delete(timerId);
                            hooks.destroy(asyncId);
                        });
                    }
                }
                hooks.post(asyncId, false);
                activeCallback = null;
                if (singleCall || clearedInCallback) {
                    clearedInCallback = false;
                    timerMap.delete(timerId);
                    hooks.destroy(asyncId);
                }
            };
            timerId = oldSetFn(...args);
            timerMap.set(timerId, asyncId);
            return timerId;
        }
    }
    oldSetFn.call = function (thisArg, ...args) {
        return Function.prototype.call.apply(patchedSetFunction, [thisArg, ...args]);
    };
    oldSetFn.apply = function (thisArg, args) {
        return Function.prototype.call.apply(patchedSetFunction, [thisArg, args]);
    };
    timers[setFn] = patchedSetFunction;
    function patchedClearFunction(timerId) {
        if (activeCallback === timerId && timerId !== null) {
            clearedInCallback = true;
        }
        else if (timerMap.has(timerId)) {
            const asyncId = timerMap.get(timerId);
            timerMap.delete(timerId);
            hooks.destroy(asyncId);
        }
        oldClearFn(timerId);
    }
    oldClearFn.call = function (thisArg, ...args) {
        return Function.prototype.call.apply(patchedClearFunction, [thisArg, ...args]);
    };
    oldClearFn.apply = function (thisArg, args) {
        return Function.prototype.call.apply(patchedClearFunction, [thisArg, args]);
    };
    timers[clearFn] = patchedClearFunction;
}
//# sourceMappingURL=timers.js.map