import sys

from Bonree.agent.interceptor.base import EntryPointInterceptor
from Bonree.agent.models.transactions import ENTRY_PYTHON_BACKGROUND
from Bonree.logger import SYS_LOGGER, BT_LOGGER


class CeleryInterceptor(EntryPointInterceptor):
    def _apply_async(self, apply_async, instance, *args, **kwargs):
        with self.log_exceptions():
            BT_LOGGER.debug("celery _apply_async start")
            btr = self.start_transaction_request(ENTRY_PYTHON_BACKGROUND, name=instance.name)
            if btr:
                BT_LOGGER.debug("celery _apply_async start with btr")
        if btr:
            try:
                response = apply_async(instance, *args, **kwargs)
            except Exception:
                with self.log_exceptions():
                    if btr:
                        btr.add_exception(*sys.exc_info())
                raise
            finally:
                with self.log_exceptions():
                    if btr:
                        btr.status_code = 0
                        self.end_transaction_request(btr)

            return response

        return apply_async(instance, *args, **kwargs)

    def _get(self, get, asyncresult, *args, **kwargs):
        with self.log_exceptions():
            btr = None
            BT_LOGGER.debug("celery _get start")
            task_name = 'tasks.defaultTask'
            if hasattr(asyncresult, 'task_name'):
                task_name = asyncresult.task_name
            else:
                celery_app = getattr(asyncresult, 'app')
                if celery_app:
                    task_dict = getattr(celery_app, '_tasks')
                    for task in task_dict:
                        task_name = task
                        break
            btr = self.start_transaction_request(ENTRY_PYTHON_BACKGROUND, name= task_name)
            if btr:
                BT_LOGGER.debug("celery _get start with btr")
        if btr:
            try:
                result = get(asyncresult, *args, **kwargs)
            except Exception:
                with self.log_exceptions():
                    if btr:
                        btr.add_exception(*sys.exc_info())
                raise
            finally:
                with self.log_exceptions():
                    if btr:
                        btr.status_code = 0
                        self.end_transaction_request(btr)

            return result

        return get(asyncresult, *args, **kwargs)


def intercept_celery(agent, mod):
    SYS_LOGGER.info("intercept_celery")
    CeleryInterceptor(agent, mod.Task).attach('apply_async',btrace_data=True)


def intercept_result(agent, mod):
    SYS_LOGGER.info("intercept_result")
    CeleryInterceptor(agent, mod.AsyncResult).attach('get',btrace_data=True)
