# Copyright (c) Bonree, Inc., and its affiliates
# 2018
# All Rights Reserved

"""Interceptor for invoke.

"""

import sys

from Bonree.agent.interceptor.base import EntryPointInterceptor
from Bonree.agent.models.transactions import ENTRY_PYTHON_BACKGROUND


class InvokeInterceptor(EntryPointInterceptor):
    def _run(self, run, runner, command, **kwargs):
        btr = self.start_transaction_request(ENTRY_PYTHON_BACKGROUND, name=command)
        if btr:
            try:
                response = run(runner, command, **kwargs)
            except Exception:
                with self.log_exceptions():
                    if btr:
                        btr.add_exception(*sys.exc_info())
                raise
            finally:
                with self.log_exceptions():
                    if btr:
                        btr.status_code = 0
                        self.end_transaction_request(btr)

            return response

        return run(runner, command, **kwargs)


    def _execute(self, execute, Executor, *tasks):
        name = ' '.join(task.name for task in tasks)
        btr = self.start_transaction_request(ENTRY_PYTHON_BACKGROUND, name=name)
        if btr:
            try:
                response = execute(Executor, *tasks)
            except Exception:
                with self.log_exceptions():
                    if btr:
                        btr.add_exception(*sys.exc_info())
                raise
            finally:
                with self.log_exceptions():
                    if btr:
                        btr.status_code = 0
                        self.end_transaction_request(btr)

            return response

        return execute(Executor, *tasks)


def intercept_invoke(agent, mod):
    InvokeInterceptor(agent, mod.Runner).attach('run',btrace_data=True)

def intercept_invoke_execute(agent, mod):
    InvokeInterceptor(agent, mod.Executor).attach('execute',btrace_data=True)