import sys

from Bonree.agent.interceptor.base import BaseInterceptor, OnlyEndInterceptor
from Bonree.agent.interceptor.base import EntryPointInterceptor
from Bonree.agent.models.transactions import ENTRY_PYTHON_BACKGROUND
from Bonree.logger import SYS_LOGGER, BT_LOGGER
from Bonree.config import setEnvironIp, setEnvironPort
from types import MethodType,FunctionType


class ThriftServerInitInterceptor(OnlyEndInterceptor):
    def end_data(self, md, origin_self, *args, **kwargs):
        hasHandler = hasattr(origin_self, '_handler')
        hasProcessMap = hasattr(origin_self, '_processMap')
        SYS_LOGGER.debug('origin_self[%s] hasHandler[%s] hasProcessMap[%s]' % (origin_self, hasHandler, hasProcessMap,))
        if not (hasHandler and hasProcessMap):
            return
        _handler = getattr(origin_self, '_handler')
        handlerClass = _handler.__class__
        _processMap = getattr(origin_self, '_processMap')
        SYS_LOGGER.debug('origin_self[%s] type(_processMap)[%s]' % (origin_self, type(_processMap),))
        if not isinstance(_processMap, dict):
            return
        for processFunctionName in _processMap:
            processFunction = getattr(handlerClass, processFunctionName, None)
            isFunctionType = isinstance(processFunction, FunctionType)
            SYS_LOGGER.debug('handlerClass[%s] processFunction[%s]' % (handlerClass, processFunction,))
            if not isFunctionType:
                continue
            SYS_LOGGER.debug('hook %s.%s' % (handlerClass.__name__, processFunctionName,))
            BaseInterceptor(self.agent, handlerClass).attach(processFunctionName, btrace_data=True)


class ThriftServerInterceptor(EntryPointInterceptor):
    def attached_process_method(self, origin_method, origin_self, *args, **kwargs):
        btr = None
        with self.log_exceptions():
            hook_method = '%s.%s' % (origin_self.__class__.__name__, origin_method.__name__,)
            BT_LOGGER.debug("hook_method[%s] start" % (hook_method))
            process_handler = getattr(origin_self, '_handler')
            # remove 'process_' prefix
            # 'process_ping' --> 'ping'
            handler_method = origin_method.__name__[8:]
            bt_name = '%s.%s' % (process_handler.__class__.__name__, handler_method)
            BT_LOGGER.debug("hook_method[%s] bt_name[%s]" % (hook_method, bt_name))
            btr = self.start_transaction_request(ENTRY_PYTHON_BACKGROUND, name=bt_name)
            if btr:
                BT_LOGGER.debug("hook_method[%s] bt_name[%s] start with btr" % (hook_method, bt_name))
        if btr:
            try:
                response = origin_method(origin_self, *args, **kwargs)
            except Exception:
                with self.log_exceptions():
                    if btr:
                        btr.add_exception(*sys.exc_info())
                raise
            finally:
                with self.log_exceptions():
                    if btr:
                        btr.status_code = 0
                        self.end_transaction_request(btr)

            return response

        return origin_method(origin_self, *args, **kwargs)


class ThriftServerListenInterceptor(OnlyEndInterceptor):
    def end_data(self, md, origin_self, *args, **kwargs):
        host = str(origin_self.host)
        port = str(origin_self.port)
        setEnvironIp(host)
        setEnvironPort([port, ])


def intercept_thrift_server(agent, mod):
    iface = getattr(mod, 'Iface', None)
    tProcessor = getattr(mod, 'TProcessor', None)
    processor = getattr(mod, 'Processor', None)
    # They are required be `class`
    if not (isinstance(iface, type) and isinstance(tProcessor, type) and isinstance(processor, type)):
        return

    porcessorIsIface = issubclass(processor, iface)
    porcessorIsTProcessor = issubclass(processor, tProcessor)
    SYS_LOGGER.debug('PorcessorIsIface[%s] PorcessorIsTProcessor[%s]' % (porcessorIsIface, porcessorIsTProcessor,))
    if not (porcessorIsIface and porcessorIsTProcessor):
        return

    module_name = mod.__name__
    processor_name = processor.__name__
    # hook __init__
    SYS_LOGGER.debug('hook [%s.%s.%s]' % (module_name, processor_name, '__init__'))
    ThriftServerInitInterceptor(agent, processor).attach('__init__')
    # hook process methods
    for processFunctionName in dir(processor):
        if not processFunctionName.startswith('process_'):
            continue
        processFunction = getattr(processor, processFunctionName, None)
        isFunctionType = isinstance(processFunction, FunctionType)
        isMethodType = isinstance(processFunction, MethodType)
        SYS_LOGGER.debug('processFunction[%s] isFunctionType[%s] isMethodType[%s]' % (processFunction, isFunctionType, isMethodType))
        if not (isFunctionType or isMethodType):
            continue
        SYS_LOGGER.debug('hook [%s.%s.%s]' % (module_name, processor_name, processFunctionName))
        ThriftServerInterceptor(agent, processor).attach(processFunctionName, patched_method_name= 'attached_process_method', btrace_data=True)


def intercept_thrift_server_listen(agent, mod):
    ThriftServerListenInterceptor(agent, mod.TServerSocket).attach('listen', btrace_data=False)
