# Copyright (c) Bonree, Inc., and its affiliates
# 2016
# All Rights Reserved

"""Interceptor for memcached.

"""

from __future__ import unicode_literals

from Bonree import rule, config
from Bonree.lib import parse_args
from Bonree.agent.interceptor.cache import CacheInterceptor
from Bonree.agent.proto.pb_type import BACKEND_TYPE_MEMCACHE

class MemcacheClientInterceptor(CacheInterceptor):
    def __init__(self, agent, cls):
        super(MemcacheClientInterceptor, self).__init__(agent, cls, rule.vendor_conversion('memcached'))

    def start_data(self, md, client, *args, **kwargs):
        md.backend = self.get_backend(['%s:%s' % server.address for server in client.servers])
        if md.backend:
            operation = md.original_method.__name__
            if config.getEnableCaptureNosqlParams():
                operation = '%s(%s)' % (operation, parse_args(args, kwargs))
            md.operation = operation

    def make_tags(self, md, client, *args, **kwargs):
        tags  = {}
        operation = md.original_method
        if config.getEnableCaptureNosqlParams():
            operation = '%s(%s)' % (operation, parse_args(args, kwargs))
        tags['db.statement'] = operation
        server_pool = ['%s:%s' % server.address for server in client.servers]
        url = rule.url_conversion('memcache', server_pool[0])
        host = server_pool[0].split(':')[0]
        port = server_pool[0].split(':')[1]
        tags['peer.address'] = rule.peer_address_conversion(url, host, port)
        tags['db.type'] = rule.connType_conversion('memcache')
        md.tags = tags

    def get_backend(self, server_pool):
        """

        Parameters
        ----------
        server_pool : list of str

        """
        backend_name_format_string = '{VENDOR_NAME}'
        backend_properties = {
            'VENDOR_NAME': self.vendor.title(),
            'VENDOR': self.vendor,
            'SERVER POOL': '\n'.join(server_pool),
            'URL': rule.url_conversion('memcache', server_pool[0]),
            'HOST': server_pool[0].split(':')[0],
            'PORT': server_pool[0].split(':')[1],
        }
        backend = self.agent.backend_registry.get_backend(BACKEND_TYPE_MEMCACHE, backend_properties,
                                                       backend_name_format_string)
        backend.connType =  rule.connType_conversion('memcache')
        return backend


def intercept_memcache(agent, mod):
    methods = [
        'set',
        'set_multi',
        'add',
        'replace',
        'append',
        'prepend',
        'cas',
        'get',
        'get_multi',
        'gets',
        'delete',
        'delete_multi',
        'incr',
        'decr',
        'flush_all',
    ]
    for method in methods:
        MemcacheClientInterceptor(agent, mod.Client).attach(method, btrace_data=True)
