# Copyright (c) Bonree, Inc., and its affiliates
# 2018
# All Rights Reserved

"""Interceptor for ssdb.

"""

from __future__ import unicode_literals

from Bonree import rule, config
from Bonree.lang import str
from Bonree.agent.interceptor.cache import CacheInterceptor
from Bonree.agent.proto.pb_type import BACKEND_TYPE_SSDB

class SsdbConnectionInterceptor(CacheInterceptor):
    def __init__(self, agent, cls):
        super(SsdbConnectionInterceptor, self).__init__(agent, cls, rule.vendor_conversion('SSDB'))

    def start_data(self, md, connection, cmd, *args, **kwargs):
        md.backend = self.get_backend(connection.host, connection.port)
        if md.backend:
            operation = cmd
            if config.getEnableCaptureNosqlParams():
                operation = '%s(%s)' % (cmd, ','.join(str(x) for x in args))
            md.operation = operation

    def make_tags(self, md, connection, cmd, *args, **kwargs):
        tags  = {}
        tags['db.statement'] = '%s(%s)' % (cmd, ','.join(str(x) for x in args))
        host = connection.host
        port = connection.port
        url = rule.url_conversion('ssdb', host, port)
        tags['peer.address'] = rule.peer_address_conversion(url, host, str(port))
        tags['db.type'] = rule.connType_conversion('ssdb')
        md.tags = tags

    def get_backend(self, host, port):
        """

        Parameters
        ----------
        host : redis server host
        port : redis server port

        """
        backend_name_format_string = rule.common_name()
        backend_properties = {
            'HOST': host,
            'PORT': str(port),
            'VENDOR': self.vendor,
            'URL': rule.url_conversion('ssdb', host, port)
        }
        backend = self.agent.backend_registry.get_backend(BACKEND_TYPE_SSDB, backend_properties,
                                                          backend_name_format_string)
        backend.connType = rule.connType_conversion('ssdb')
        return backend


def intercept_ssdb(agent, mod):
    SsdbConnectionInterceptor(agent, mod.Connection).attach('send', btrace_data=True)
