# Copyright (c) Bonree, Inc., and its affiliates
# 2018
# All Rights Reserved

"""Interceptor for hot method. the hot methods is config by users on web

"""
import sys

from Bonree.agent.interceptor.base import BaseInterceptor
from Bonree.logger import SYS_LOGGER
from Bonree.agent.services.wrapper import *

class HotspotInterceptor(BaseInterceptor):
    hooked_methods = {}

    def attach(self, mod_name, method):
        original_method = getattr(self.cls, method)
        if hasattr(original_method, '_bonr_intercepted'):
            HotspotInterceptor.hooked_methods[mod_name + ':' + self.cls.__name__ + '.' + method].btrace_data = True
        else:
            HotspotInterceptor.hooked_methods[mod_name + ':' + self.cls.__name__ + '.' + method] = self
            super(HotspotInterceptor, self).attach(method,  btrace_data=True)


def unhook_method(config_methods):
        for method in list(HotspotInterceptor.hooked_methods):
            if method not in config_methods:
                HotspotInterceptor.hooked_methods[method].btrace_data = False

def get_hotshot_method(mod_name, cls_name, method):
    def hook_method(agent, mod):
        if cls_name and hasattr(mod, cls_name):
            mod = getattr(mod, cls_name)
            if hasattr(mod, method):
                HotspotInterceptor(agent, mod).attach(mod_name, method)
            else:
                SYS_LOGGER.warning('Exception in module: {0} class: {1} hook, No such method: {2}'.format(mod.__name__, cls_name, method))
        else:
            SYS_LOGGER.warning('Exception in module: {0} hook, No such class: {1}'.format(mod.__name__, cls_name))

    return hook_method

def hotshot_hook(methods, SO, CALL_GRAPH_CONFIG):
    from Bonree.agent import get_agent_instance
    config_methods = set()
    agent = get_agent_instance()
    for method in methods:
        str_list = method['className'].split(':')
        mod_name = str_list[0]
        cls_Name = ''
        if len(str_list) == 2:
            cls_Name = str_list[1]

        if cls_Name:
            if bool(SO.in_exclude_names(pointer(CALL_GRAPH_CONFIG), pointer(Str().with_value(cls_Name)))):
                continue

        for method_name in method['methodNames']:
            mod_method = mod_name + ':' + cls_Name + '.' + method_name
            config_methods.add(mod_method)

            if mod_method in list(HotspotInterceptor.hooked_methods):
                HotspotInterceptor.hooked_methods[mod_method].btrace_data = True
                continue
            if agent.module_interceptor:
                agent.module_interceptor.call_on_import(mod_name,get_hotshot_method(mod_name, cls_Name, method_name))

    unhook_method(config_methods)
