# Copyright (c) Bonree, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Entry-point interceptors for various web1/application frameworks.

"""
from Bonree import config
from Bonree.agent.proto.pb_config import HTTP_CODE_TYPE_SUCESS
from Bonree.lang import string_t
from Bonree.agent.interceptor.base import html_re, BaseInterceptor
from Bonree.logger import only_log_exception

def inject_js_code(rc, btr):
    with only_log_exception:
        if config.getgetJsConfigJs() and isinstance(rc, string_t) and html_re.match(rc):
            if not config.getJsSite():
                pos = rc.find("<head>")
                if pos >= 0:
                    pos += len('<head>')
                    rc = rc[:pos] + ("<script>%s</script>" % config.getJsCode()) + rc[pos:]
                    if hasattr(btr, '__inject_js__'):
                        btr.__inject_js__ = True
            else:
                pos = rc.find("</head>")
                if pos >= 0:
                    rc = rc[:pos] + ("<script>%s</script>" % config.getJsCode()) + rc[pos:]
                    if hasattr(btr, '__inject_js__'):
                        btr.__inject_js__ = True
    return rc


class ExitPointInterceptor(BaseInterceptor):
    def inject_js_code(self, rc):
        with self.log_exceptions():
            if config.getgetJsConfigJs() and isinstance(rc, string_t) and html_re.match(rc):
                if not config.getJsSite():
                    pos = rc.find("<head>")
                    if pos >= 0:
                        pos += len('<head>')
                        rc = rc[:pos] + ("<script>%s</script>" % config.getJsCode()) + rc[pos:]
                        if hasattr(self.btr, '__inject_js__'):
                            self.btr.__inject_js__ = True
                else:
                    pos = rc.find("</head>")
                    if pos >= 0:
                        rc = rc[:pos] + ("<script>%s</script>" % config.getJsCode()) + rc[pos:]
                        if hasattr(self.btr, '__inject_js__'):
                            self.btr.__inject_js__ = True
        return rc

    def handle_http_status_code(self, btr, status_code, msg):
        status_code = int(status_code)
        btr.status_code = status_code

        error_msg, error_type = config.getErrorMsg(status_code)
        if error_type and int(error_type) == HTTP_CODE_TYPE_SUCESS:
            return

        if error_msg and error_msg != 'None':
            btr.add_http_error(status_code, error_msg)
            return
        if status_code < 400:
            return
        else:
            btr.add_http_error(status_code, msg)


from .bottle_interceptor import intercept_bottle
from .tornado_interceptor import intercept_tornado_web
from .flask_interceptor import intercept_flask, intercept_flask_response, intercept_flask_request
from .werkzeug_interceptor import intercept_werkzeug_serving
from .pyramid_interceptor import intercept_pyramid, intercept_pyramid_response
from .cherrypy_interceptor import intercept_cherrypy, intercept_cherrypy_wspbus, intercept_cherrypy_request
from .django_interceptor import intercept_django_base_handler, intercept_django_wsgi_handler, intercept_django_response, \
    intercept_django_render, intercept_uncaught_exception, \
    intercept_django_runserver, intercept_django_asgi_handler
from .odoo_interceptor import intercept_odoo_wsgi_handler, intercept_odoo6_wsgi_handler, intercept_odoo_request_handler
from .web_interceptor import interceptor_web, interceptor_web_static, interceptor_start_response
from .sanic_interceptor import intercept_sanic, intercept_sanic_response, intercept_sanic_response_send
from .apistar_interceptor import intercept_APIStar, intercept_APIStar_ASGItoWSGIAdapter, intercept_apistar_request
from .wsgiref_interceptor import intercept_wsgiref
from .gunicorn_interceptor import intercept_gunicorn
from .quart_interceptor import intercept_quart, intercept_quart_response, intercept_quart_request
from .uvicorn_interceptor import intercept_uvicorn
from .starlette_interceptor import intercept_starlette, intercept_starlette_response
from .daphne_interceptor import intercept_daphne
from .webob import intercept_webob_response, intercept_webob_client, intercept_webob_dec_wsgify
from .nova import (
    intercept_nova_compute_api,
    intercept_nova_conductor_api,
    intercept_nova_scheduler_api,

    intercept_nova_conductor_manage,
    intercept_nova_compute_manage,
    intercept_nova_schedule_manage,
)


__all__ = [
    'intercept_bottle',
    'intercept_flask',
    'intercept_flask_response',
    'intercept_flask_request',
    'intercept_werkzeug_serving',
    'intercept_django_base_handler',
    'intercept_django_wsgi_handler',
    'intercept_django_asgi_handler',
    'intercept_django_response',
    'intercept_django_render',
    'intercept_django_runserver',
    'intercept_uncaught_exception',
    'intercept_tornado_web',
    'intercept_pyramid',
    'intercept_pyramid_response',
    'intercept_cherrypy',
    'intercept_cherrypy_wspbus',
    'intercept_cherrypy_request',
    'intercept_odoo_wsgi_handler',
    'intercept_odoo6_wsgi_handler',
    'intercept_odoo_request_handler',
    'interceptor_web',
    'interceptor_web_static',
    'intercept_gunicorn',
    'intercept_sanic',
    'intercept_sanic_response',
    'intercept_sanic_response_send',
    'intercept_APIStar',
    'intercept_apistar_request',
    'intercept_APIStar_ASGItoWSGIAdapter',
    'interceptor_start_response',
    'intercept_wsgiref',
    'intercept_quart',
    'intercept_quart_response',
    'intercept_quart_request',
    'intercept_uvicorn',
    'intercept_starlette',
    'intercept_starlette_response',
    'intercept_daphne',
    'intercept_webob_response',
    'intercept_webob_client',
    'intercept_webob_dec_wsgify',

    'intercept_nova_compute_api',
    'intercept_nova_conductor_api',
    'intercept_nova_scheduler_api',

    'intercept_nova_conductor_manage',
    'intercept_nova_compute_manage',
    'intercept_nova_schedule_manage',
]
