# Copyright (c) Bonree, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Interceptor for Bottle.

"""
from __future__ import unicode_literals
import sys

from Bonree.agent.interceptor.frameworks import ExitPointInterceptor
from Bonree.agent.interceptor.frameworks.wsgi import WSGIInterceptor
from Bonree.agent.interceptor.base import OnlyStartInterceptor
from Bonree.config import is_content_type
from Bonree.config import setEnvironIp, setEnvironPort


class BottleErrorInterceptor(OnlyStartInterceptor):
    def start_data(self, md, *args, **kwargs):
        btr = self.btr
        if btr:
            btr.add_exception(*sys.exc_info())


class BottleRequestInterceptor(ExitPointInterceptor):
    def end_data(self, md, req, *args, **kwargs):
        btr = self.btr
        if not btr:
            return
        content_type = req.content_type
        if is_content_type(content_type):
            content = req.params.dict()
            text = ''
            for name, value in content.items():
                text += name + ' ' + ' '.join(value)
            btr.request_content = text

class BottleResponseInterceptor(ExitPointInterceptor):
    def end_data(self, md, resp, *args, **kwargs):
        btr = self.btr
        if not btr:
            return

        content_type = resp.content_type if resp.content_type else resp.default_content_type
        # if is_content_type(content_type):
        #     btr.request_content = resp.body
        if resp.body and content_type and 'text/html' in content_type:
            resp.body = self.inject_js_code(resp.body)


class BottleRunInterceptor(OnlyStartInterceptor):
    def start_data(self, md, *args, **kwargs):
        host = kwargs.get('host', '127.0.0.1')
        port = kwargs.get('port', 8080)
        # print('host[%s]' % (host,))
        # print('port[%s]' % (port,))
        setEnvironIp(host)
        setEnvironPort([port,])


def intercept_bottle(agent, mod):
    WSGIInterceptor(agent, mod.Bottle).attach('__call__')
    BottleErrorInterceptor(agent, mod.HTTPError).attach('__init__')
    # BottleRequestInterceptor(agent, mod.BaseRequest).attach('__init__')
    BottleResponseInterceptor(agent, mod.BaseResponse).attach('__init__')
    BottleRunInterceptor(agent, mod).attach('run')