# Copyright (c) Bonree, Inc., and its affiliates
# 2017
# All Rights Reserved

from __future__ import unicode_literals
import sys

from Bonree.agent.interceptor.frameworks import ExitPointInterceptor
from Bonree.config import is_content_type, getgetJsConfigJs
from Bonree.agent.interceptor.frameworks.wsgi import WSGIInterceptor
from Bonree.agent.interceptor.base import OnlyStartInterceptor
from Bonree.config import setEnvironIp, setEnvironPort
from Bonree.lang import PY3


class CherrypyExceptionAdder(OnlyStartInterceptor):
    def start_data(self, md, *args, **kwargs):
        btr = self.btr
        if btr:
            btr.add_exception(*sys.exc_info())


class CherrypyRequestInterceptor(ExitPointInterceptor):
    def end_data(self, md, req, *args, **kwargs):
        btr = self.btr
        if not btr:
            return

        content_type = req.body.content_type.value
        if is_content_type(content_type):
            btr.request_content = req.body.fp.read()

class CherrypyResponseInterceptor(ExitPointInterceptor):
    def start_data(self, md, response):
        btr = self.btr
        if not btr:
            return

        content_type = response.headers.get('Content-Type', None)
        if content_type and 'text/html' in content_type and getgetJsConfigJs() and 'utf-8' in content_type:
            if isinstance(response.body, list) and response.body:
                body = response.body[0]     # body[0] is `str` in python2 and `bytes` in python3.
                if not PY3 and isinstance(body, str) :
                    body = body.decode('utf-8') # `str` to `unicode` in python2
                if PY3 and isinstance(body, bytes):
                    body = body.decode('utf-8')
                body = self.inject_js_code(body)
                if not PY3 and isinstance(body, unicode):
                    # `unicode` to `str` in python2
                    body = body.encode('utf-8')
                if PY3:
                    body = body.encode('utf-8')
                response.body[0] = body

        # if is_content_type(content_type) and isinstance(response.body, list) and response.body:
        #     btr.response_content = ' '.join(response.body[0])

class CherrypyWspBusInterceptor(OnlyStartInterceptor):
    def start_data(self, md, *args, **kwargs):
        import cherrypy
        host = cherrypy.config.get('server.socket_host', '127.0.0.1')
        port = cherrypy.config.get('server.socket_port', 8000)
        # print('host:[%s] port:[%s]' % (host, port,))
        setEnvironIp(host)
        setEnvironPort([port,])

def intercept_cherrypy(agent, mod):
    WSGIInterceptor(agent, mod.Application).attach('__call__')
    CherrypyExceptionAdder(agent, mod._cprequest.Request).attach('handle_error')
    CherrypyExceptionAdder(agent, mod.HTTPError).attach('set_response')


def intercept_cherrypy_wspbus(agent, mod):
    CherrypyWspBusInterceptor(agent, mod.Bus).attach('start')


def intercept_cherrypy_request(agent, mod):
    # CherrypyRequestInterceptor(agent, mod.Request).attach('_do_respond')
    CherrypyResponseInterceptor(agent, mod.Response).attach('collapse_body')