# Copyright (c) Bonree, Inc., and its affiliates
# 2016
# All Rights Reserved
from __future__ import unicode_literals
import sys

from Bonree.agent.interceptor.frameworks import ExitPointInterceptor
from Bonree.agent.interceptor.frameworks.wsgi import WSGIInterceptor
from Bonree.agent.interceptor.base import BaseInterceptor, OnlyEndInterceptor, OnlyStartInterceptor
from Bonree.config import is_content_type, getgetJsConfigJs
from Bonree.logger import SYS_LOGGER
from Bonree.config import setEnvironIp, setEnvironPort


class DjangoBaseHandlerInterceptor(OnlyEndInterceptor):
    def end_data(self, md, base_handler, *args, **kwargs):
        base_handler._exception_middleware.insert(0, BonrDjangoMiddleware(self).process_exception)

class DjangoRequestInterceptor(BaseInterceptor):
    def start_data(self, md, base, request):
        btr = self.btr
        if not btr:
            return

        try:
            content_type = request.content_type
        except Exception:
            content_type = getattr(request, 'META', {})
            content_type = content_type.get('CONTENT_TYPE', None)
        if is_content_type(content_type):
            btr.request_content = request.body


class DjangoExceptionInterceptor(OnlyStartInterceptor):
    def start_data(self, md, request, resolver, exc_info):
        btr = self.btr
        if btr:
            btr.add_exception(*exc_info)


class BonrDjangoMiddleware(object):
    def __init__(self, interceptor):
        self.interceptor = interceptor

    def process_exception(self, request, exception):
        with self.interceptor.log_exceptions():
            btr = self.interceptor.btr
            if btr:
                btr.add_exception(*sys.exc_info())


class DjangoResponseInterceptor(OnlyEndInterceptor, ExitPointInterceptor):
    def end_data(self, md, ins, content=b'', *args, **kwargs):
        if not getgetJsConfigJs():
            return
        btr = self.btr
        if not btr:
            return

        btr.__inject_js__ = False
        content_type = ins.get('content-type', None)
        if content_type and 'text/html' in content_type:
            content = ins.content
            if isinstance(content, bytes) and 'utf-8' in content_type:
                content = content.decode('utf-8')
                ins.content = self.inject_js_code(content)
        # if is_content_type(content_type):
        #     btr.response_content = ins.content


class DjangoRenderInterceptor(OnlyEndInterceptor, ExitPointInterceptor):
    def end_data(self, md, simpleresponse):
        if not getgetJsConfigJs():
            return
        btr = self.btr
        if not btr:
            return
        content_type = simpleresponse.get('Content-Type', None)
        if hasattr(self.btr, '__inject_js__') and btr.__inject_js__ is False:
            if content_type and 'text/html' in content_type:
                content = simpleresponse.content
                if isinstance(content, bytes) and 'utf-8' in content_type:
                    content = content.decode('utf-8')
                    simpleresponse.content = self.inject_js_code(content)
            # if is_content_type(content_type):
            #     btr.response_content = simpleresponse.content

class DjangoRunServer(OnlyStartInterceptor):
    def start_data(self, md, origin_self, *args, **kwargs):
        SYS_LOGGER.info('django inner_run addr[%s] port[%s]' % (origin_self.addr, origin_self.port))
        setEnvironIp(origin_self.addr)
        setEnvironPort([origin_self.port,])


def intercept_django_wsgi_handler(agent, mod):
    WSGIInterceptor(agent, mod.WSGIHandler).attach('__call__')


def intercept_django_asgi_handler(agent, mod):
    from Bonree.agent.interceptor.frameworks.asgi import ASGIInterceptor
    ASGIInterceptor(agent, mod.ASGIHandler).attach('__call__')


def intercept_django_base_handler(agent, mod):
    DjangoBaseHandlerInterceptor(agent, mod.BaseHandler).attach('load_middleware')
    # DjangoRequestInterceptor(agent, mod.BaseHandler).attach('get_response')


def intercept_uncaught_exception(agent, mod):
    DjangoExceptionInterceptor(agent, mod).attach('handle_uncaught_exception')


def intercept_django_response(agent, mod):
    DjangoResponseInterceptor(agent, mod.HttpResponse).attach('__init__')


def intercept_django_render(agent, mod):
    DjangoRenderInterceptor(agent, mod.SimpleTemplateResponse).attach('render')

def intercept_django_runserver(agent, mod):
    DjangoRunServer(agent, mod.Command).attach('inner_run')
