# Copyright (c) Bonree, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Interceptor for Flask framework.

"""
import sys

from Bonree.agent.interceptor.frameworks import ExitPointInterceptor, inject_js_code
from Bonree.agent.interceptor.frameworks.wsgi import WSGIInterceptor
from Bonree.agent.interceptor.base import BaseInterceptor, OnlyStartInterceptor, simple_wrapper
from Bonree.config import is_content_type, getgetJsConfigJs
from Bonree.lib import get_btr_from_attr
from Bonree.logger import only_log_exception


class FlaskResponseInterceptor(ExitPointInterceptor):
    def _set_data(self, method, ins, value):
        if getgetJsConfigJs():
            with self.log_exceptions():
                btr = self.btr
                if btr:
                    content_type = ins.headers.get('Content-Type', None)
                    if content_type and 'text/html' in content_type:
                        value = self.inject_js_code(value)
                    # if is_content_type(content_type):
                    #     btr.response_content = value
        return method(ins, value)


def flask_js_inject(original_method, ins, value):
    if not getgetJsConfigJs():
        return original_method(ins, value)

    with only_log_exception:
        btr = get_btr_from_attr()
        if btr:
            content_type = ins.headers.get('Content-Type', None)
            if content_type and 'text/html' in content_type:
                value = inject_js_code(value, btr)
    return original_method(ins, value)


def flask_user_exception(original_method, *args, **kwargs):
    btr = get_btr_from_attr()
    if btr:
        btr.add_exception(*sys.exc_info())
    return original_method(*args, **kwargs)


def intercept_flask(agent, mod):
    WSGIInterceptor(agent, mod.Flask).attach('wsgi_app')
    simple_wrapper(mod.Flask, 'handle_user_exception', flask_user_exception)


def intercept_flask_response(agent, mod):
    #FlaskResponseInterceptor(agent, mod.Response).attach('set_data', patched_method_name='_set_data', entry=True)
    simple_wrapper(mod.Response, 'set_data', flask_js_inject)


def intercept_flask_request(agent, mod):
    # FlaskRequestInterceptor(agent, mod.RequestContext).attach('__init__')
    pass
