from types import FunctionType, MethodType

from Bonree.agent.interceptor.base import CustomBtInterceptor
from Bonree.agent.interceptor.frameworks.openstack import OpenStackBackGroundInterceptor
from Bonree.agent.interceptor.frameworks.wsgi import WSGIInterceptor
from Bonree.logger import SYS_LOGGER


def intercept_nova_compute_api(agent, mod):
    """nova.conductor.rpcapi"""
    methods = ['object_action', 'object_backport_versions', 'object_class_action', 'object_class_action_versions']
    for method in methods:
        OpenStackBackGroundInterceptor(agent, mod.ConductorAPI).set_bt_name(method).attach(method, 'custom_bt_entry', True, True)


def intercept_nova_conductor_api(agent, mod):
    cls = mod.ComputeTaskAPI
    for method in dir(cls):
        if method.startswith('_'):
            continue
        obj = getattr(cls, method, None)
        if isinstance(obj, (FunctionType, MethodType)):
            #SYS_LOGGER.info('hook %s.%s' % (cls.__name__, method))
            OpenStackBackGroundInterceptor(agent, cls).set_bt_name(method).attach(method, 'custom_bt_entry', True, True)


def intercept_nova_scheduler_api(agent, mod):
    """from nova.scheduler.rpcapi"""
    cls = mod.SchedulerAPI
    for method in dir(cls):
        if method.startswith('_'):
            continue
        obj = getattr(cls, method, None)
        if isinstance(obj, (FunctionType, MethodType)):
            #SYS_LOGGER.info('hook %s.%s' % (cls.__name__, method))
            OpenStackBackGroundInterceptor(agent, cls).set_bt_name(method).attach(method, 'custom_bt_entry', True, True)


def intercept_nova_conductor_manage(agent, mod):
    """nova.conductor.manager"""
    cls = mod.ConductorManager
    for method in dir(cls):
        if method.startswith('_'):
            continue
        obj = getattr(cls, method, None)
        if isinstance(obj, (FunctionType, MethodType)):
            # SYS_LOGGER.info('hook %s.%s' % (cls.__name__, method))
            OpenStackBackGroundInterceptor(agent, cls).set_bt_name(method).attach(method, 'custom_bt_entry', True, True)


def intercept_nova_compute_manage(agent, mod):
    """nova.compute.manager.ComputeManager"""
    cls = mod.ComputeManager
    for method in dir(cls):
        if method.startswith('_'):
            continue
        obj = getattr(cls, method, None)
        if isinstance(obj, (FunctionType, MethodType)):
            # SYS_LOGGER.info('hook %s.%s' % (cls.__name__, method))
            OpenStackBackGroundInterceptor(agent, cls).set_bt_name(method).attach(method, 'custom_bt_entry', True, True)


def intercept_nova_schedule_manage(agent, mod):
    """nova.scheduler.manager.SchedulerManager"""
    cls = mod.SchedulerManager
    for method in dir(cls):
        if method.startswith('_'):
            continue
        obj = getattr(cls, method, None)
        if isinstance(obj, (FunctionType, MethodType)):
            # SYS_LOGGER.info('hook %s.%s' % (cls.__name__, method))
            OpenStackBackGroundInterceptor(agent, cls).set_bt_name(method).attach(method, 'custom_bt_entry', True, True)
