# Copyright (c) Bonree, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Interceptor for Pyramid.

"""

from __future__ import unicode_literals
import sys

from Bonree.agent.interceptor.frameworks import ExitPointInterceptor
from Bonree.agent.interceptor.frameworks.wsgi import WSGIInterceptor
from Bonree.agent.interceptor.base import EntryPointInterceptor, OnlyStartInterceptor
from Bonree.config import is_content_type


class PyramidRouterInterceptor(OnlyStartInterceptor):

    def start_data(self, md, router_instance, req):
        btr = self.btr
        if not btr:
            return

        content_type = req.content_type
        if is_content_type(content_type):
            btr.request_content = req.body


class PyramidResponseInterceptor(EntryPointInterceptor, ExitPointInterceptor):

    def init(self, origin_init, instance, body=None, status=None, headerlist=None, app_iter=None,
                   content_type=None, *args, **kwargs):
        with self.log_exceptions():
            btr = self.btr
            if btr:
                if content_type and 'text/html' in content_type:
                    body = self.inject_js_code(body)
                # if is_content_type(content_type):
                #     btr.response_content = body
        return origin_init(instance, body, status=None, headerlist=None, app_iter=None,
                   content_type=None, *args, **kwargs)



def intercept_pyramid(agent, mod):
    WSGIInterceptor(agent, mod.Router).attach('__call__')
    # PyramidRouterInterceptor(agent, mod.Router).attach('handle_request')


def intercept_pyramid_response(agent, mod):
    PyramidResponseInterceptor(agent, mod.Response).attach('__init__', patched_method_name='init')
