# Copyright (c) Bonree, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Interceptor for Flask framework.

"""
import sys

from Bonree.agent.interceptor.frameworks import ExitPointInterceptor

from Bonree.agent.interceptor.base import OnlyStartInterceptor, OnlyEndInterceptor
from Bonree.config import is_content_type, setEnvironIp, setEnvironPort


class QuartResponseInterceptor(ExitPointInterceptor):
    def _set_data(self, method, ins, value):
        with self.log_exceptions():
            btr = self.btr
            if btr:
                content_type = ins.headers.get('Content-Type', None)
                if content_type and 'text/html' in content_type:
                    value = self.inject_js_code(value)
                # if is_content_type(content_type):
                #     btr.response_content = value
        return method(ins, value)


class QuartRequestInterceptor(OnlyEndInterceptor):
    def end_data(self, md, app, *args, **kwargs):
        btr = md.btr
        if btr:
            content_type = app.request.content_type
            if is_content_type(content_type):
                btr.request_content = app.request.data


class QuartMakeServerInterceptor(OnlyStartInterceptor):
    def start_data(self, md, app, host='127.0.0.1', port=5000, *args, **kwargs):
        setEnvironIp(host)
        setEnvironPort([port,])

def intercept_quart(agent, mod):
    from Bonree.agent.interceptor.frameworks.asgi import ASGIInterceptor
    ASGIInterceptor(agent, mod.Quart).attach('asgi_app')
    QuartMakeServerInterceptor(agent, mod.Quart).attach('run')

    # handler quart exception
    from Bonree.agent.interceptor.async_base import AsyncOnlyStartInterceptor
    class QuartUserExceptionHandler(AsyncOnlyStartInterceptor):
        def start_data(self, md, *args, **kwargs):
            btr = self.btr
            if btr:
                btr.add_exception(*sys.exc_info())

    QuartUserExceptionHandler(agent, mod.Quart).attach('handle_user_exception')


def intercept_quart_response(agent, mod):
    QuartResponseInterceptor(agent, mod.Response).attach('set_data', patched_method_name='_set_data', entry=True)


def intercept_quart_request(agent, mod):
    # QuartRequestInterceptor(agent, mod.RequestContext).attach('__init__')
    pass
