# Copyright (c) AppDynamics, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Base class for HTTP connection interceptors.

"""

from __future__ import unicode_literals

from Bonree import rule
from ..base import ExitCallInterceptor
from Bonree.lang import str, urlparse
from Bonree.agent.proto.pb_type import BACKEND_TYPE_HTTP


class MessageQueueInterceptor(ExitCallInterceptor):
    def __init__(self, agent, cls, vendor=None):
        super(MessageQueueInterceptor, self).__init__(agent, cls)
        self.vendor = vendor


class HTTPConnectionInterceptor(ExitCallInterceptor):
    # If the library you are intercepting has an HTTPSConnection class which
    # does not subclass httplib.HTTPSConnection, add it to this set.
    https_connection_classes = set()
    backend_name_format_string = '%s-{HOST}:{PORT}'

    def make_tags(self, *args, **kwargs):
        tags  = {}
        btr = self.btr
        if btr:
            tags['http.method'] = btr.request.method if btr.request else ''
            tags['http.status_code'] = str(btr.status_code) if btr.status_code else ''
            tags['http.url'] = rule.url_del_parameter(btr.request.url) if btr.request else ''
            tags['peer.service'] = 'http'
        return tags

    @classmethod
    def _request_is_https(cls, connection):
        if connection.port == 443:
            return True
        return isinstance(connection, tuple(cls.https_connection_classes))

    def get_backend(self, host, port, scheme, url):
        parsed_url = urlparse(url)
        if port in (80, 443, '80', '443'):
            URL = '%s://%s' % (scheme, host)
        else:
            URL = '%s://%s:%s' % (scheme, host, port)
        backend_properties = {
            'HOST': host,
            'PORT': str(port),
            'URL': URL, #str(url),
            'QUERY STRING': parsed_url.query,
        }
        backend = self.agent.backend_registry.get_backend(BACKEND_TYPE_HTTP, backend_properties,
                                                       self.backend_name_format_string % scheme)
        backend.connType = scheme  #connection.__class__
        return backend


from .httplib import intercept_httplib, intercept_http_client
from .urllib3 import intercept_urllib3
from .requests import intercept_requests
from .tornado_httpclient import intercept_tornado_httpclient
from .rabbitmq import intercept_rabbitmq
from .thrift import (interceptor_thrift, interceptor_thrift_binary_field, interceptor_thrift_json_field,
                     interceptor_thrift_compact_field, interceptor_thrift_skip)
from .grpc import intercept_grpc, intercept_grpc_channel, intercept_grpc_server, intercept_xmlrpc_server
from .activemq import intercept_activemq, intercept_listen_activemq, intercept_stompy, intercept_stompy_frame
from .kafka import intercept_kafka_python, intercept_kafka_consumer
from .elasticsearch import intercept_elasticsearch
from .aiohttp import intercept_aiohttp_request
from .oslo_messaging import intercept_oslo_message_rpc_client, intercept_oslo_message_rabbit

__all__ = ['intercept_httplib', 'intercept_http_client', 'intercept_urllib3', 'intercept_requests', 'intercept_tornado_httpclient', 'intercept_rabbitmq', 'interceptor_thrift', 'intercept_grpc', 'intercept_grpc_channel', 'intercept_activemq', 'intercept_kafka_python', 'intercept_kafka_consumer', 'intercept_elasticsearch', 'intercept_xmlrpc_server', 'interceptor_thrift_binary_field', 'interceptor_thrift_json_field',
           'interceptor_thrift_compact_field', 'interceptor_thrift_skip', 'intercept_aiohttp_request',
           'intercept_oslo_message_rpc_client', 'intercept_oslo_message_rabbit', ]
