from Bonree.agent.interceptor.base import ExitCallInterceptor
from Bonree.agent.interceptor.http import HTTPConnectionInterceptor
from Bonree.agent.models.cross import BR_CS_REQ
from Bonree.agent.proto.pb_type import BACKEND_TYPE_HTTP
from Bonree.lang import parse_url

def intercept_aiohttp_request(agent, mod):
    from Bonree.agent.interceptor.async_base import AsyncBaseInterceptor

    class AioHttpRequestInterceptor(AsyncBaseInterceptor, HTTPConnectionInterceptor):
        backend_name_format_string = '%s-{HOST}:{PORT}'
        def start_data(self, md, instance, method, str_or_url, **kwargs):
            if method != 'GET' and method != 'POST':
                return
            btr = self.btr
            if not btr:
                return
            url = str(str_or_url)
            bonree_url = parse_url(url)

            scheme = bonree_url.scheme.capitalize() if bonree_url.scheme else 'Http'
            host = bonree_url.hostname
            port = bonree_url.port or ('443' if scheme == 'Https' else '80')

            md.is_rpc = True
            md.custom = True
            md.backend = self.get_backend(host, port, scheme, url)
            md.operation = url
            md.params = bonree_url.params
            md.response_method = self.parse_http_response
            self.btrace_data = True
            self.start_exit_call(md)
            self.start_trace_call(md)
            self.btrace_data = False
            if md.cross_head and isinstance(md.cross_head, bytes):
                br_cs_req = md.cross_head.decode('latin1')
                if 'headers' not in md.kwargs:
                    md.kwargs['headers'] = {}
                headers = md.kwargs['headers']
                headers[BR_CS_REQ] = br_cs_req

        @staticmethod
        def parse_http_response(response):
            if not hasattr(response, 'headers'):
                return {}
            response_dict_header = response.headers
            return response_dict_header

        def end_data(self, md, instance, method, str_or_url, **kwargs):
            md.custom = True
            md.handle_rpc_http_status_code(md.response)
            self.end_trace_call(md)
            self.end_exit_call(md)

        def make_tags(self, md, instance, method, str_or_url, **kwargs):
            tags = {}
            tags['http.method'] = str(method)
            tags['http.url'] = str(str_or_url)
            tags['peer.service'] = 'aiohttp'
            md.tags = tags


    AioHttpRequestInterceptor(agent, mod.ClientSession).attach('_request')
