# Copyright (c) Bonr, Inc., and its affiliates
# 2016
# All Rights Reserved

"""Interceptor for httplib/http.client.

"""
from __future__ import unicode_literals

from Bonree.lang import str
from Bonree.config import PY3
from Bonree.lib import now
from . import HTTPConnectionInterceptor
from Bonree.agent.models.cross import BR_CS_REQ
from Bonree.agent.interceptor.base import OnlyEndInterceptor, ExitCallInterceptor


class HttplibInterceptor(HTTPConnectionInterceptor):
    @staticmethod
    def parse_http_response(response):
        response_dict_header = {}
        if hasattr(response, 'msg') and hasattr(response.msg, 'dict'):
            response_dict_header = response.msg.dict
        elif hasattr(response, 'msg') and isinstance(response.msg._headers, list):
            for item in response.msg._headers:
                if isinstance(item, tuple):
                    response_dict_header[item[0]] = item[1]

        return response_dict_header

    def make_tags(self, md, connection, *args, **kwargs):
        tags  = {}
        tags['http.method'] = connection._method
        if md.backend:
            tags['http.url'] = md.backend.identifying_properties.get('URL', None)
        tags['peer.service'] = 'httplib'
        md.tags = tags

    @staticmethod
    def get_url(connection, scheme):
        """the url must be exist."""
        Host = ''
        method = ''
        if not PY3:
            for buf in connection._buffer:
                if connection._method in buf:
                    method = buf.split()[1]
                if 'Host' in buf:
                    Host = buf.split()[1]
        else:
            for buf in connection._buffer:
                if connection._method in str(buf):
                    method = str(buf.split()[1])
                if 'Host' in str(buf):
                    Host = str(buf.split()[1])
        if Host:
            return scheme + '://' + Host + method
        else:
            return ''


class HttplibEndheadersInterceptor(HttplibInterceptor):
    def start_data(self, md, connection, *args, **kwargs):
        scheme = 'Https' if self._request_is_https(connection) else 'Http'
        url = self.get_url(connection, scheme)
        md.backend = self.get_backend(connection.host, connection.port, scheme, url)
        if self.btr and md.backend:
            md.operation="{0}".format(url)
            md.is_rpc = True
            md.custom = True
            setattr(connection, '_bonr_method_data', md)
            self.btrace_data = True
            self.start_exit_call(md)
            self.start_trace_call(md)

            connection.putheader(BR_CS_REQ, md.cross_head)
            md.response_method = self.parse_http_response
            self.btrace_data = False
            md.exit_flag = True

    def end_trace_call(self, *args, **kwargs):
        pass

    def end_exit_call(self, *args, **kwargs):
        pass


class HttplibGetResponseInterceptor(OnlyEndInterceptor, ExitCallInterceptor):
    def start_trace_call(self, *args, **kwargs):
        pass

    def start_exit_call(self, *args, **kwargs):
        pass

    def end_data(self, md, connection, *args, **kwargs):
        if hasattr(connection, '_bonr_method_data'):
            start_md = getattr(connection, '_bonr_method_data')
            start_md.exit_flag = False
            start_md.response = md.response
            start_md.handle_rpc_http_status_code(start_md.response)
            self.btrace_data = True
            self.end_trace_call(start_md)
            self.end_exit_call(start_md)
            delattr(connection, '_bonr_method_data')
        self.btrace_data = False




def interceptor_http(agent, mod):
    HTTPConnectionInterceptor.https_connection_classes.add(mod.HTTPSConnection)
    interceptor = HttplibEndheadersInterceptor(agent, mod.HTTPConnection)
    interceptor.attach('endheaders')

def intercept_httplib(agent, mod):
    interceptor_http(agent, mod)
    HttplibGetResponseInterceptor(agent, mod.HTTPConnection).attach('getresponse',btrace_data=True)

def intercept_http_client(agent, mod):
    interceptor_http(agent, mod)
    HttplibGetResponseInterceptor(agent, mod.HTTPConnection).attach('getresponse',btrace_data=True)

