# Copyright (c) Bonr, Inc., and its affiliates
# 2016
# All Rights Reserved

"""Intercept requests to ensure that HTTPS is reported correctly.

"""

from __future__ import unicode_literals

from Bonree.lang import parse_url
from .urllib3 import intercept_urllib3
from . import HTTPConnectionInterceptor
from Bonree.agent.models.cross import BR_CS_REQ
from Bonree.agent.services.cross import parse_cross_response_header


class RequestInterceptor(HTTPConnectionInterceptor):
    def start_data(self, md, session, method, url, *args, **kwargs):
        bonree_url = parse_url(url)
        scheme = bonree_url.scheme.capitalize() if bonree_url.scheme else 'Http'
        host = bonree_url.hostname
        port = bonree_url.port or ('443' if scheme == 'Https' else '80')
        md.backend = self.get_backend(host, port, scheme, url)
        md.is_rpc = True
        md.operation = bonree_url.path
        md.params = bonree_url.params
        md.response_method = self.parse_http_response

        md.custom = True
        self.start_exit_call(md)
        self.start_trace_call(md)

        if md.cross_head:
            session.headers[BR_CS_REQ] = md.cross_head
            md.args[0] = session
        md.response_method = self.parse_http_response

    @staticmethod
    def parse_http_response(response):
        if not hasattr(response, 'headers'):
            return {}
        response_dict_header = response.headers
        return response_dict_header

    def make_tags(self, md, session, method, url, *args, **kwargs):
        tags  = {}
        tags['http.method'] = str(method)
        tags['http.url'] = url
        tags['peer.service'] = 'requests'
        md.tags = tags

def intercept_requests(agent, mod):
    # requests ships with its own version of urllib3, so we need to manually intercept it.
    if(hasattr(mod, 'packages')):
        intercept_urllib3(agent, mod.packages.urllib3)
    elif(hasattr(mod, 'Session')):
        RequestInterceptor(agent, mod.Session).attach('request', btrace_data=True)
