import functools

from Bonree.lang import urlparse
from . import HTTPConnectionInterceptor
from Bonree.agent.models.cross import BR_CS_REQ
from Bonree.agent.services.cross import parse_cross_response_header


def intercept_tornado_httpclient(agent, mod):
    import tornado

    if getattr(tornado, 'version_info', (0, 0, 0, 0)) < (3, 0, 0, 0):
        agent.logger.warning("Older versions of tornado < 3.0.0.0 not are not supported. Skipping instrumentation")
    else:
        import tornado.httputil
        try:
            # Tornado 6.0 remove `tornado.stack_context` module
            import tornado.stack_context as stack_context
        except ImportError:
            stack_context = None

        class AsyncHTTPClientInterceptor(HTTPConnectionInterceptor):
            def end_exit_call(self, md, future):
                if md:
                    try:
                        exception_info = future.exc_info()
                    except AttributeError:
                        # this is for _DummyFuture
                        exception = getattr(future, '_exception', None)
                        if exception:
                            exception_info = (exception.__class__,
                                              exception,
                                              getattr(exception, '__traceback__', None))
                        else:
                            exception_info = None

                    if exception_info is None:
                        md.response_method = self.parse_http_response

                    md.exc_info = exception_info
                    super(AsyncHTTPClientInterceptor, self).end_exit_call(md)

            def start_data(self, md, client, request, *args, **kwargs):
                if isinstance(request, str) or isinstance(request, mod.HTTPRequest):
                    if isinstance(request, mod.HTTPRequest):
                        url = urlparse(request.url)
                        full_url = request.url
                        headers = tornado.httputil.HTTPHeaders(request.headers)
                    else:
                        url = urlparse(request)
                        full_url = request
                        kwargs.setdefault('headers', tornado.httputil.HTTPHeaders())
                        headers = kwargs['headers']

                    port = url.port or ('443' if url.scheme == 'https' else '80')
                    md.backend = self.get_backend(url.hostname, port, url.scheme.capitalize(), full_url)
                    md.operation = url.path
                    md.is_rpc = True
                    self.start_exit_call(md)
                    if md.cross_head:
                        if isinstance(headers, tornado.httputil.HTTPHeaders):
                            headers.add(BR_CS_REQ, md.cross_head)
                        elif isinstance(headers, dict):
                            headers[BR_CS_REQ] = md.cross_head
                    md.kwargs = kwargs

            def end_data(self, md, *args, **kwargs):
                future = md.response
                md.custom = True
                self.end_trace_call(md)

                if future is not None and stack_context:
                    callback = functools.partial(stack_context.wrap(self.end_exit_call), md)
                    if future._callbacks is None:
                        future.add_done_callback(callback)
                    else:
                        future._callbacks.insert(0, callback)

            @staticmethod
            def parse_http_response(future):
                response = future.result()
                response_dict_header = response.headers
                return parse_cross_response_header(response_dict_header)

            def make_tags(self, md, client, request, *args, **kwargs):
                tags  = {}
                tags['peer.service'] = 'httpclient'
                if isinstance(request, mod.HTTPRequest):
                    tags['http.method'] = str(request.method)
                    tags['http.url'] = str(request.url)
                else:
                    tags['http.method'] = 'GET'
                    tags['http.url'] = str(request)
                md.tags = tags

        AsyncHTTPClientInterceptor(agent, mod.AsyncHTTPClient).attach('fetch')
