# Copyright (c) Bonree, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Intercept the logbook.

"""
from Bonree.agent.interceptor.base import OnlyEndInterceptor
from Bonree.config import getTraceEnableAddTraceidToApplog, getPythonEnableDetectLogBook

LEVEL_DICT = {
    0: 'NOTSET',
    9: 'TRACE',
    10: 'DEBUG',
    11: 'INFO',
    12: 'NOTICE',
    13: 'WARNING',
    14: 'ERROR',
    15: 'CRITICAL',
}

class LogbookFetchIntercept(OnlyEndInterceptor):
    def end_data(self, md, handler, record, *args, **kwargs):
        message = md.response
        if not message or not isinstance(message, str):
            return

        btr = self.btr
        if btr and getPythonEnableDetectLogBook():
            level = getattr(record, 'level', 0)
            btr.detect_logs.append({'level': LEVEL_DICT.get(level, 'NOTSET'), 'logMsg': message})


class LogbookAppendTraceIdIntercept(OnlyEndInterceptor):
    def end_data(self, md, record, *args, **kwargs):
        btr = self.btr
        if btr and btr.cross_svc.trace_id and getTraceEnableAddTraceidToApplog() and isinstance(record.msg, str):
            record.msg = "traceId=%s %s" % (btr.cross_svc.trace_id, record.msg)


def intercept_logbook(agent, mod):
    LogbookFetchIntercept(agent, mod.Handler).attach('format')


def intercept_logbook_append_trace_id(agent, mod):
    LogbookAppendTraceIdIntercept(agent, mod.LogRecord).attach('__init__')
