# Copyright (c) Bonree, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Intercept the Logging.

"""
import re

from Bonree.agent.interceptor.base import OnlyEndInterceptor, simple_wrapper
from Bonree.config import getTraceEnableAddTraceidToApplog, getPythonEnableDetectLogging
from Bonree.lib import get_btr_from_attr

LEVEL_DICT = {
     0: 'NOTSET',
    10: 'DEBUG',
    20: 'INFO',
    30: 'WARNING',
    40: 'ERROR',
    50: 'CRITICAL',
}

color_re = re.compile(r'\033\[\d+m')


def is_bonree_logger(record):
    return record.name == 'Bonree.agent'


def logging_fetch_intercept(original_method, handler, record, *args, **kwargs):
    if not getPythonEnableDetectLogging() or is_bonree_logger(record):
        return original_method(handler, record, *args, **kwargs)

    message = original_method(handler, record, *args, **kwargs)
    if not message or not isinstance(message, str):
        return message

    btr = get_btr_from_attr()
    if btr:
        level = getattr(record, 'levelno', 0)
        message = color_re.sub("", message)  # filter color flag
        btr.detect_logs.append({'level': LEVEL_DICT.get(level, 'NOTSET'), 'logMsg': message})
    return message


def logging_append_trace_id(original_method, record, *args, **kwargs):
    if not getTraceEnableAddTraceidToApplog():
        return original_method(record, *args, **kwargs)

    original_method(record, *args, **kwargs)
    if is_bonree_logger(record):
        return
    btr = get_btr_from_attr()
    if btr and btr.cross_svc.trace_id and isinstance(record.msg, str):
        record.msg = "traceId=%s %s" % (btr.cross_svc.trace_id, record.msg)


def intercept_logging(agent, mod):
    simple_wrapper(mod.Handler, 'format', logging_fetch_intercept)


def intercept_logging_append_trace_id(agent, mod):
    simple_wrapper(mod.LogRecord, '__init__', logging_append_trace_id)
