
from __future__ import unicode_literals

import re
import sys

from Bonree import rule
from Bonree.agent.models.methods import MethodData
from ..base import ExitCallInterceptor
from Bonree.agent.models.methods import exit_call_summary
from Bonree.lang import str
from Bonree.agent.proto.pb_type import BACKEND_TYPE_MONGODB


def intercept_pymongo(agent, mod):
    class ExitCallListener(mod.monitoring.CommandListener):
        backend_name_format_string = rule.common_name()

        def __init__(self):
            self.interceptor = ExitCallInterceptor(agent, mod)
            self.interceptor.btrace_data = True
            self.md_map = {}
            self.vendor = rule.vendor_conversion('Mongo')
            self.tags = {}

        def get_backend(self, connection_id, database_name):
            host, port = connection_id
            backend_properties = {
                'HOST': host,
                'PORT': str(port),
                'VENDOR': self.vendor,
                'DATABASE': database_name,
                'URL': rule.url_sql_conversion('mongodb', host, port, database_name)
            }
            backend = self.interceptor.agent.backend_registry.get_backend(BACKEND_TYPE_MONGODB,
                                                                          backend_properties,
                                                                          self.backend_name_format_string)
            backend.connType = rule.connType_conversion('mongodb')
            self.tags['peer.address'] = rule.peer_address_conversion(backend_properties.get('URL'))
            self.tags['db.instance'] = str(database_name)
            self.tags['db.type'] = backend.connType
            return backend

        def started(self, event):
            with self.interceptor.log_exceptions():
                btr = self.interceptor.btr
                if not btr:
                    return
                md = MethodData((event, ), {}, self.interceptor.cls, self.started, btr)
                md.backend = self.get_backend(event.connection_id, event.database_name)
                if md.backend:
                    operation = str(event.command)
                    if operation.startswith('SON'):
                        rep_str = re.search(r"\('lsid',.*?}\), ", operation)
                        if rep_str:
                            try:
                                operation = operation.replace(rep_str.group(), '')
                            except Exception:
                                pass
                    md.operation = operation

                    self.interceptor.start_exit_call(md)
                    self.interceptor.start_trace_call(md)

                    self.md_map[event.operation_id] = md
                    self.tags['db.statement'] = exit_call_summary(md.backend.tipe,str(event.command))

        def succeeded(self, event):
            md = self.md_map.pop(event.operation_id, None)
            if md:
                md.tags = self.tags
                self.interceptor.end_trace_call(md)
                self.interceptor.end_exit_call(md)

        def failed(self, event):
            md = self.md_map.pop(event.operation_id, None)
            if md:
                md.tags = self.tags
                md.error = self.interceptor.add_error(*sys.exc_info())
                self.interceptor.end_trace_call(md)
                self.interceptor.end_exit_call(md)

    mod.monitoring.register(ExitCallListener())