# Copyright (c) Bonree, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Intercept the MySQLdb package.

"""
import weakref

from .dbapi import DbAPICursorInterceptor, db_api_attach


def get_backend_properties(conn, *args, **kwargs):
    try:
        host = args[0]
    except IndexError:
        host = kwargs.get('host', 'localhost')
    try:
        port = args[4]
    except IndexError:
        port = kwargs.get('port', '3306')
    try:
        db = args[3]
    except IndexError:
        db = kwargs.get('database', kwargs.get('db', ''))
    try:
        version = '.'.join(str(x) for x in conn._server_version)
    except Exception:
        version = ''

    return host, port, db, version


class MySQLdbCursorInterceptor(DbAPICursorInterceptor):
    def get_connection(self, cursor):
        connection = cursor.connection
        if isinstance(connection, weakref.ReferenceType):   # MySQLdb <= 1.3.0
            connection = connection()
        return connection


def intercept_MySQLdb_connection(agent, mod):
    # MySQLdbConnectionInterceptor(agent, mod.Connection, MySQLdbCursorInterceptor, 'MySQL').attach('__init__')

    db_api_attach(agent, mod.Connection, MySQLdbCursorInterceptor, get_backend_properties, '__init__', 'MySQL')

