# Copyright (c) Bonree, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Models for errors and exceptions.

"""
import traceback

from Bonree.lib import now_ms


class ErrorInfo(object):

    __slots__ = ('name', 'sumary', 'detail', 'time')

    def __init__(self, name, message):
        self.name = name
        self.sumary = name
        self.detail = message
        self.time = now_ms()


class ExceptionInfo(object):
    
    __slots__ = ('name', 'sumary', 'detail', 'time')

    def __init__(self, exc_type, exc_value, exc_tb):
        self.name = get_exception_name(exc_value)
        self.sumary = get_exception_message(exc_value)
        self.detail = ''.join(traceback.format_exception(exc_type, exc_value, exc_tb))
        self.time = now_ms()


def get_exception_name(exc):
    """Get the qualified name of an exception.

    This is the name of the exception class, qualified by the module that
    defines it. In the case of builtin exceptions, this is just the exception
    class's name (e.g., ``'ValueError'``), whereas for other exceptions it
    will have the module name (e.g., ``'Queue.Empty'``).

    Parameters
    ----------
    exc : BaseException

    Returns
    -------
    name : str

    """
    class_ = exc.__class__
    class_name = class_.__name__
    module_name = class_.__module__

    if module_name in ('__builtin__', 'exceptions'):
        return class_name
    else:
        return '%s.%s' % (module_name, class_name)


def get_exception_message(exc):
    """Get the message from an exception.

    Parameters
    ----------
    exc : BaseException

    Returns
    -------
    message : str

    """
    return repr(exc)
