import time
import random
import datetime

from Bonree import config
from Bonree.agent.proto.pb_type import BACKEND_TYPE_HTTP, BT_UNKNOWN
from Bonree.agent.services.wrapper import *
from Bonree.lib import arr_http_header
from Bonree.logger import BT_LOGGER
from Bonree.agent.models.cross import *
from Bonree.lang import items, str


def parse_cross_header(trace_info):
    hdr = trace_info.split(TRACE_SPE)
    cross_header = {}
    for item in hdr:
        if item.startswith((NEED_SAMPLE, IS_ASYNC, TRACE_ID, RPC_ID, FROM_AGENT_ID, FROM_TRANSACTION_ID, RPC_HOST, BACKEND_TYPE, TO_AGENT_ID)):
            cross_header[item[0]] = item[2:]
    return cross_header

def parse_cross_response_header(header):
    hdr_lower = {}
    for k, v in items(header):
        hdr_lower[k.lower()] = v
    if BR_CS_RSP not in hdr_lower:
        return None

    cross_header = parse_cross_header(hdr_lower[BR_CS_RSP])
    BT_LOGGER.debug('trace response header:{0}'.format(cross_header))
    return cross_header

def parse_cross_request_header(header):
    hdr_lower = {}
    for k, v in items(header):
        hdr_lower[k.lower()] = v
    if BR_CS_REQ not in hdr_lower:
        return None

    cross_header = parse_cross_header(hdr_lower[BR_CS_REQ])
    BT_LOGGER.debug('Get trace request ', cross_header)
    return cross_header

def generateTraceId(agent, btr):
    hexchars = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f']
    date10 = datetime.datetime.fromtimestamp(btr.timer.start_time / 1000000).strftime('%y%m%d%H%M')
    datems = int(round(time.time() * 1000)) % (60 * 1000)
    date4 = [0] * 4
    for i in range(4):
        date4[i] = hexchars[datems & 0xf]
        datems = datems >> 4
    date4 = ''.join(str(item) for item in date4)
    agent_id = [0] * 8
    agentid = int(agent.agent_id)
    for i in range(8):
        agent_id[i] = hexchars[agentid & 0xF]
        agentid >>= 4
    agent_id = ''.join(str(item) for item in agent_id)
    thread_id = [0] * 4
    id = int(btr.concurrent_ident)
    for i in range(4):
        thread_id[i] = hexchars[id & 0xf]
        id >>= 4
    thread_id = ''.join(str(item) for item in thread_id)
    random1 = ''.join(random.sample(hexchars, 4))
    random2 = ''.join(random.sample(hexchars, 4))
    return '{0}{1}{2}{3}{4}{5}'.format(date10, date4, agent_id, thread_id, random1, random2)

def cross_dict_to_str(hdr):
    hdr_list = []
    for key, value in items(hdr):
        hdr_list.append(key)
        hdr_list.append(SPLIT_KEYVALUE)
        hdr_list.append('{0}'.format(value))
        hdr_list.append(TRACE_SPE)
    return hdr_list


def make_cross_response_header(agent, btr):

    hdr = {}
    if config.getEnableDataAggregate() and btr.cross_svc.trace_id:
        hdr['br-resp-key'] = '{0}'.format(btr.cross_svc.trace_id)
        # hdr[SET_COOKIE] = 'br-resp-key=g:{0}'.format(btr.cross_svc.trace_id)
    if btr.cross_svc.cross_header:
        hdr[BR_CS_RSP] = '{0}'.format(str(btr.cross_resp))
        BT_LOGGER.debug('Make trace response header: {0}'.format(hdr))
    return hdr

def br_cs_req_from_header(headers):
    if not headers:
        return
    if BR_CS_REQ in headers:
        return "{0}: {1}".format(BR_CS_REQ, headers[BR_CS_REQ])
    return None

def create_cross_resp(btr, empty):
    SO = config.get_sdk_so()

    cross_req_head = CrossReqHead()
    if not empty:
        req_headers = Str().with_value(br_cs_req_from_header(btr.request.headers))
        SO.get_cross_req_head_from_head(pointer(req_headers), pointer(cross_req_head))

    cross_req_options = CrossReqOptions()
    cross_req_options.is_sampling = int(config.getEnableTrackRemoteCall())
    cross_req_options.cross_req = cross_req_head

    if not btr.cross_req_created:
        cross_req = Instance()
        SO.get_or_create_cross_req(pointer(btr.transport_sdk.transaction), pointer(cross_req_options), pointer(cross_req))
        btr.cross_req_created = True

    cross_resp = CrossReqHead()
    SO.gen_cross_resp_head(pointer(btr.transport_sdk.transaction), pointer(cross_resp))
    btr.cross_resp = cross_resp.data
    btr.agent.agent_id = config.getAgentId()


class CrossServer(object):
    __slots__ = ('cross_header', 'trace_id', 'rpc_id', '_current_rpc_id')
    def __init__(self):
        self.cross_header = None
        self.trace_id = None
        self.rpc_id = None
        self._current_rpc_id = None

    def load_trace_header(self, btr):
        if self.cross_header:
            self.trace_id = self.cross_header.get(TRACE_ID)
            self.rpc_id = self.cross_header.get(RPC_ID)
            self._current_rpc_id = 1
        else:
            # self.trace_id will be set when `TransportTrace` start_trace_call
            # self.trace_id = generateTraceId(btr.agent, btr)
            self.rpc_id = '0'
            self._current_rpc_id = 1

    def get_cross_data(self):
        mydic = {}
        if self.cross_header:
            if BACKEND_TYPE in self.cross_header:
                mydic['backendType'] = int(self.cross_header.get(BACKEND_TYPE))
            if FROM_AGENT_ID in self.cross_header:
                mydic['FromAgentID'] = int(self.cross_header.get(FROM_AGENT_ID))
            if IS_ASYNC in self.cross_header:
                mydic['isAsync'] = False if int(self.cross_header.get(IS_ASYNC)) == 0 else True

        return mydic

    def get_parent_data(self):
        mydic = {}
        if self.cross_header:
            parentBT = self.cross_header.get(FROM_TRANSACTION_ID, None)
            if parentBT:
                parentBT = str(parentBT).split('||')[-1]
                if str(parentBT).startswith('t'):
                    parentBT = str(parentBT).split(TRACE_SPLIT_TRANS_REGISTER)
                    mydic = {
                        'type': int(parentBT[0][2:]),
                        'internalName': parentBT[1][2:],
                        'name': parentBT[1][2:]
                    }
                else:
                    mydic = {
                        'id': int(parentBT)
                }
        return mydic

    def get_cross_header_need_sample(self, btr):
        if self.rpc_id == '0':
            need_sample = 1 if btr.traceshot_svc else 0
        else:
            need_sample = self.cross_header.get(NEED_SAMPLE, 0)
        return need_sample

    def get_cross_header_rpc_id(self):
        rpc_id = '{0}.{1}'.format(self.rpc_id, self._current_rpc_id)
        self._current_rpc_id += 1
        return rpc_id

    def get_cross_header_transaction_id(self, name, bt_type):
        hdr = {}
        if self.cross_header:
            hdr[FROM_TRANSACTION_ID] = self.cross_header.get(FROM_TRANSACTION_ID)
        if FROM_TRANSACTION_ID in hdr:
            hdr[FROM_TRANSACTION_ID] = '{0}{1}'.format(hdr[FROM_TRANSACTION_ID], TRACE_SPLIT_TRANS)
        if hdr.get(FROM_TRANSACTION_ID):
            hdr[FROM_TRANSACTION_ID] = '{0}{1}'.format(hdr[FROM_TRANSACTION_ID],
                                                       't={0}{1}n={2}'.format(bt_type, TRACE_SPLIT_TRANS_REGISTER,
                                                                              name))
        else:
            hdr[FROM_TRANSACTION_ID] = 't={0}{1}n={2}'.format(bt_type, TRACE_SPLIT_TRANS_REGISTER, name)
        return hdr[FROM_TRANSACTION_ID]

    def get_cross_header_parent_bt(self):
        if not self.cross_header:
            return None
        else:
            parentBT = self.cross_header.get(FROM_TRANSACTION_ID, None)
            if parentBT:
                parentBT = str(parentBT).split('||')[-1]
                if str(parentBT).startswith('t'):
                    parentBT = str(parentBT).split(TRACE_SPLIT_TRANS_REGISTER)
                    return {
                        'type': int(parentBT[0][2:]),
                        'name': parentBT[1][2:]
                    }
                else:
                    return {
                        'id': int(parentBT)
                    }


def make_cross_server(btr, request):
    cross_svc = CrossServer()
    headers = getattr(request, 'headers', None)
    if headers:
        cross_svc.cross_header = parse_cross_request_header(headers)
        if cross_svc.cross_header:
            create_cross_resp(btr, False)
        else:
            create_cross_resp(btr, True)
    else:
        create_cross_resp(btr, True)

    cross_svc.load_trace_header(btr)
    return cross_svc