#https://docs.python.org/2/library/ctypes.html
from ctypes import *
from Bonree.lang import bytes, PY3

class Str(Structure):
    _fields_ = [("value", c_char_p), ("length", c_uint)]
    def with_value(self, v = "",):
        if v is None or v == "" or v == 'None':
            self.value = b''
            self.length = 0
            return self

        if not isinstance(v, str):
            v = str(v)
        self.value = bytes(v)
        self.length = len(self.value)
        return self

def str_cache(v):
    return Str().with_value(v)

if PY3:
    from functools import lru_cache
    str_cache = lru_cache(maxsize=2048)(str_cache)

class SdkLogCfg(Structure):
    _fields_ = [("level", c_int), ("log_dir", Str)]
    def __init__(self):
        self.level = 0
        self.log_dir = Str()

class ContainerInfo(Structure):
    _fields_ = [("agent_version", Str), ("agent_install_timestamp", c_ulonglong), ("agent_install_dir", Str), ("listens", Str), ("container_version", Str), ("container_install_dir", Str), ("vm_info", Str),("vm_startup_options", Str), ("vm_system_options", Str)]
    def __init__(self):
        self.agent_version = Str()
        self.agent_install_timestamp = 0
        self.agent_install_dir = Str()
        self.listens = Str()
        self.container_version= Str()
        self.container_install_dir = Str()
        self.vm_info = Str()
        self.vm_startup_options = Str()
        self.vm_system_options = Str()

class SdkOptions(Structure):
    _fields_ = [("agent_type", c_int), ("pid", c_ulonglong), ("flags", c_int), ("log_cfg", SdkLogCfg), ("container_info", POINTER(ContainerInfo)), ("cfg_dir", Str)]
    def __init__(self):
        self.agent_type = 0
        self.pid = 0
        self.flags = 0
        self.log_cfg = SdkLogCfg()
        self.container_info = POINTER(ContainerInfo)()
        self.cfg_dir = Str()

class AgentState(Structure):
    _fields_ = [("state", c_int), ]

class BtNaming(Structure):
    _fields_ = [("type", c_int), ("enable", c_int),("properties", Str)]

class BTThresholdConfig(Structure):
    _fields_ = [("bt_threshold", c_uint), ("bt_threshold_slow", c_uint),("bt_threshold_very_slow", c_uint)]

class HotMethodcfg(Structure):
    _fields_ = [("class_name", Str), ("method_names", POINTER(Str)),("method_name_size", c_uint), ("enable_collect_params", POINTER(Str)), ("enable_collect_param_size", c_uint)]

class BtOptions(Structure):
    _fields_ = [("type", c_int), ("name", Str), ("internal_name", Str), ("uri", Str), ("start_time_stamp", c_ulonglong)]
    def __init__(self):
        self.type = 0
        self.name = Str()
        self.internal_name = Str()
        self.uri = Str()
        self.start_time_stamp = 0

class HttpReqData(Structure):
    _fields_ = [("url", Str), ("http_method", Str),("parameters", Str), ("req_headers", Str), ("req_content", Str)]
    def __init__(self):
        self.url = Str()
        self.http_method = Str()
        self.parameters = Str()
        self.req_headers = Str()
        self.req_content = Str()

class HttpRespData(Structure):
    _fields_ = [("status_code", Str), ("res_headers", Str), ("resp_content", Str)]
    def __init__(self):
        self.status_code = Str()
        self.res_headers = Str()
        self.resp_content = Str()

class BtError(Structure):
    _fields_ = [("time_stamp", c_ulonglong),("error_type", c_int), ("error_name", Str), ("error_type_str", Str), ("error_sumary", Str),("error_details", Str)]
    def __init__(self):
        self.time_stamp = 0
        self.error_type = 0
        self.error_name = Str()
        self.error_type_str = Str()
        self.error_sumary = Str()
        self.error_details = Str()

class FinishBtOptions(Structure):
    _fields_ = [("flags", c_uint), ("finish_time_stamp", c_ulonglong)]
    def __init__(self):
        self.flags = 0
        self.finish_time_stamp = 0

class ClassMethod(Structure):
    _fields_ = [("class_name", Str), ("method_name", Str)]
    def __init__(self):
        self.class_name = Str()
        self.method_name = Str()

class BeCallDetail(Structure):
    _fields_ = [("is_async", c_int), ("detail", Str),]
    def __init__(self):
        self.is_async = 0
        self.detail = Str()

class Be(Structure):
    _fields_ = [("type", c_int),("conn_type", Str), ("name", Str), ("host", Str), ("port", c_uint),("url", Str),("db_vendor", Str),("db_version", Str),("db_schema", Str)]
    def __init__(self):
        self.type = 0
        self.conn_type = Str()
        self.name = Str()
        self.host = Str()
        self.port = 0
        self.url = Str()
        self.db_vendor = Str()
        self.db_version = Str()
        self.db_schema = Str()

class BeCallOptions(Structure):
    _fields_ = [("be", Be), ("call_detail", BeCallDetail), ("class_method", ClassMethod), ("start_time_stamp", c_ulonglong)]
    def __init__(self):
        self.be = Be()
        self.call_detail = BeCallDetail()
        self.class_method = ClassMethod()
        self.start_time_stamp = 0

class FinishBeCallOptions(Structure):
    _fields_ = [("flags", c_int), ("finish_time_stamp", c_ulonglong), ("error_name",Str), ("error_detail",Str), ("sql_explain_plan",Str)]
    def __init__(self):
        self.flags = 0
        self.finish_time_stamp = 0
        self.error_detail = Str()
        self.error_name = Str()
        self.sql_explain_plan = Str()

class MQInfo(Structure):
    _fields_ = [("mq_size", c_ulonglong), ("direction", c_uint)]
    def __init__(self):
        self.mq_size = 0
        self.direction = 0

MAX_CROSS_REQ_HEAD_SIZE = 1024
ArrayType = c_char * MAX_CROSS_REQ_HEAD_SIZE
class CrossReqHead(Structure):
    _fields_ = [("data", ArrayType),]

class TraceId(Structure):
    _fields_ = [("data", ArrayType),]

class CrossReqOptions(Structure):
    _fields_ = [("is_sampling", c_int), ("cross_req", CrossReqHead)]
    def __init__(self):
        self.is_sampling = 0
        self.cross_req = CrossReqHead()

class SpanThreadInfo(Structure):
    _fields_ = [("thread_name", Str), ("thread_id", c_ulonglong),]
    def __init__(self):
        self.thread_name = Str()
        self.thread_id = 0

class SpanMethodInfo(Structure):
    _fields_ = [("class_name", Str), ("method_name", Str), ("ret_value", Str), ("params", Str), ("line_number", c_uint)]
    def __init__(self):
        self.class_name = Str()
        self.method_name = Str()
        self.ret_value = Str()
        self.params = Str()
        self.line_number = 0

class StartSpanOptions(Structure):
    _fields_ = [("thread_info", SpanThreadInfo), ("method_info", SpanMethodInfo), ("start_time_stamp", c_ulonglong)]
    def __init__(self):
        self.thread_info = SpanThreadInfo()
        self.method_info = SpanMethodInfo()
        self.start_time_stamp = 0

class FinishSpanOptions(Structure):
    _fields_ = [("finish_time_stamp", c_ulonglong),]
    def __init__(self):
        self.finish_time_stamp = 0

class TraceError(Structure):
    _fields_ = [("error_summary", Str), ("error_details", Str),]
    def __init__(self):
        self.error_summary = Str()
        self.error_details = Str()

class TraceLog(Structure):
    _fields_ = [("level", Str), ("log_msg", Str),]
    def __init__(self):
        self.level = Str()
        self.log_msg = Str()

class Instance(Structure):
    _fields_ = [("ins", c_void_p),]