import sys

from types import FunctionType

from Bonree.agent.models.methods import HOOK_FLAG
from Bonree.lang import is_async_coroutine
from Bonree.agent import get_agent_instance
from Bonree.agent.interceptor.base import CustomBtInterceptor, CustomTraceInterceptor


def create_cls(func_name, func):
    frame = sys._getframe(2)

    co = frame.f_code
    cls_or_module_name = co.co_name
    if cls_or_module_name == '<module>':
        cls_or_module_name = func.__module__

    cls = type(cls_or_module_name, (object, ), {})
    return cls


class CustomBtEntry(object):
    def __init__(self, name=None):
        self.name = name

    def __call__(self, func):
        if not isinstance(func, FunctionType):
            return func

        # check hooked
        if hasattr(func, HOOK_FLAG):
            return func

        agent = get_agent_instance()
        if not agent.enabled:
            return func

        func_name = func.__name__
        cls = create_cls(func_name, func)

        if is_async_coroutine(func):
            from Bonree.agent.interceptor.async_base import AsyncCustomBtInterceptor
            intercept_cls = AsyncCustomBtInterceptor
        else:
            intercept_cls = CustomBtInterceptor

        bt_name = self.name if self.name else func_name
        patch_func = intercept_cls(agent, cls).set_bt_name(bt_name).create_wrapper(func)
        return patch_func


def profiler_enter(func=None, name=None):
    if isinstance(func, FunctionType):
        return CustomBtEntry(name)(func)

    return CustomBtEntry(name)


def profiler_trace(func=None):
    if func is None:
        return profiler_trace

    if not isinstance(func, FunctionType):
        return func

    # check hooked
    if hasattr(func, HOOK_FLAG):
        return func

    agent = get_agent_instance()
    if not agent.enabled:
        return func

    func_name = func.__name__
    cls = create_cls(func_name, func)
    if is_async_coroutine(func):
        from Bonree.agent.interceptor.async_base import AsyncCustomTraceInterceptor
        intercept_cls = AsyncCustomTraceInterceptor
    else:
        intercept_cls = CustomTraceInterceptor
    patch_func = intercept_cls(agent, cls).create_wrapper(func)
    return patch_func
