# Copyright (c) Bonree, Inc., and its affiliates
# 2015
# All Rights Reserved

"""Trick Python into loading the agent.

"""
import os
import sys
import locale
import platform

def platform_can_inject():
    """
    The probe is not running in the following cases
    - Python version lower than 2.6 and higher than 3.9
    - Python's implementation is not Cpython
    - Python has not `cytpes` module
    """
    version = (sys.version_info[0], sys.version_info[1])
    if version < (2, 6) or version > (3, 9):
        return False
    if version >= (3, 0) and version < (3, 4):
        return False
    if platform.python_implementation() != 'CPython':
        return False

    try:
        # The probe requires `ctypes` module
        import ctypes
    except ImportError:
        return False

    return True


def is_pip_command():
    """
    In python < 3.7, it will raise `AttributeError`
        Error in sitecustomize; set PYTHONVERBOSE for traceback:
        AttributeError: module 'sys' has no attribute 'argv'
    so use `getattr(sys, 'argv', [])` get argv
    """
    argv = getattr(sys, 'argv', [])
    if len(argv) >= 2:  # pip <command> or setup.py install
        script_name = os.path.basename(argv[0])
        if script_name in ('pip', 'pip2', 'pip3'):
            return True
        if script_name == 'setup.py' and (argv[1] == 'install' or argv[1] == 'build'):
            return True

    return False


def inject():
    if platform.system().lower() == 'windows':
        try:
            PY3 = sys.version_info[0] > 2
            if PY3:
                locale.setlocale(locale.LC_ALL, 'English')
                locale.setlocale(locale.LC_CTYPE, 'chinese')
            else:
                reload(sys)
                sys.setdefaultencoding(locale.getpreferredencoding())
        except ValueError:
            pass

    try:
        path = os.path.dirname(__file__)
        if path in sys.path:
            sys.path.remove(path)
        sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
    except ValueError:
        pass

    try:
        import Bonree.agent
        if Bonree.agent.configure():
            Bonree.agent.bootstrap()
    except Exception as ex:
        try:
            import syslog
            syslog.openlog("[bonree_python_agent startup error]", syslog.LOG_PID)
            syslog.syslog(str(ex))
        except Exception:
            # do nothing
            pass

if platform_can_inject() and not is_pip_command():
    inject()

try:
    # remove this `sitecustomize` module
    path = os.path.dirname(__file__)
    if path in sys.path:
        sys.path.remove(path)
    sys.modules.pop('sitecustomize', None)
    import sitecustomize
except ImportError as e:
    raise e
