"""This file is provide the compatibility between different version python.
The aim here is make Python 2 behave like Python 3. This means we write code in
modern Python 3 style, where everything is an iterator, everything is unicode etc.
"""
from __future__ import unicode_literals
import itertools
import os
import sys

PY3 = sys.version_info[0] > 2

if PY3:
    # Imports
    import queue
    from http.cookies import SimpleCookie
    try:
        from importlib import reload
    except ImportError:
        from imp import reload

    from urllib.parse import parse_qs, parse_qsl, urlparse
    import builtins as __builtin__
    from urllib.parse import parse_qs, parse_qsl
    from urllib.parse import urlparse as parse_url
    from http.cookies import SimpleCookie, Morsel
    import asyncio


    if sys.version_info[1] > 6:
        import importlib.util as imp
    else:
        import imp

    b = lambda x: x.encode('ISO-8859-1')
    u = lambda x: x
    b2u = lambda x: x.decode('ISO-8859-1')
    u2b = lambda x: x.encode('ISO-8859-1')

    range = range

    iterkeys = lambda m: iter(m.keys())
    itervalues = lambda m: iter(m.values())
    iteritems = lambda m: iter(m.items())

    keys = lambda d: list(d.keys())
    values = lambda d: d.values()
    items = lambda d: d.items()

    long_t = int
    str_t = str
    bytes_t = bytes
    string_t = str

    # Functions
    filter = filter
    getcwd = os.getcwd
    long = int
    map = map
    range = range
    zip = zip

    def is_async_coroutine(func):
        return asyncio.iscoroutinefunction(func)

else:
    # Imports
    import imp
    import Queue as queue
    from Cookie import SimpleCookie
    from __builtin__ import reload
    from urlparse import parse_qs, parse_qsl, urlparse
    import codecs
    import __builtin__
    from urlparse import parse_qs, parse_qsl
    from urlparse import urlparse as parse_url
    from Cookie import SimpleCookie, Morsel

    b = lambda x: x
    u = lambda x: codecs.unicode_escape_decode(x)[0]
    b2u = lambda x: u(x)
    u2b = lambda x: x.encode('ISO-8859-1')

    # Types
    long_t = long
    str_t = unicode
    bytes_t = bytes
    string_t = basestring

    iteritems = lambda m: m.iteritems()
    itervalues = lambda m: m.itervalues()
    iterkeys = lambda m: m.iterkeys()

    items = lambda d: d.iteritems()
    values = lambda d: d.itervalues()
    keys = lambda d: d.keys()

    # Functions
    filter = lambda f, it: itertools.ifilter(f, it)
    getcwd = os.getcwdu
    long = long
    map = lambda f, *it: itertools.imap(f, *it)
    range = xrange
    zip = lambda *it: itertools.izip(*it)

    def is_async_coroutine(func):
        return False


def bytes(s, encoding='utf-8'):
    # Encode all bytes strings as utf-8.
    if isinstance(s, bytes_t):
        return s
    else:
        # This None thing isn't ideal, but since we use this function mainly
        # for making byte strings for protobufs, we want None to stay None.
        return None if s is None else s.encode(encoding)


def str(s, encoding='utf-8'):
    # Assume all bytes strings are utf-8 encoded.

    if isinstance(s, (bytes_t, bytearray)):
        try:
            return s.decode(encoding)
        except UnicodeDecodeError:
            has_int = False
            for x in s:
                if isinstance(x, int):
                   has_int = True
            if has_int:
                return ''.join(hex(x) for x in s)
            else:
                return ''.join(hex(ord(x)) for x in s)

    else:
        return str_t(s)
