import unittest
import memcache
import Bonree
from Bonree.agent.interceptor.cache import memcache as agent_memcache
from Bonree.agent import Agent
from Bonree.agent.models.exitcalls import ExitCall, Backend
from Bonree.agent.models import transactions
from config import MEMCACHE_HOST, MEMCACHE_PORT


class TestMemcache(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.agent = Agent()
        cls.agent.start()
        cls.agent.enable = True
        Bonree.agent.bootstrap()
        cls.btr = cls.agent.start_transaction_request(transactions.ENTRY_PYTHON_WSGI, name='test_memcache')
        cls.con = memcache.Client(['{0}:{1}'.format(MEMCACHE_HOST, MEMCACHE_PORT)])
        cls.con.set('hello', 'agent')
        cls.btr.status_code = 0
        cls.agent.end_transaction_request(cls.btr)

    def test_memcache(self):
        self.assertIsInstance(self.btr, transactions.TransactionReq)
        self.assertEqual(self.btr.name, 'test_memcache')
        self.assertEqual(self.btr.entry_type, 201)
        self.assertIsNot(self.btr.request_id, None)

    def test_memcache_Backend(self):
        self.assertIsInstance(self.btr.exitcalls[0].backend, Backend)
        self.assertEqual(self.btr.exitcalls[0].backend.connType, 'memcache')
        self.assertEqual(self.btr.exitcalls[0].backend.name, '{0}:{1} - MEMCACHED'.format(MEMCACHE_HOST, MEMCACHE_PORT))
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['SERVER POOL'], '{0}:{1}'.format(MEMCACHE_HOST, MEMCACHE_PORT))
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['VENDOR'], 'MEMCACHED')

    def test_memcache_Exitcall(self):
        self.assertIsInstance(self.btr.exitcalls[0], ExitCall)
        self.assertEqual(self.btr.exitcalls[0].operation, "set('hello', 'agent')")
        self.assertEqual(self.btr.exitcalls[0].summary, 'set')

    @classmethod
    def tearDownClass(cls):
        cls.con = memcache.Client(['{0}:{1}'.format(MEMCACHE_HOST, MEMCACHE_PORT)])
        cls.con.delete('hello')
