import unittest
import redis
import Bonree
from Bonree.agent.interceptor.cache import redis as agent_redis
from Bonree.agent import Agent
from Bonree.agent.models.exitcalls import ExitCall, Backend
from Bonree.agent.models import transactions
from config import REDIS_HOST, REDIS_PORT


class TestRedis(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.agent = Agent()
        cls.agent.start()
        cls.agent.enable = True
        Bonree.agent.bootstrap()
        cls.btr = cls.agent.start_transaction_request(transactions.ENTRY_PYTHON_WSGI, name='test_redis')
        cls.con = redis.StrictRedis(host=REDIS_HOST, port=REDIS_PORT)
        cls.con.set('hello', 'agent')
        cls.btr.status_code = 0
        cls.agent.end_transaction_request(cls.btr)

    def test_redis(self):
        self.assertIsInstance(self.btr, transactions.TransactionReq)
        self.assertEqual(self.btr.name, 'test_redis')
        self.assertEqual(self.btr.entry_type, 201)
        self.assertIsNotNone(self.btr.request_id)

    def test_redis_Backend(self):
        self.assertIsInstance(self.btr.exitcalls[0].backend, Backend)
        self.assertEqual(self.btr.exitcalls[0].backend.connType, 'redis')
        url = 'REDIS://{0}:{1}'.format(REDIS_HOST, REDIS_PORT)
        self.assertEqual(self.btr.exitcalls[0].backend.name, url)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['HOST'], REDIS_HOST)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['PORT'], str(REDIS_PORT))
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['VENDOR'], 'REDIS')

    def test_redis_Exitcall(self):
        self.assertIsInstance(self.btr.exitcalls[0], ExitCall)
        self.assertEqual(self.btr.exitcalls[0].operation, 'SET hello agent')
        self.assertEqual(self.btr.exitcalls[0].summary, 'set')

    @classmethod
    def tearDownClass(cls):
        cls.con = redis.StrictRedis(host=REDIS_HOST, port=REDIS_PORT)
        cls.con.delete('hello')
