import Bonree
from Bonree.agent.core.agent import Agent
from Bonree.agent.interceptor import http as agent_http
from Bonree.agent.models.exitcalls import ExitCall, Backend
from Bonree.agent.models import transactions
import httplib
import unittest


class TestHttp(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.agent = Agent()
        cls.agent.start()
        cls.agent.enable = True
        Bonree.agent.bootstrap()
        cls.btr = cls.agent.start_transaction_request(transactions.ENTRY_PYTHON_WSGI, name='test_http')
        cls.conn = httplib.HTTPConnection('www.baidu.com')
        cls.conn.request('GET', '/', '', {'user-agent':'test'})
        cls.btr.status_code = 0
        cls.agent.end_transaction_request(cls.btr)

    def test_http(self):
        self.assertIsInstance(self.btr, transactions.TransactionReq)
        self.assertEqual(self.btr.name, 'test_http')
        self.assertEqual(self.btr.entry_type, 201)
        self.assertIsNotNone(self.btr.request_id)

    def test_httpExitcall(self):
        self.assertIsInstance(self.btr.exitcalls[0], ExitCall)
        self.assertEqual(self.btr.exitcalls[0].operation, '/')
        self.assertEqual(self.btr.exitcalls[0].summary, '/')

    def test_httpBackend(self):
        self.assertIsInstance(self.btr.exitcalls[0].backend, Backend)
        self.assertEqual(self.btr.exitcalls[0].backend.connType, 'http')
        self.assertEqual(self.btr.exitcalls[0].backend.name, 'http://www.baidu.com:80')
        self.assertEqual(self.btr.exitcalls[0].backend.tipe, 201)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['HOST'], 'www.baidu.com')
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['PORT'], '80')
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['QUERY STRING'], '')
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['URL'], '/')

    @classmethod
    def tearDownClass(cls):
        cls.conn.close()
