import Bonree
from Bonree.agent.core.agent import Agent
from Bonree.agent.models.exitcalls import ExitCall, Backend
from Bonree.agent.models import transactions
import unittest
from tornado.httpclient import HTTPClient, AsyncHTTPClient


class TestTornadoHttpClient(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.agent = Agent()
        cls.agent.start()
        cls.agent.enable = True
        Bonree.agent.bootstrap()
        cls.btr = cls.agent.start_transaction_request(transactions.ENTRY_PYTHON_WSGI, name='test_tornado_httpclient')
        cls.http_client = HTTPClient()
        cls.http_client.fetch("http://www.baidu.com")
        cls.async_httpclient = AsyncHTTPClient()
        cls.async_httpclient.fetch("http://www.sina.com.cn")
        cls.btr.status_code = 0
        cls.agent.end_transaction_request(cls.btr)

    def test_tornado_httpclient(self):
        self.assertIsInstance(self.btr, transactions.TransactionReq)
        self.assertEqual(self.btr.name, 'test_tornado_httpclient')
        self.assertEqual(self.btr.entry_type, 201)
        self.assertIsNotNone(self.btr.request_id)

    def test_tornado_httpclientExitcall(self):
        self.assertIsInstance(self.btr.exitcalls[0], ExitCall)
        self.assertEqual(self.btr.exitcalls[0].operation, '')
        self.assertEqual(self.btr.exitcalls[0].summary, '/')

    def test_tornado_httpclientBackend(self):
        self.assertIsInstance(self.btr.exitcalls[0].backend, Backend)
        self.assertEqual(self.btr.exitcalls[0].backend.connType, 'http')
        self.assertEqual(self.btr.exitcalls[0].backend.name, 'http://www.baidu.com:80')
        self.assertEqual(self.btr.exitcalls[0].backend.tipe, 201)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['HOST'], 'www.baidu.com')
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['PORT'], '80')
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['QUERY STRING'], '')
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['URL'], '')

    @classmethod
    def tearDownClass(cls):
        cls.http_client.close()
        cls.async_httpclient.close()
