import unittest
import pymongo as mongo
import Bonree
from Bonree.agent.models import transactions
from Bonree.agent.models.transactions import TransactionReq
from Bonree.agent.interceptor.mongodb import pymongo
from Bonree.agent import Agent
from Bonree.agent.models.exitcalls import ExitCall, Backend
from config import MONGO_HOST, MONGODB_PORT, MONGO_DB


def drop_data():
    connection = mongo.MongoClient(MONGO_HOST, MONGODB_PORT)
    db = connection['{0}'.format(MONGO_DB)]
    db_info = db.test
    db_info.drop()


class TestPymongo(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.agent = Agent()
        cls.agent.start()
        cls.agent.enable = True
        Bonree.agent.bootstrap()
        cls.btr = cls.agent.start_transaction_request(transactions.ENTRY_PYTHON_WSGI, name='test_pymongo')
        cls.connection = mongo.MongoClient(MONGO_HOST, MONGODB_PORT)
        testdb = cls.connection['{0}'.format(MONGO_DB)]
        test_info = testdb.test
        if test_info.find_one({"name": "God"}) is None:
            test_info.insert_one({'name': 'God', 'age': 0, 'skill': 'Create'})
        cls.btr.status_code = 0
        cls.agent.end_transaction_request(cls.btr)

    def test_pymongo(self):
        self.assertIsInstance(self.btr, TransactionReq)
        self.assertEqual(self.btr.name, 'test_pymongo')
        self.assertEqual(self.btr.status_code, 0)
        self.assertEqual(self.btr.entry_type, 201)

    def test_pymongo_Exitcall(self):
        self.assertEqual(self.btr.exitcalls[0].summary, 'find')
        self.assertIsInstance(self.btr.exitcalls[0], ExitCall)

    def test_pymongo_Backend(self):
        self.assertIsInstance(self.btr.exitcalls[0].backend, Backend)
        self.assertEqual(self.btr.exitcalls[0].backend.connType, 'mongodb')
        url = 'MONGODB://{0}:{1}/{2}'.format(MONGO_HOST, MONGODB_PORT, MONGO_DB)
        self.assertEqual(self.btr.exitcalls[0].backend.name, url)
        self.assertEqual(self.btr.exitcalls[0].backend.tipe, 101)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['HOST'], MONGO_HOST)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['PORT'], str(MONGODB_PORT))
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['DATABASE'], MONGO_DB)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['VENDOR'], 'MONGODB')

    @classmethod
    def tearDownClass(cls):
        cls.connection.close()
        drop_data()

