# -*- coding:utf-8 -*-
import unittest
import Bonree
import mysql.connector
from Bonree.agent.interceptor.sql import mysql_connector
from Bonree.agent import Agent
from Bonree.agent.models.exitcalls import ExitCall, Backend
from Bonree.agent.models import transactions
from config import MYSQL_HOST, MYSQL_PORT, MYSQL_DB, MYSQL_PASSWD, MYSQL_USERNAME
from mysql.connector.errors import DatabaseError


def create_table():
    con = mysql.connector.connect(host=MYSQL_HOST, user=MYSQL_USERNAME, password=MYSQL_PASSWD)
    cur = con.cursor()
    try:
        cur.execute("CREATE database {0}".format(MYSQL_DB))
    except DatabaseError:
        cur.execute('drop database {0}'.format(MYSQL_DB))
        cur.execute("CREATE database {0}".format(MYSQL_DB))
    cur.execute('use {0}'.format(MYSQL_DB))
    cur.close()
    con.close()
    con = mysql.connector.connect(host=MYSQL_HOST, user=MYSQL_USERNAME, password=MYSQL_PASSWD, database=MYSQL_DB)
    cur = con.cursor()
    cur.execute("DROP TABLE IF EXISTS `mete`;")
    cur.execute("DROP TABLE IF EXISTS `cat`;")
    cur.execute("DROP TABLE IF EXISTS `users`;")
    create_cat_sql = "CREATE TABLE `cat` (" \
                     "`tid` int(11) NOT NULL AUTO_INCREMENT," \
                     "`value` varchar(20) NOT NULL," \
                     "PRIMARY KEY (`tid`)) ENGINE=InnoDB AUTO_INCREMENT=3 DEFAULT CHARSET=utf8;"
    create_mete_sql = "CREATE TABLE `mete` (" \
                      "`cid` int(11) NOT NULL," \
                      "`uid` int(11) NOT NULL AUTO_INCREMENT," \
                      "PRIMARY KEY (`uid`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;"
    create_user_sql = "CREATE TABLE `users` (" \
                      "`uid` int(11) NOT NULL AUTO_INCREMENT," \
                      "`value` varchar(255) DEFAULT NULL," \
                      "PRIMARY KEY (`uid`)) ENGINE=InnoDB AUTO_INCREMENT=5 DEFAULT CHARSET=utf8;"
    cur.execute(create_cat_sql)
    cur.execute(create_mete_sql)
    cur.execute(create_user_sql)
    cur.close()
    con.close()


def create_data():
    con = mysql.connector.connect(host=MYSQL_HOST, user=MYSQL_USERNAME, password=MYSQL_PASSWD, database=MYSQL_DB)
    cur = con.cursor()
    insert_sql_list = ["INSERT INTO `users` VALUES ('1', 'Tom');", "INSERT INTO `users` VALUES ('2', 'Bule');",
                       "INSERT INTO `users` VALUES ('3', 'Jack');", "INSERT INTO `users` VALUES ('4', 'Jerry');",
                       "INSERT INTO `cat` VALUES ('1', '男');", "INSERT INTO `cat` VALUES ('2', '女');",
                       "INSERT INTO `mete` VALUES ('1', '1');", "INSERT INTO `mete` VALUES ('2', '2');",
                       "INSERT INTO `mete` VALUES ('2', '3');", "INSERT INTO `mete` VALUES ('1', '4');"]
    for item in insert_sql_list:
        cur.execute(item)
    con.commit()
    cur.close()
    con.close()


def drop_data():
    con = mysql.connector.connect(host=MYSQL_HOST, user=MYSQL_USERNAME, password=MYSQL_PASSWD)
    cur = con.cursor()
    cur.execute('use {0}'.format(MYSQL_DB))
    cur.execute('drop table cat')
    cur.execute('drop table mete')
    cur.execute('drop table users')
    cur.execute('drop database {0}'.format(MYSQL_DB))
    con.commit()
    cur.close()
    con.close()


class TestMysqlConnector(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        create_table()
        cls.agent = Agent()
        cls.agent.start()
        cls.agent.enable = True
        Bonree.agent.bootstrap()
        cls.btr = cls.agent.start_transaction_request(transactions.ENTRY_PYTHON_WSGI, name='test_mysql_connector')
        cls.con = mysql.connector.connect(host=MYSQL_HOST, user=MYSQL_USERNAME, password=MYSQL_PASSWD, database=MYSQL_DB)
        cls.cur = cls.con.cursor(buffered=True)
        cls.version = '.'.join(str(x) for x in cls.con._server_version)
        create_data()
        cls.cur.execute('select users.value from users where users.uid in (select mete.uid from mete where mete.cid=1)', multi=True)
        cls.cur.execute('select U.value,C.value from mete as M join cat as C on C.tid=M.cid join users as U on U.uid=M.uid', multi=True)
        cls.cur.execute('select * from mete as M left join cat as C on C.tid=M.cid left join users as U on U.uid=M.uid ', multi=True)
        cls.btr.status_code = 0
        cls.agent.end_transaction_request(cls.btr)

    def test_mysqlconnector(self):
        self.assertIsInstance(self.btr, transactions.TransactionReq)
        self.assertEqual(self.btr.name, 'test_mysql_connector')
        self.assertEqual(self.btr.entry_type, 201)
        self.assertIsNot(self.btr.request_id, None)

    def test_mysqlconnector_Backend(self):
        self.assertIsInstance(self.btr.exitcalls[0].backend, Backend)
        self.assertEqual(self.btr.exitcalls[0].backend.connType, 'mysql')
        url = 'mysql://{0}:{1}/{2}'.format(MYSQL_HOST, MYSQL_PORT, MYSQL_DB)
        self.assertEqual(self.btr.exitcalls[0].backend.name, url)
        self.assertEqual(self.btr.exitcalls[0].backend.tipe, 1)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['DATABASE'], MYSQL_DB)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['HOST'], MYSQL_HOST)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['PORT'], str(MYSQL_PORT))
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['VENDOR'], 'mysql')
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['VERSION'], self.version)

    def test_mysqlconnector_Exitcall(self):
        self.assertIsInstance(self.btr.exitcalls[0], ExitCall)
        self.assertEqual(len(self.btr.exitcalls), 13)

    @classmethod
    def tearDownClass(cls):
        cls.cur.close()
        cls.con.close()
        drop_data()
