# -*- coding:utf-8 -*-
import unittest
import Bonree
from Bonree.agent.interceptor.sql import mysqldb
from Bonree.agent import Agent
from Bonree.agent.models.exitcalls import ExitCall, Backend
from Bonree.agent.models import transactions
from Bonree.agent.proto.pb_backend_pb2 import BACKEND_TYPE_MYSQL
from config import MYSQL_HOST, MYSQL_PORT, MYSQL_DB, MYSQL_PASSWD, MYSQL_USERNAME
import MySQLdb
from MySQLdb import connections


Result = []


def create_table():
    con = MySQLdb.connect(MYSQL_HOST, MYSQL_USERNAME, MYSQL_PASSWD)
    cur = con.cursor()
    try:
        cur.execute("CREATE database {0}".format(MYSQL_DB))
    except MySQLdb.ProgrammingError:
        cur.execute('drop database {0}'.format(MYSQL_DB))
        cur.execute("CREATE database {0}".format(MYSQL_DB))
    cur.execute('use {0}'.format(MYSQL_DB))
    create_employee_sql = "Create table If Not Exists Employee (Id int, Name varchar(255), Salary int, DepartmentId int);"
    create_department_sql = "Create table If Not Exists Department (Id int, Name varchar(255));"
    cur.execute(create_employee_sql)
    cur.execute(create_department_sql)
    con.commit()
    cur.close()
    con.close()


def create_data():
    con = MySQLdb.connect(MYSQL_HOST, MYSQL_USERNAME, MYSQL_PASSWD, MYSQL_DB)
    cur = con.cursor()
    cur.execute('Truncate table Employee;')
    cur.execute("insert into Employee (Id, Name, Salary, DepartmentId) values ('1', 'Joe', '70000', '1');")
    cur.execute("insert into Employee (Id, Name, Salary, DepartmentId) values ('2', 'Henry', '80000', '2');")
    cur.execute("insert into Employee (Id, Name, Salary, DepartmentId) values ('3', 'Sam', '60000', '2');")
    cur.execute("insert into Employee (Id, Name, Salary, DepartmentId) values ('4', 'Max', '90000', '1');")
    cur.execute("Truncate table Department;")
    cur.execute("insert into Department (Id, Name) values ('1', 'IT');")
    cur.execute("insert into Department (Id, Name) values ('2', 'Sales');")
    con.commit()
    cur.close()
    con.close()


def drop_data():
    con = MySQLdb.connect(MYSQL_HOST, MYSQL_USERNAME, MYSQL_PASSWD)
    cur = con.cursor()
    cur.execute('use {0}'.format(MYSQL_DB))
    try:
        cur.execute('drop table employee')
        cur.execute('drop table Department')
        cur.execute('drop database {0}'.format(MYSQL_DB))
        con.commit()
    except BaseException:
        pass
    cur.close()
    con.close()


class TestMysqldb(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        create_table()
        cls.agent = Agent()
        cls.agent.start()
        cls.agent.enable = True
        Bonree.agent.bootstrap()
        cls.btr = cls.agent.start_transaction_request(
            transactions.ENTRY_PYTHON_WSGI, name='test_mysqldb')
        cls.con = MySQLdb.connect(MYSQL_HOST, MYSQL_USERNAME, MYSQL_PASSWD, MYSQL_DB)
        cls.cur = cls.con.cursor()
        cls.version = '.'.join(str(x) for x in cls.con._server_version)
        create_data()
        cls.cur.execute('''SELECT d.Name AS Department, e.Name AS Employee, e.Salary FROM Employee e
                        JOIN Department d on e.DepartmentId = d.Id
                        WHERE (SELECT COUNT(DISTINCT Salary) FROM Employee WHERE Salary > e.Salary
                        AND DepartmentId = d.Id) < 3 ORDER BY d.Name, e.Salary DESC;''')
        Result.insert(0, cls.cur.fetchall())
        cls.btr.status_code = 0
        cls.agent.end_transaction_request(cls.btr)

    def test_mysqldb(self):
        self.assertIsInstance(self.btr, transactions.TransactionReq)
        self.assertEqual(self.btr.name, 'test_mysqldb')
        self.assertEqual(self.btr.entry_type, 201)
        self.assertIsNot(self.btr.request_id, None)

    def test_mysqldb_Backend(self):
        try:
            self.assertIsInstance(self.btr.exitcalls[0].backend, Backend)
            self.assertEqual(self.btr.exitcalls[0].backend.connType, 'mysql')
            url = 'mysql://{0}:{1}/{2}'.format(MYSQL_HOST, MYSQL_PORT, MYSQL_DB)
            self.assertEqual(self.btr.exitcalls[0].backend.name, url)
            self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['DATABASE'], MYSQL_DB)
            self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['HOST'], MYSQL_HOST)
            self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['PORT'], str(MYSQL_PORT))
            self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['VENDOR'], 'mysql')
            self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['VERSION'], self.version)
        except IndexError as e:
            print('Test Error: {0}'.format(e.message))

    def test_mysqldb_Exitcall(self):
        try:
            self.assertIsInstance(self.btr.exitcalls[0], ExitCall)
            self.assertEqual(len(self.btr.exitcalls), 9)
            self.assertIsNotNone(Result)
        except IndexError as e:
            print('Test Error: {0}'.format(e.message))

    @classmethod
    def tearDownClass(cls):
        cls.cur.close()
        cls.con.close()
        drop_data()
