# -*- coding:utf-8 -*-
import unittest
import pymysql
from pymysql.err import ProgrammingError
import Bonree
from Bonree.agent.interceptor.sql import pymysql as agent_pymysql
from Bonree.agent import Agent
from Bonree.agent.models.exitcalls import ExitCall, Backend
from Bonree.agent.models import transactions
from config import MYSQL_HOST, MYSQL_PORT, MYSQL_DB, MYSQL_PASSWD, MYSQL_USERNAME
from warnings import filterwarnings, resetwarnings
filterwarnings('ignore', category=pymysql.Warning)
resetwarnings()

Result = []


def create_table():
    con = pymysql.connect(MYSQL_HOST, MYSQL_USERNAME, MYSQL_USERNAME)
    con.autocommit = True
    cur = con.cursor()
    try:
        cur.execute("CREATE database {0}".format(MYSQL_DB))
    except ProgrammingError:
        cur.execute('drop database {0}'.format(MYSQL_DB))
        cur.execute("CREATE database {0}".format(MYSQL_DB))
    cur.execute('use {0}'.format(MYSQL_DB))
    cur.close()
    con.close()
    con = pymysql.connect(host=MYSQL_HOST, user=MYSQL_USERNAME, password=MYSQL_PASSWD, database=MYSQL_DB)
    cur = con.cursor()
    cur.execute("DROP TABLE IF Exists mete;")
    cur.execute("DROP TABLE IF Exists cat;")
    cur.execute("DROP TABLE IF Exists users;")
    create_cat_sql = "CREATE TABLE `cat` (" \
                     "`tid` int(11) NOT NULL AUTO_INCREMENT," \
                     "`value` varchar(20) NOT NULL," \
                     "PRIMARY KEY (`tid`)) AUTO_INCREMENT=3 DEFAULT CHARSET=utf8;"
    create_mete_sql = "CREATE TABLE `mete` (" \
                      "`cid` int(11) NOT NULL," \
                      "`uid` int(11) NOT NULL AUTO_INCREMENT," \
                      "PRIMARY KEY (`uid`)) DEFAULT CHARSET=utf8;"
    create_user_sql = "CREATE TABLE `users` (" \
                      "`uid` int(11) NOT NULL AUTO_INCREMENT," \
                      "`value` varchar(255) DEFAULT NULL," \
                      "PRIMARY KEY (`uid`)) AUTO_INCREMENT=5 DEFAULT CHARSET=utf8;"
    try:
        cur.execute(create_cat_sql)
        cur.execute(create_mete_sql)
        cur.execute(create_user_sql)
        con.commit()
    except Exception:
        con.rollback()
    cur.close()
    con.close()


def create_data():
    con = pymysql.connect(MYSQL_HOST, MYSQL_USERNAME, MYSQL_USERNAME, MYSQL_DB)
    cur = con.cursor()
    insert_sql_list = [
        "INSERT INTO `users` VALUES ('1', 'Tom');",
        "INSERT INTO `users` VALUES ('2', 'Bule');",
        "INSERT INTO `users` VALUES ('3', 'Jack');",
        "INSERT INTO `users` VALUES ('4', 'Jerry');",
        "INSERT INTO `cat` VALUES ('1', '男');",
        "INSERT INTO `cat` VALUES ('2', '女');",
        "INSERT INTO `mete` VALUES ('1', '1');",
        "INSERT INTO `mete` VALUES ('2', '2');",
        "INSERT INTO `mete` VALUES ('2', '3');",
        "INSERT INTO `mete` VALUES ('1', '4');"]
    for item in insert_sql_list:
        cur.execute(item)
    con.commit()
    cur.close()
    con.close()


def drop_data():
    con = pymysql.connect(MYSQL_HOST, MYSQL_USERNAME, MYSQL_USERNAME, MYSQL_DB)
    cur = con.cursor()
    cur.execute('drop table cat')
    cur.execute('drop table mete')
    cur.execute('drop table users')
    cur.execute('drop database {0}'.format(MYSQL_DB))
    con.commit()
    cur.close()
    con.close()


class TestPyMysql(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        create_table()
        cls.agent = Agent()
        cls.agent.start()
        cls.agent.enable = True
        Bonree.agent.bootstrap()
        cls.btr = cls.agent.start_transaction_request(transactions.ENTRY_PYTHON_WSGI, name='test_pymysql')
        cls.con = pymysql.connect(MYSQL_HOST, MYSQL_USERNAME, MYSQL_USERNAME, MYSQL_DB)
        cls.cur = cls.con.cursor()
        cls.version = cls.con.server_version
        create_data()
        cls.cur.execute('select users.value from users where users.uid in (select mete.uid from mete where mete.cid=1)')
        Result.insert(0, cls.cur.fetchall())
        cls.cur.execute(
            'select U.value,C.value from mete as M join cat as C on C.tid=M.cid join users as U on U.uid=M.uid')
        Result.insert(1, cls.cur.fetchall())
        cls.cur.execute(
            'select * from mete as M left join cat as C on C.tid=M.cid left join users as U on U.uid=M.uid ')
        Result.insert(2, cls.cur.fetchall())
        cls.btr.status_code = 0
        cls.agent.end_transaction_request(cls.btr)

    def test_pymysql(self):
        self.assertIsInstance(self.btr, transactions.TransactionReq)
        self.assertEqual(self.btr.name, 'test_pymysql')
        self.assertEqual(self.btr.entry_type, 201)
        self.assertIsNot(self.btr.request_id, None)

    def test_pymysql_Backend(self):
        self.assertIsInstance(self.btr.exitcalls[0].backend, Backend)
        self.assertEqual(self.btr.exitcalls[0].backend.connType, 'mysql')
        url = 'mysql://{0}:{1}/{2}'.format(MYSQL_HOST, MYSQL_PORT, MYSQL_DB)
        self.assertEqual(self.btr.exitcalls[0].backend.name, url)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['DATABASE'], MYSQL_DB)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['HOST'], MYSQL_HOST)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['PORT'], str(MYSQL_PORT))
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['VENDOR'], 'mysql')
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['VERSION'], self.version)

    def test_pymysql_Exitcall(self):
        self.assertIsInstance(self.btr.exitcalls[0], ExitCall)

    @classmethod
    def tearDownClass(cls):
        cls.cur.close()
        cls.con.close()
        drop_data()
