# -*- coding:utf-8 -*-
import unittest
import psycopg2
import Bonree
from Bonree.agent.interceptor.sql import psycopg2 as agent_psycopg
from Bonree.agent import Agent
from Bonree.agent.models.exitcalls import ExitCall, Backend
from Bonree.agent.models import transactions
from config import PostgreSQL_HOST, PostgreSQL_PASSWD, PostgreSQL_USER, PostgreSQL_PORT, PostgreSQL_DB

RESULT = []


def create_table():
    con = psycopg2.connect(user=PostgreSQL_USER, password=PostgreSQL_PASSWD, host=PostgreSQL_HOST)
    con.autocommit = True
    cur = con.cursor()
    # cur.execute("DROP DATABASE IF EXISTS {0}".format(DATABASE))
    # cur.execute("CREATE database {0}".format(DATABASE))
    cur.execute("DROP TABLE IF EXISTS mete;")
    cur.execute("DROP TABLE IF EXISTS cat;")
    cur.execute("DROP TABLE IF EXISTS users;")
    create_cat_sql = '''CREATE TABLE cat (
                     tid integer PRIMARY KEY ,
                     value varchar NOT NULL)'''
    create_mete_sql = '''CREATE TABLE public.mete (
                      cid integer NOT NULL,
                      uid integer PRIMARY KEY)'''
    create_user_sql = '''CREATE TABLE public.users (
                      uid integer PRIMARY KEY ,
                      value varchar DEFAULT NULL)'''
    cur.execute(create_cat_sql)
    cur.execute(create_mete_sql)
    cur.execute(create_user_sql)
    cur.close()
    con.close()


def create_data():
    con = psycopg2.connect(user=PostgreSQL_USER, password=PostgreSQL_PASSWD, host=PostgreSQL_HOST)
    cur = con.cursor()
    insert_sql_list = [
        "INSERT INTO users VALUES ('1', 'Tom');",
        "INSERT INTO users VALUES ('2', 'Bule');",
        "INSERT INTO users VALUES ('3', 'Jack');",
        "INSERT INTO users VALUES ('4', 'Jerry');",
        "INSERT INTO cat VALUES ('1', '男');",
        "INSERT INTO cat VALUES ('2', '女');",
        "INSERT INTO mete VALUES ('1', '1');",
        "INSERT INTO mete VALUES ('2', '2');",
        "INSERT INTO mete VALUES ('2', '3');",
        "INSERT INTO mete VALUES ('1', '4');"]
    for item in insert_sql_list:
        cur.execute(item)
    con.commit()
    cur.close()
    con.close()


def drop_data():
    con = psycopg2.connect(user=PostgreSQL_USER, password=PostgreSQL_PASSWD, host=PostgreSQL_HOST)
    con.autocommit = True
    cur = con.cursor()
    cur.execute("DROP TABLE IF EXISTS mete;")
    cur.execute("DROP TABLE IF EXISTS cat;")
    cur.execute("DROP TABLE IF EXISTS users;")
    cur.close()
    con.close()


class TestPyscopg2(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        create_table()
        cls.agent = Agent()
        cls.agent.start()
        cls.agent.enable = True
        Bonree.agent.bootstrap()
        cls.btr = cls.agent.start_transaction_request(
            transactions.ENTRY_PYTHON_WSGI, name='test_psycopg2')
        cls.con = psycopg2.connect(user=PostgreSQL_USER, password=PostgreSQL_PASSWD, host=PostgreSQL_HOST)
        cls.cur = cls.con.cursor()
        context = cls.con.server_version
        cls.version = '.'.join([str(context/10000%100), str(context/100%100), str(context%100)])
        create_data()
        cls.cur.execute('select users.value from users where users.uid in (select mete.uid from mete where mete.cid=1)')
        RESULT.insert(0, cls.cur.fetchall())
        cls.cur.execute(
            'select U.value,C.value from mete as M join cat as C on C.tid=M.cid join users as U on U.uid=M.uid')
        RESULT.insert(1, cls.cur.fetchall())
        cls.cur.execute(
            'select * from mete as M left join cat as C on C.tid=M.cid left join users as U on U.uid=M.uid ')
        RESULT.insert(2, cls.cur.fetchall())
        cls.btr.status_code = 0
        cls.agent.end_transaction_request(cls.btr)

    def test_Pyscopg2(self):
        self.assertIsInstance(self.btr, transactions.TransactionReq)
        self.assertEqual(self.btr.name, 'test_psycopg2')
        self.assertEqual(self.btr.entry_type, 201)
        self.assertIsNot(self.btr.request_id, None)

    def test_Pyscopg2_Backend(self):
        self.assertIsInstance(self.btr.exitcalls[0].backend, Backend)
        self.assertEqual(self.btr.exitcalls[0].backend.connType, 'postgresql')
        url = 'postgresql://{0}:{1}/{2}'.format(PostgreSQL_HOST, PostgreSQL_PORT, PostgreSQL_USER)
        self.assertEqual(self.btr.exitcalls[0].backend.name, url)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['DATABASE'], PostgreSQL_USER)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['HOST'], PostgreSQL_HOST)
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['PORT'], str(PostgreSQL_PORT))
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['VENDOR'], 'postgresql')
        self.assertEqual(self.btr.exitcalls[0].backend.identifying_properties['VERSION'], self.version)

    def test_Pyscopg2_Exitcall(self):
        self.assertIsInstance(self.btr.exitcalls[0], ExitCall)
        self.assertEqual(len(self.btr.exitcalls), 13)

    @classmethod
    def tearDownClass(cls):
        cls.cur.close()
        cls.con.close()
        drop_data()

